!-------------------------------------------------------------------------
!               MODULE FOR MONTE CARLO RADIATIVE TRANSFER
!
! NOTE: The spatial indexing is done in an unstructured way. Each cell
!       simply has two indentifier numbers. The ID number, called "icell",
!       is a number from 1 to the number of currently used cells. This ID
!       may change when cells are added/removed upon successive calls of the
!       Monte Carlo code. The second identifier number, called "index", is
!       used to specify the location in memory where the cell is. This
!       number can be anywhere between 1 and nrcellsmax, where
!       nrcellsmax.ge.nrcells. Each cell will retain this ID for as long
!       as it lives. Not all index numbers are used. In particular if a set
!       of grid points/cells are created, and then some are removed, then 
!       typically this produces "holes" in the memory, i.e. some memory
!       locations remain unused. This corresponds to come index number to
!       be unused. In general, if one wants to visit all cells, one should
!       make a loop do icell=1,nrcells, and at each icell one gets the
!       index through index=cellindex(icell), and one then uses index to
!       access data of this cell from the memory. There are a very few
!       exceptions to this rule, in which the data is indexed directly
!       via icell. But that is then noted in the code.
!       -------------------------------------------------------------------------
module montecarlo_module
use mathroutines_module
use rtglobal_module
use dust_module
use stars_module
use quantum_module
use amr_module
use amrray_module
use ioput_module
use constants_module
!
!-----------------------------------------------------------------------
!
! Current photon number
! (Must ensure huge range here)
!
integer(kind=8) :: iphotcurr,ieventcounttot
!
! Flag saying whether or not to interpolate the temperature emission
! database in temperature
!
integer :: intplt
!
! Some defines 
!
double precision, parameter :: minieps = 1d-12
!
! The energy of each photon package
!
double precision :: energy
!
! When using weighted photon packages, the energy of these packages
!
logical :: mc_weighted_photons=.true.
double precision :: mc_energy_extlum,mc_energy_stellarsrc
double precision :: mc_energy_qvisc,mc_energy_quant,mc_energy_thermal
double precision, allocatable :: mc_energy_stars(:)
!
! The overall averaged output spectrum (averaged over all directions)
!
double precision,allocatable :: mc_average_spectrum(:)
double precision,allocatable :: mc_integerspec(:)
!
! The partial energy for each dust species
!
double precision,allocatable :: mc_enerpart(:)
!
! The cumulative energy in each cell for each dust species
!
double precision,allocatable :: mc_cumulener(:,:),mc_cumulener_bk(:,:)
!
! The opacities used for this MC module
!
double precision,allocatable :: kappa_a(:,:),kappa_s(:,:),kappa_g(:,:)
double precision,allocatable :: alpha_a(:),alpha_s(:),alphacum(:)
double precision :: alpha_a_tot,alpha_s_tot
!
! For photon statistics 
! (Must ensure huge range here)
!
!!!!!double precision,allocatable :: mc_ilastphot(:),mc_iphotcount(:)
!!!!!double precision :: mc_iphotcurr
integer(kind=8),allocatable :: mc_ilastphot(:),mc_iphotcount(:)
integer(kind=8) :: mc_iphotcurr
! 
! For arrays of integrals as a function of temperature
!
double precision, allocatable :: db_temp(:),db_enertemp(:,:),db_cumul(:)
double precision, allocatable :: db_logenertemp(:,:)
double precision, allocatable :: db_emiss(:,:,:),db_cumulnorm(:,:,:)
integer :: db_ntemp
!
! For internal use in the routines that compute the above arrays
!
double precision, allocatable :: cellalpha(:),fnu_diff(:),diffemis(:)
double precision, allocatable :: enercum(:)
!
! For convenience for choosing which source of photons to choose
!
double precision :: mc_cumlum1,mc_cumlum2,mc_cumlum3,mc_cumlum4
!
! --------------------------------------------------------------
!
! The maximum absorption tau a photon package travels in a scattering
! Monte Carlo simulation before the package is dropped.
!
double precision :: mc_scat_maxtauabs
double precision :: mc_scat_energy_rellimit
!
! For some Monte Carlo calculation types we need a special local
! set of frequencies. This is, by the way, not true for the Bjorkman
! & Wood (thermal Monte Carlo) transfer. That will be always done on the 
! global frequency grid.
!
double precision, allocatable :: mc_frequencies(:)
integer :: mc_nrfreq=0
!
! The number of directions for which the scattering source function must
! be stored.
!
integer :: mcscat_nrdirs=0, mcscat_current_dir=1
double precision, allocatable :: mcscat_dirs(:,:)
!
! The storage array for the scattering source function at the frequencies
! stored in mc_frequencies. This includes the possibility for isotropic
! scattering (with mcscat_nrdirs=1)
!
real, allocatable :: mcscat_scatsrc(:,:,:)
!
! The storage array for the mean intensity function at the frequencies
! stored in mc_frequencies. 
!
real, allocatable :: mcscat_meanint(:,:)
!
! The array of the thermal emissivity
!
double precision, allocatable :: mc_cumulthermemis(:)
double precision :: mc_thermemistot
!
! For anisotropic scattering mode
!
double precision, allocatable :: mcscat_phasefunc(:,:)
!
! For the local observer mode
!
logical :: mcscat_localobserver=.false.
double precision :: mcscat_localobs_pos(1:3)
!
!    Arrays for templates of stellar source spectra
!
double precision, allocatable :: mc_stellarsrc_templates(:,:)
!
! Global flag for Monte Carlo
!
logical :: mc_photon_destroyed
!----TO-ADD----
!
! Array for skipping very optically thick cells
!
! double precision, allocatable :: mc_scat_src_skip(:)
!----TO-ADD----

contains

!--------------------------------------------------------------------------
!                    Initialize the Monte Carlo arrays
!
! This routine is automatically called by the do_monte_carlo_bjorkmanwood()
! routine. It allocates some more arrays and does a number of consistency
! checks.
!
! ARGUMENTS:
!  params             The parameter structure
!  ierr               Error code. 0 means no error.
!  mcaction           =1   --> Bjorkman & Wood
!                     =101 --> Scattering MC to compute the scattering source
!                     =102 --> Scattering MC to compute the mean intensity
!--------------------------------------------------------------------------
subroutine montecarlo_init(params,ierr,mcaction,resetseed)
  implicit none
  type(mc_params) :: params
  integer :: ierr,mcaction
  logical, optional :: resetseed
  !
  integer :: ispec,inu,iinu,itempl,istar
  double precision :: temp,eps,dum
  logical :: iwarned
  !
  ! First clean up any possible remaining crap from earlier runs
  !
  call montecarlo_partial_cleanup()
  !
  ! If requested, then reset the random seed to the original value. This can be useful
  ! if you want to make movies and you don't want to be annoyed by constantly changing
  ! noise for nearly identical frames.
  !
  if(present(resetseed)) then
     if(resetseed) then
        iseed = iseed_start
     endif
  endif
  !
  ! If we must store a scattering source function, we must have at least
  ! mcscat_nrdirs.ge.1. This is true for the scattering Monte Carlo, but
  ! also for the thermal Monte Carlo IFF the incl_scatsrc_mctherm is
  ! switched on.
  !
  if((mcaction.eq.101).or.(params%incl_scatsrc_mctherm.eq.1)) then
     if(mcscat_nrdirs.le.0) then
        mcscat_nrdirs = 1
     endif
  endif
  !
  ! Do some tests
  !
  if(.not.rt_incl_dust) then
     write(stdo,*) 'ERROR in Montecarlo module: rt_incl_dust is set to false, but the Monte Carlo'
     write(stdo,*) '        simulations are meant for dust radiative transfer...'
     write(stdo,*) '      Tip: Are you sure you have the file dustopac.inp present?'
     stop
  endif
  if(nrcells.le.0) then 
     write(stdo,*) 'ERROR in Montecarlo module: zero number of cells'
     stop
  endif
  if(igrid_mirror.eq.1) then
     if(igrid_type.ge.100) then
        write(stdo,*) 'ERROR: Mirror symmetry only available for 2-D/3-D spherical coordinates in AMR- or regular type grid'
        stop
     endif
     if((igrid_coord.lt.100).or.(igrid_coord.ge.200)) then
        write(stdo,*) 'ERROR: Mirror symmetry only available for 2-D/3-D spherical coordinates'
        stop
     endif
     if(amr_ydim.eq.0) then
        write(stdo,*) 'ERROR: Mirror symmetry only available for 2-D/3-D spherical coordinates, not 1-D spherical coordinates.'
        stop
     endif
  endif
  if(nrcells.gt.nrcellsmax) then
     write(stdo,*) 'ERROR: Nr of cells larger than max'
     stop
  endif
  if(.not.allocated(cellindex)) then
     write(stdo,*) 'INTERNAL ERROR IN MONTECARLO: Cellindex array not allocated'
     stop
  endif
  if(.not.allocated(cellvolume)) then
     write(stdo,*) 'INTERNAL ERROR IN MONTECARLO: Cellvolume array not allocated'
     stop
  endif
  if(.not.allocated(freq_nu)) then
     write(stdo,*) 'INTERNAL ERROR IN MONTECARLO: Frequency grid not yet allocated.'
     stop
  endif
  if(freq_nr.lt.1) then
     write(stdo,*) 'INTERNAL ERROR IN MONTECARLO: Nr of frequencies has wrong value.'
     stop
  endif
  if(star_sphere.and.mc_weighted_photons) then
     iwarned = .false.
     do istar=1,nstars
        dum = sqrt((star_pos(1,istar)-grid_contsph_x)**2+&
                   (star_pos(2,istar)-grid_contsph_y)**2+&
                   (star_pos(3,istar)-grid_contsph_z)**2)
        if(dum.ge.grid_contsph_r) then
           if(.not.iwarned) then
              write(stdo,*) 'WARNING: When using non-point-like stars outside the grid'
              write(stdo,*) '         as well as having weighted photons mode switched on,'
              write(stdo,*) '         please be warned that to simplify the launching of'
              write(stdo,*) '         photons we treat (only for the Monte Carlo run, and'
              write(stdo,*) '         only for the external stars) these stars as point'
              write(stdo,*) '         sources. If the star is located far enough out of'
              write(stdo,*) '         the grid, this yields only minimal errors, so it is fine.'
              iwarned = .true.
           endif
           if(dum.lt.10*star_r(istar)) then
              write(stdo,*) ' '
              write(stdo,*) '         BUT: We found external star that is too much in near field.'
              write(stdo,*) '         This means that our approximations are no longer valid.'
              write(stdo,*) '         Either move the star further out, or switch off the'
              write(stdo,*) '         weighted photons mode, or put the star inside the grid...'
              write(stdo,*) '         Aborting... Sorry!'
              stop
           endif
        endif
     enddo
  endif
  !
  ! If the stars are located outside the containing sphere (outside the
  ! grid), and if the weighted photons mode is switched on, then we 
  ! calculate the fraction of coverage of the grid w.r.t. the star
  !
  if(mc_weighted_photons) then
     do istar=1,nstars
        star_fraclum(istar) = 1.d0
        dum = sqrt((star_pos(1,istar)-grid_contsph_x)**2+&
                   (star_pos(2,istar)-grid_contsph_y)**2+&
                   (star_pos(3,istar)-grid_contsph_z)**2)
        if(dum.ge.grid_contsph_r) then
           if(dum.le.grid_contsph_r+star_r(istar)) then
              write(stdo,*) 'ERROR: Star outside of grid still crosses'
              write(stdo,*) '       the grid (or at least the containing '
              write(stdo,*) '       sphere around the grid). Aborting...'
              stop
           endif
           if(grid_contsph_r.gt.1d-4*dum) then
              star_fraclum(istar) = 0.5d0 * ( 1.d0 - &
                   sqrt(1.d0-(grid_contsph_r/dum)**2) )
           else
              star_fraclum(istar) = 0.25d0 * (grid_contsph_r/dum)**2
           endif
        endif
     enddo
  endif
  !
  ! Make sure that the dust optical data is read, but only read if not yet
  ! read before.
  !
  call read_dustdata(1)
  !
  ! Make sure that the dust density is read, but only read it from file
  ! if it is not yet in memory.
  !
  call read_dust_density(1)
  !
  ! Get some information about the grid
  !
  if(igrid_type.eq.101) then
     !
     ! The Delaunay triangulated grid
     !
     write(stdo,*) 'NOT YET READY: Delaunay unstructured grid'
     stop
  endif
  if(igrid_type.eq.201) then
     !
     ! The Voronoi grid
     !
     write(stdo,*) 'NOT YET READY: Voronoi unstructured grid'
     stop
  endif
  !
  ! For which kind of Monte Carlo simulation are we going to initialize?
  !
  if(mcaction.eq.1) then
     !
     ! Bjorkman & Wood thermal Monte Carlo
     !
     !
     ! Allocate the dust temperature array
     !
     if(allocated(dusttemp)) deallocate(dusttemp)
     allocate(dusttemp(1:dust_nr_species,1:nrcells),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate dusttemp array.'
        stop 
     endif
     dusttemp(:,:) = 0.d0
     !
     ! The Bjorkman & Wood algorithm is always done on the global
     ! frequency array. But for consistency (because some arrays are
     ! being used by the B&W thermal Monte Carlo as well as by the
     ! frequency-by-frequency scattering Monte Carlo) we will allocate
     ! the mc_frequencies(:) array nevertheless, and fill it with a copy
     ! of the freq_nu(:) array.
     !
     allocate(mc_frequencies(1:freq_nr),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_frequencies()'
        stop 
     endif
     mc_frequencies(:) = freq_nu(:)
     mc_nrfreq = freq_nr
     !
     ! Allocate the average output spectrum array
     !
     allocate(mc_average_spectrum(1:freq_nr),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_average_spectrum()'
        stop 
     endif
     mc_average_spectrum(:) = 0.d0
     allocate(mc_integerspec(1:freq_nr),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_integerspec()'
        stop 
     endif
     mc_integerspec(:) = 0
     !
     ! Allocate the cumulative energy arrays
     !
     allocate(mc_cumulener(1:dust_nr_species,1:nrcellsmax),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_cumulener()'
        stop 
     endif
     mc_cumulener(:,:) = 0.d0
     allocate(mc_cumulener_bk(1:dust_nr_species,1:nrcellsmax),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_cumulener_bk()'
        stop 
     endif
     mc_cumulener_bk(:,:) = 0.d0
     !
     ! Allocate the partial energy arrays
     !
     allocate(mc_enerpart(1:dust_nr_species),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_enerpart()'
        stop 
     endif
     mc_enerpart(:) = 0.d0
     !
     ! If we want to record the scattering source function during the 
     ! thermal Monte Carlo (which is, for memory reasons, rarely the case!!)
     ! then we must allocate the scattering source function array.
     !
     if(params%incl_scatsrc_mctherm.eq.1) then
        write(stdo,*) 'WARNING: Allocating HUGE array for scattering source function'
        write(stdo,*) '         for thermal Monte Carlo... Are you sure about this?'
        if(mcscat_nrdirs.le.0) then
           write(stdo,*) 'ERROR: mcscat_nrdirs.le.0 while initiating MC therm...'
           stop
        endif
        allocate(mcscat_scatsrc(1:mcscat_nrdirs,1:mc_nrfreq,1:nrcellsmax),STAT=ierr)
        if(ierr.ne.0) then
           write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mcscat_scatsrc()'
           write(stdo,*) '      As this can be a huge array, perhaps you have memory limitations?'
           write(stdo,*) '      This array requires ',mcscat_nrdirs*mc_nrfreq* &
                (nrcellsmax*4.d0)/(1024.*1024),' Mbytes'
           stop 
        else
           mcscat_scatsrc(:,:,:) = 0.d0
        endif
     endif
     !
     ! Some other checks
     !
     ! *** For now we do not allow the fast mode when incl_scatsrc_mctherm ***
     ! *** No particular reason for this, just that I am unsure if it works ***
     !
     if(params%optimized_motion.and.(scattering_mode.gt.1) &
          .and.(params%incl_scatsrc_mctherm.eq.1)) then
        write(stdo,*) 'ERROR: Sorry, cannot (for now) use the optimized'
        write(stdo,*) '       motion together with the incl_scatrsc_mctherm.'
        stop
     endif
     !
  elseif((mcaction.eq.101).or.(mcaction.eq.102)) then
     !
     ! The frequency-by-frequency scattering Monte Carlo simulation
     !
     !
     ! Check that the dust temperatures are there
     !
     call read_dust_temperature(1)
     !
     ! Check that the Monte Carlo frequencies array is set by the caller
     !
     if((mc_nrfreq.le.0).or.(.not.allocated(mc_frequencies))) then
        write(stdo,*) 'ERROR: When you want to do Monte Carlo scattering'
        write(stdo,*) '       you must set the mc_frequencies(:).'
        stop
     endif
     !
     ! Allocate the average output spectrum array
     !
     allocate(mc_average_spectrum(1:mc_nrfreq),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_average_spectrum()'
        stop 
     endif
     mc_average_spectrum(:) = 0.d0
     allocate(mc_integerspec(1:mc_nrfreq),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_integerspec()'
        stop 
     endif
     mc_integerspec(:) = 0
     !
     ! If scattering source function is to be calculated, then allocate the
     ! scattering source function array.  In the future we may include MC
     ! modes where this array is not required.
     !
     if(mcaction.eq.101) then
        if(mcscat_nrdirs.le.0) then
           write(stdo,*) 'ERROR: mcscat_nrdirs.le.0 while initiating MC scat...'
           stop
        endif
        allocate(mcscat_scatsrc(1:mcscat_nrdirs,1:mc_nrfreq,1:nrcellsmax),STAT=ierr)
        if(ierr.ne.0) then
           write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mcscat_scatsrc()'
           write(stdo,*) '      As this can be a huge array, perhaps you have memory limitations?'
           write(stdo,*) '      This array requires ',mcscat_nrdirs*mc_nrfreq* &
                (nrcellsmax*4.d0)/(1024.*1024),' Mbytes'
           stop 
        else
           mcscat_scatsrc(:,:,:) = 0.d0
        endif
     else
        allocate(mcscat_meanint(1:mc_nrfreq,1:nrcellsmax),STAT=ierr)
        if(ierr.ne.0) then
           write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mcscat_meanint()'
           write(stdo,*) '      As this can be a huge array, perhaps you have memory limitations?'
           write(stdo,*) '      This array requires ',mc_nrfreq* &
                (nrcellsmax*4.d0)/(1024.*1024),' Mbytes'
           stop 
        else
           mcscat_meanint(:,:) = 0.d0
        endif
     endif
     !
     ! 
     ! Check if the nr of viewing directions is set by the caller
     ! Note that even if isotropic scattering is used, nrdirs must be
     ! 1, because if nrdirs is 0 there will be no scattering source
     ! function. In the future we may include Monte Carlo scattering modes
     ! that do not need scattering source functions. 
     !
     if(mcscat_nrdirs.gt.1) then
        if(.not.(allocated(mcscat_dirs))) then
           write(stdo,*) 'ERROR: nrdirs>1, but no directions array set'
           stop
        endif
        if(scattering_mode.eq.1) then
           write(stdo,*) 'ERROR: Isotropic scattering mode must be with nrdirs=1.'
           stop
        endif
     endif
!----TO-ADD----
!!
!!   Allocate the skipping array for skipping extremely optically thick 
!!   cells in the scattering Monte Carlo.
!!
!    allocate(mc_scat_src_skip(1:nrcellsmax),STAT=ierr)
!    if(ierr.ne.0) then
!       write(stdo,*) 'ERROR in Monte Carlo: Could not allocate skipping array.'
!       stop
!    endif
!----TO-ADD----
     !
  else
     stop 7012
  endif
  !
  ! For anisotropic scattering, allocate phasefunction array
  !
  if(mcscat_nrdirs.ge.1) then
     allocate(mcscat_phasefunc(1:mcscat_nrdirs,1:dust_nr_species),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in camera module: Could not allocate phasefunction.'
        stop
     endif
  endif
  !
  ! Allocate arrays for the stellar source templates, and
  ! map the templates onto these frequencies
  !
  if(stellarsrc_nrtemplates.gt.0) then
     allocate(mc_stellarsrc_templates(1:mc_nrfreq,1:stellarsrc_nrtemplates),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in camera module: Could not allocate stellarsrc templates.'
        stop
     endif
     do itempl=1,stellarsrc_nrtemplates
        do inu=1,mc_nrfreq
           call hunt(freq_nu,freq_nr,mc_frequencies(inu),iinu)
           if((iinu.le.0).or.(iinu.gt.freq_nr)) then
              mc_stellarsrc_templates(inu,itempl) = 0.d0
           else
              if(iinu.eq.freq_nr) then
                 if(mc_frequencies(inu).eq.freq_nu(freq_nr)) then
                    mc_stellarsrc_templates(inu,itempl) = stellarsrc_templates(freq_nr,itempl)
                 else
                    mc_stellarsrc_templates(inu,itempl) = 0.d0
                 endif
              else
                 eps = ( mc_frequencies(inu) - freq_nu(iinu) ) / &
                       ( freq_nu(iinu+1) - freq_nu(iinu) )
                 if((eps.lt.0.d0).or.(eps.gt.1.d0)) stop 674
                 mc_stellarsrc_templates(inu,itempl) = &
                      (1.d0-eps)*stellarsrc_templates(iinu,itempl) + &
                             eps*stellarsrc_templates(iinu+1,itempl)
              endif
           endif
        enddo
     enddo
  endif
  !
  ! Allocate the opacities used for the MC code. Note that a more complex
  ! and generalized form of the opacities is loaded in the dust_module.f90,
  ! but we want to use a simplified form here, to speed up the code.
  !
  allocate(kappa_a(1:mc_nrfreq,1:dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate kappa_a()'
     stop 
  endif
  kappa_a(:,:) = 0.d0
  allocate(kappa_s(1:mc_nrfreq,1:dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate kappa_s()'
     stop 
  endif
  kappa_s(:,:) = 0.d0
  allocate(kappa_g(1:mc_nrfreq,1:dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate kappa_g()'
     stop 
  endif
  kappa_g(:,:) = 0.d0
  allocate(alpha_a(1:dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate alpha_a()'
     stop 
  endif
  alpha_a(:) = 0.d0
  allocate(alpha_s(1:dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate alpha_s()'
     stop 
  endif
  alpha_s(:) = 0.d0
  allocate(alphacum(1:dust_nr_species+1),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate alphacum()'
     stop 
  endif
  alphacum(:) = 0.d0
  !
  ! Get the opacities (assume global opacities for the moment)
  !
  temp    = 100.d0     ! Arbitrary temperature...
  do inu=1,mc_nrfreq
     do ispec=1,dust_nr_species
        kappa_a(inu,ispec) = find_dust_kappa_interpol(mc_frequencies(inu),ispec,temp,1,0,0)
        kappa_s(inu,ispec) = find_dust_kappa_interpol(mc_frequencies(inu),ispec,temp,0,1,0)
        kappa_g(inu,ispec) = find_dust_kappa_interpol(mc_frequencies(inu),ispec,temp,0,0,1)
     enddo
  enddo
  !
  ! If scattering_mode is 0, then put kappa_s to zero
  ! 
  if(scattering_mode.eq.0) then
     write(stdo,*) 'In Monte Carlo initialization: found that scattering_mode==0, so putting kappa_scat=0.'
     do inu=1,mc_nrfreq
        do ispec=1,dust_nr_species
           kappa_s(inu,ispec) = 0.d0
        enddo
     enddo
  endif
  !
  ! Allocate, if necessary, the mc_energy_stars(:) array
  !
  if(mc_weighted_photons.and.(nstars.gt.0)) then
     allocate(mc_energy_stars(1:nstars),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR: Could not allocate mc_energy_stars() array'
        stop
     endif
  endif
  !
  ! If photon statistics are requested, then allocate the arrays
  !
  if(params%debug_write_stats.ne.0) then
     allocate(mc_ilastphot(1:nrcellsmax),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_ilastphot()'
        stop 
     endif
     mc_ilastphot(:) = 0
     allocate(mc_iphotcount(1:nrcellsmax),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate mc_iphotcount()'
        stop 
     endif
     mc_iphotcount(:) = 0
     mc_iphotcurr = 0
  endif
  !
end subroutine montecarlo_init


!--------------------------------------------------------------------------
!                 Finish the Monte Carlo module
!--------------------------------------------------------------------------
subroutine montecarlo_cleanup()
  implicit none
  if(allocated(mc_frequencies)) deallocate(mc_frequencies)
  if(allocated(mcscat_dirs)) deallocate(mcscat_dirs)
  mc_nrfreq      = 0
  mcscat_nrdirs  = 0
  call montecarlo_partial_cleanup()
end subroutine montecarlo_cleanup


!--------------------------------------------------------------------------
!                 Finish the Monte Carlo module
!--------------------------------------------------------------------------
subroutine montecarlo_partial_cleanup()
  implicit none
  if(allocated(mc_average_spectrum)) deallocate(mc_average_spectrum)
  if(allocated(mc_integerspec)) deallocate(mc_integerspec)
  if(allocated(mc_enerpart)) deallocate(mc_enerpart)
  if(allocated(mc_cumulener)) deallocate(mc_cumulener)
  if(allocated(mc_cumulener_bk)) deallocate(mc_cumulener_bk)
  if(allocated(mc_ilastphot)) deallocate(mc_ilastphot)
  if(allocated(mc_iphotcount)) deallocate(mc_iphotcount)
  if(allocated(kappa_a)) deallocate(kappa_a)
  if(allocated(kappa_s)) deallocate(kappa_s)
  if(allocated(kappa_g)) deallocate(kappa_g)
  if(allocated(alpha_a)) deallocate(alpha_a)
  if(allocated(alpha_s)) deallocate(alpha_s)
  if(allocated(alphacum)) deallocate(alphacum)
  if(allocated(mcscat_scatsrc)) deallocate(mcscat_scatsrc)
  if(allocated(mcscat_meanint)) deallocate(mcscat_meanint)
  if(allocated(mc_cumulthermemis)) deallocate(mc_cumulthermemis)
  if(allocated(mc_stellarsrc_templates)) deallocate(mc_stellarsrc_templates)
  if(allocated(mcscat_phasefunc)) deallocate(mcscat_phasefunc)
  if(allocated(mc_energy_stars)) deallocate(mc_energy_stars)
!----TO-ADD----
! if(allocated(mc_scat_src_skip)) deallocate(mc_scat_src_skip)
!----TO-ADD----
end subroutine montecarlo_partial_cleanup


!----------------------------------------------------------------------------
!          READ THE FREQUENCIES FOR THE MONOCHROMATIC MONTE CARLO
!----------------------------------------------------------------------------
subroutine read_mc_frequencies()
  implicit none
  logical :: flag
  integer :: inu,ierr,i,iformat
  logical :: fex
  !
  ! Deallocate frequencies
  !
  if(allocated(mc_frequencies)) deallocate(mc_frequencies)
  !
  ! Read frequencies / wavelengths from a file
  !
  inquire(file='mcmono_wavelength_micron.inp',exist=fex)
  if(fex) then
     open(unit=1,file='mcmono_wavelength_micron.inp')
     read(1,*) mc_nrfreq
     allocate(mc_frequencies(mc_nrfreq),STAT=ierr)
     if(ierr.ne.0) then
        write(stdo,*) 'ERROR in montecarlo module: Could not allocate '
        write(stdo,*) '      mc_frequencies(:).'
        stop 
     endif
     do inu=1,mc_nrfreq
        read(1,*) mc_frequencies(inu)
        mc_frequencies(inu) = 1d4*cc/mc_frequencies(inu)
     enddo
     close(1)
  else
     write(stdo,*) 'ERROR: Cannot find mcmono_wavelength_micron.inp'
     stop
  endif
end subroutine read_mc_frequencies



!--------------------------------------------------------------------------
!       COMPUTE ALL TOTAL LUMINOSITIES AND CUMULATIVE LUMINOSITIES
!--------------------------------------------------------------------------
subroutine montecarlo_compute_total_luminosities(params,incl_therm)
  implicit none
  type(mc_params) :: params
  logical :: incl_therm
  integer :: istar,inu,icell,index,itemplate,ispec,ierr
  double precision :: dum,ener
  !
  ! Determine the luminosity of the star(s) in all these wavelength bins
  !
  if(.not.allocated(star_lum)) stop 8709
  if(.not.allocated(star_lumcum)) stop 8708
  starlumtot = 0.d0
  do istar=1,nstars
     star_lum(istar) = 0.d0
     do inu=1,freq_nr
        dum = pi * star_spec(inu,istar) * fourpi * star_r(istar)**2
        star_lum(istar) = star_lum(istar) + dum * freq_dnu(inu)
     enddo
     starlumtot = starlumtot + star_lum(istar)
  enddo
  if(starlumtot.gt.0.d0) then
     star_lumcum(1) = 0.d0
     do istar=1,nstars
        star_lumcum(istar+1) = star_lum(istar) / starlumtot
     enddo
     star_lumcum(nstars+1) = 1.d0
  endif
  !
  ! Determine the luminosity of the external radiation field.  This
  ! luminosity is defined as the integral of the incoming part of the
  ! external flux at a sphere encompassing the entire grid.  Of course, if
  ! you take the size of the grid larger (for the same problem), then the
  ! external luminosity increases, because the surface of the encompassing
  ! sphere increases. But most of these photons will then simply escape to
  ! infinity again without even interacting with the matter. Therefore this
  ! scaling is not a major problem. One should keep in mind, though, that it
  ! may strain the photon-statistics when you have too larger outer boundary
  ! (photons then rarely reach the inner zones).
  !
  if(incl_extlum.ne.0) then 
     !
     ! Determine the luminosity of the external radiation field
     !
     extlumtot = 0.d0
     do inu=1,freq_nr
        dum = pi * extlum_intens(inu) * fourpi * grid_contsph_r**2
        extlumtot = extlumtot + dum * freq_dnu(inu)
     enddo
     extlum_intens_cum(1) = 0.d0
     do inu=1,freq_nr
        extlum_intens_cum(inu+1) = extlum_intens_cum(inu) + &
             pi * extlum_intens(inu) * fourpi *             &
             grid_contsph_r**2 * freq_dnu(inu) / extlumtot
     enddo
     if(abs(extlum_intens_cum(freq_nr+1)-1.d0).gt.1d-10) stop 82791
  else
     extlumtot = 0.d0
  endif
  !
  ! Compute the luminosity of the internally produced energy qvisc
  ! and make the cumulative array for the qvisc. Note that the
  ! cumulative array qvisc_cum is indexed with icell=1,nrcells+1,
  ! while most other arrays are indexed through the indexing array
  ! cellindex in order to allow adding and removal of cells without
  ! having to reshuffle data to fill in "holes" in the arrays when
  ! cells are removed.
  !
  if(incl_qvisc.ne.0) then
     !
     !######### ALLOCATE QVISC_CUM 
     !
     write(stdo,*) 'FOR NOW VISCOSITY IS NOT ACTIVATED'
     stop
     !
     if(.not.allocated(qvisc)) then
        write(stdo,*) 'INTERNAL ERROR: Viscous heating array not allocated'
        stop
     endif
     if(.not.allocated(qvisc_cum)) then
        write(stdo,*) 'INTERNAL ERROR: Viscous cumulative array not allocated'
        stop
     endif
     qvisclumtot = 0.d0
     do icell=1,nrcells
        index = cellindex(icell)
        qvisclumtot = qvisclumtot + qvisc(index)*cellvolume(index)
     enddo
     if(qvisclumtot.gt.0.d0) then 
        dum = 0.d0
        do icell=1,nrcells
           index = cellindex(icell)
           qvisc_cum(icell) = dum
           dum = dum + qvisc(index)*cellvolume(index)/qvisclumtot
        enddo
        qvisc_cum(nrcells+1) = dum
        if(abs(dum-1.d0).gt.1d-12) then
           write(stdo,*) 'INTERNAL ERROR: qplus somehow doesnt add up'
           stop
        endif
     endif
  else
     qvisclumtot = 0.d0
  endif
  !
  ! Compute the luminosity of the locally produced continuous stellar source.
  ! This is used in simulations of entire galaxies, where individual stars
  ! are too many to be modeled as discrete objects. 
  !
  ! The stellar source is specified as density for each stellar template.
  ! That is: you pre-specify a number (1 or larger) of star spectra that
  ! are luminosity L_nu per gram of the star. Then in each cell of the grid
  ! for each of the stellar templates a density of stars in g/cm^3 is specified
  ! and the local emissivity of the stars are then determined according to
  ! the product of the stellar density and the template spectrum. Note that
  ! a template can also be a galaxy template (consisting of a mix of many type
  ! of stars). Only if you wish to have different stellar populations in 
  ! different regions will you have to define multiple templates.
  !
  if(incl_stellarsrc.ne.0) then
     if(stellarsrc_nrtemplates.le.0) then
        write(stdo,*) 'ERROR: No stellar templates available for the distributed'
        write(stdo,*) '       stellar sources (used for galaxy simulations)'
        stop
     endif
     if(.not.allocated(stellarsrc_dens)) then
        write(stdo,*) 'INTERNAL ERROR: stellar source density not allocated'
        stop
     endif
     if(.not.allocated(stellarsrc_lumcum)) then
        write(stdo,*) 'INTERNAL ERROR: stellar source cumulative array not allocated'
        stop
     endif
     stellarsrclumtot = 0.d0
     do icell=1,nrcells
        !
        ! NOTE: All cumulative arrays are indexed by "icell", all others by "index"
        ! Note: The templateslum is already including fourpi
        !
        index = cellindex(icell)
        stellarsrc_lumcum(icell) = stellarsrclumtot
        dum = 0.d0
        do itemplate=1,stellarsrc_nrtemplates
           dum = dum + stellarsrc_dens(itemplate,index) *               &
                       stellarsrc_templateslum(itemplate)
        enddo
        stellarsrclumtot = stellarsrclumtot + dum * cellvolume(index)
     enddo
     if(stellarsrclumtot.gt.0.d0) then 
        do icell=1,nrcells
           stellarsrc_lumcum(icell) = stellarsrc_lumcum(icell) / stellarsrclumtot
        enddo
        stellarsrc_lumcum(nrcells+1) = 1.d0
     endif
  else
     stellarsrclumtot = 0.d0
  endif
  !
  ! Compute the luminosity of the quantum heated grains
  !
  ! NOTE: All cumulative arrays are indexed by "icell", all others by "index"
  !
  emisquanttot = 0.d0
  if(incl_quantum.ne.0) then
     if(incl_quantum.gt.0) then
        write(stdo,*) 'HALTING: For now only the externally computed quantum '
        write(stdo,*) '         mode is available (incl_quantum<0).'
        stop
     endif
     if(.not.allocated(emisquant_loccum)) then
        write(stdo,*) 'ERROR: When quantum heating is included,'
        write(stdo,*) '       the array emisquant_loccum must be allocated.'
        stop
     endif
     do icell=1,nrcells
        index = cellindex(icell)
        dum = 0.d0
        do inu=1,freq_nr
           emisquant_loccum(inu,index) = dum
           dum = dum + emisquant(inu,index) * fourpi *                   &
                       cellvolume(index) * freq_dnu(inu)
        enddo
        emisquant_loctot(index)           = dum
        emisquanttot                      = emisquanttot + dum
        if(dum.gt.0.d0) then
           dum = 1.d0 / dum
           do inu=1,freq_nr
              emisquant_loccum(inu,index) = emisquant_loccum(inu,index) * dum
           enddo
           emisquant_loccum(freq_nr+1,index) = 1.d0
        else
           do inu=1,freq_nr+1
              emisquant_loccum(inu,index) = 0.d0
           enddo
        endif
     enddo
     if(emisquanttot.gt.0.d0) then 
        dum = 0.d0
        do icell=1,nrcells
           index = cellindex(icell)
           emisquant_cum(icell) = dum
           dum = dum + emisquant_loctot(index)
        enddo
        emisquant_cum(nrcells+1) = dum
        if(abs(dum-1.d0).gt.1d-12) then
           write(stdo,*) 'INTERNAL ERROR: emisquant somehow doesnt add up'
           stop
        endif
        write(stdo,*) 'L_emisquant/L_stars = ',emisquanttot/starlumtot
     else
        do icell=1,nrcells
           emisquant_cum(icell) = 0.d0
        enddo
        emisquant_cum(nrcells+1) = 0.d0
     endif
  endif
  !
  ! Include the thermal emission as well, IF requested
  !
  mc_thermemistot = 0.d0
  if(incl_therm) then
     !
     ! Allocate the mc_cumulthermemis(:) array
     ! 
     if(allocated(mc_cumulthermemis)) deallocate(mc_cumulthermemis)
     allocate(mc_cumulthermemis(1:nrcells+1),STAT=ierr)
     if(ierr.gt.0) then
        write(stdo,*) 'ERROR: Could not allocate mc_cumulthermemis(:)'
        stop
     endif
     !
     ! First compute for each cell the total thermal emissivity in 
     ! erg/s/cm^3 (not per sterradian!)
     !
     mc_cumulthermemis(1) = 0.d0
     do icell=1,nrcells
        index = cellindex(icell)
        ener = 0.d0
        do ispec=1,dust_nr_species
           !
           ! Compute the emissivity and the total emitted energy
           ! per second per cm^3 by this species.
           !
           do inu=1,freq_nr
              cellalpha(inu) = kappa_a(inu,ispec)*dustdens(ispec,index)
           enddo
           ener = ener + absevfunc(dusttemp(ispec,index))
        enddo
        !
        ! Now add the contribution 
        ! NOTE: Not the index but the icell!!
        !
        index = cellindex(icell)
        mc_cumulthermemis(icell+1) = mc_cumulthermemis(icell) +    &
                                     ener * cellvolume(index)
     enddo
     !
     ! Store the total thermal emissivity
     !
     mc_thermemistot = mc_cumulthermemis(nrcells+1)
     !
     ! Now normalize to unity
     !
     mc_cumulthermemis(:) = mc_cumulthermemis(:) / ( mc_thermemistot + 1d-90 )
     !
     ! Now make sure to put the normalization at nrcells+1 to 1
     !
     mc_cumulthermemis(nrcells+1) = 1.d0
   endif
   !
end subroutine montecarlo_compute_total_luminosities


!--------------------------------------------------------------------------
!    COMPUTE ALL FREQ-SPECIFIC LUMINOSITIES AND CUMULATIVE LUMINOSITIES
!--------------------------------------------------------------------------
subroutine montecarlo_compute_freqdep_luminosities(params,freq)
  implicit none
  type(mc_params) :: params
  integer :: istar,inu,icell,index,ispec,ierr,itemplate
  double precision :: ener,freq,eps,temp,dum
  double precision, allocatable :: kabs(:)
!----TO-ADD----
! type(amr_branch), pointer :: b
! double precision :: alp,cellsize,tau
!----TO-ADD----
  !
  ! Do checks
  !
  if(.not.allocated(dusttemp)) stop 4098
  if(.not.allocated(dustdens)) stop 4099
  if(freq.le.0.d0) then
     write(stdo,*) 'ERROR: Find frequency that is .le.0'
     stop
  endif
  !
  ! First find the inu and eps of the freq in the global frequency array
  !
  call hunt(freq_nu,freq_nr,freq,inu)
  if(inu.eq.freq_nr) then
     if(abs(freq_nu(freq_nr)/freq-1.d0).lt.1d-12) then
        inu=freq_nr-1
        eps=1.d0
     endif
  endif
  if(inu.eq.0) then
     if(abs(freq_nu(1)/freq-1.d0).lt.1d-12) then
        inu=1
        eps=0.d0
     endif
  endif
  if((inu.lt.1).or.(inu.ge.freq_nr)) then
     write(stdo,*) 'ERROR in montecarlo module: Wavelength out of range of the global wavelength array'
     write(stdo,*) 'inu = ',inu,', freq_nr = ',freq_nr
     write(stdo,*) 'freq = ',freq,'  nu0,nu1 = ',freq_nu(1),freq_nu(freq_nr)
     stop
  endif
  eps = (freq-freq_nu(inu)) / (freq_nu(inu+1)-freq_nu(inu))
  if((eps.gt.1.d0).or.(eps.lt.0.d0)) stop 7301
  !
  ! Find the dust opacities at this wavelength
  !
  allocate(kabs(1:dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate kabs(:)'
     stop 
  endif
  temp = 100   ! Dummy temperature
  do ispec=1,dust_nr_species
     kabs(ispec) = find_dust_kappa_interpol(freq,ispec,temp,1,0,0)
  enddo
  !
  ! Determine the luminosity of the star(s) at this wavelength
  !
  if(.not.allocated(star_lum)) stop 8709
  if(.not.allocated(star_lumcum)) stop 8708
  starlumtot = 0.d0
  do istar=1,nstars
     star_lum(istar) = (1.d0-eps) * pi * star_spec(inu,istar) * fourpi * star_r(istar)**2 + &
                              eps * pi * star_spec(inu+1,istar) * fourpi * star_r(istar)**2
     starlumtot = starlumtot + star_lum(istar)
  enddo
  if(starlumtot.gt.0.d0) then
     star_lumcum(1) = 0.d0
     do istar=1,nstars
        star_lumcum(istar+1) = star_lum(istar) / starlumtot
     enddo
     star_lumcum(nstars+1) = 1.d0
  endif
  !
  ! Determine the luminosity of the external radiation field.  This
  ! luminosity is defined as the integral of the incoming part of the
  ! external flux at a sphere encompassing the entire grid.  Of course, if
  ! you take the size of the grid larger (for the same problem), then the
  ! external luminosity increases, because the surface of the encompassing
  ! sphere increases. But most of these photons will then simply escape to
  ! infinity again without even interacting with the matter. Therefore this
  ! scaling is not a major problem. One should keep in mind, though, that it
  ! may strain the photon-statistics when you have too large outer boundary
  ! (photons then rarely reach the inner zones).
  !
  if(incl_extlum.ne.0) then 
     !
     ! Determine the luminosity of the external radiation field
     !
     extlumtot = (1.d0-eps) * pi * extlum_intens(inu) * fourpi * grid_contsph_r**2 + &
                        eps * pi * extlum_intens(inu+1) * fourpi * grid_contsph_r**2
  else
     extlumtot = 0.d0
  endif
  !
  ! Compute the luminosity of the locally produced continuous stellar source.
  ! This is used in simulations of entire galaxies, where individual stars
  ! are too many to be modeled as discrete objects. 
  !
  ! The stellar source is specified as density for each stellar template.
  ! That is: you pre-specify a number (1 or larger) of star spectra that
  ! are luminosity L_nu per gram of the star. Then in each cell of the grid
  ! for each of the stellar templates a density of stars in g/cm^3 is specified
  ! and the local emissivity of the stars are then determined according to
  ! the product of the stellar density and the template spectrum. Note that
  ! a template can also be a galaxy template (consisting of a mix of many type
  ! of stars). Only if you wish to have different stellar populations in 
  ! different regions will you have to define multiple templates.
  !
  stellarsrclumtot = 0.d0
  if(incl_stellarsrc.ne.0) then
     if(stellarsrc_nrtemplates.le.0) then
        write(stdo,*) 'ERROR: No stellar templates available for the distributed'
        write(stdo,*) '       stellar sources (used for galaxy simulations)'
        stop
     endif
     if(.not.allocated(stellarsrc_dens)) then
        write(stdo,*) 'INTERNAL ERROR: stellar source density not allocated'
        stop
     endif
     if(.not.allocated(stellarsrc_templateslum)) then
        write(stdo,*) 'INTERNAL ERROR: stellar source templateslum not allocated'
        stop
     endif
     if(.not.allocated(stellarsrc_lumcum)) then
        write(stdo,*) 'INTERNAL ERROR: stellar source cumulative array not allocated'
        stop
     endif
     stellarsrclumtot = 0.d0
     do icell=1,nrcells
        !
        ! NOTE: All cumulative arrays are indexed by "icell", all others by "index"
        !
        index = cellindex(icell)
        stellarsrc_lumcum(icell) = stellarsrclumtot
        dum = 0.d0
        do itemplate=1,stellarsrc_nrtemplates
           dum = dum + stellarsrc_dens(itemplate,index) * fourpi *             &
                 ( (1-eps) * mc_stellarsrc_templates(inu,itemplate) +          &
                       eps * mc_stellarsrc_templates(inu+1,itemplate) )
        enddo
        stellarsrclumtot = stellarsrclumtot + dum * cellvolume(index)
     enddo
     if(stellarsrclumtot.gt.0.d0) then 
        do icell=1,nrcells
           stellarsrc_lumcum(icell) = stellarsrc_lumcum(icell) / stellarsrclumtot
        enddo
        stellarsrc_lumcum(nrcells+1) = 1.d0
     endif
  else
     stellarsrclumtot = 0.d0
  endif
  !
  ! Compute the luminosity of the quantum heated grains
  !
  ! NOTE: All cumulative arrays are indexed by "icell", all others by "index"
  !
  emisquanttot = 0.d0
!!
!! ******* NOT YET READY *******
!!
!!  if(incl_quantum.ne.0) then
!!     if(incl_quantum.gt.0) then
!!        write(stdo,*) 'HALTING: For now only the externally computed quantum '
!!        write(stdo,*) '         mode is available (incl_quantum<0).'
!!        stop
!!     endif
!!     do icell=1,nrcells
!!        index = cellindex(icell)
!!        dum = 0.d0
!!        do inu=1,freq_nr
!!           emisquant_loccum(inu,index) = dum
!!           dum = dum + emisquant(inu,index) * fourpi *                   &
!!                       cellvolume(index) * freq_dnu(inu)
!!        enddo
!!        emisquant_loctot(index)           = dum
!!        emisquanttot                      = emisquanttot + dum
!!        if(dum.gt.0.d0) then
!!           dum = 1.d0 / dum
!!           do inu=1,freq_nr
!!              emisquant_loccum(inu,index) = emisquant_loccum(inu,index) * dum
!!           enddo
!!           emisquant_loccum(freq_nr+1,index) = 1.d0
!!        else
!!           do inu=1,freq_nr+1
!!              emisquant_loccum(inu,index) = 0.d0
!!           enddo
!!        endif
!!     enddo
!!     if(emisquanttot.gt.0.d0) then 
!!        dum = 0.d0
!!        do icell=1,nrcells
!!           index = cellindex(icell)
!!           emisquant_cum(icell) = dum
!!           dum = dum + emisquant_loctot(index)
!!        enddo
!!        emisquant_cum(nrcells+1) = dum
!!        if(abs(dum-1.d0).gt.1d-12) then
!!           write(stdo,*) 'INTERNAL ERROR: emisquant somehow doesnt add up'
!!           stop
!!        endif
!!        write(stdo,*) 'L_emisquant/L_stars = ',emisquanttot/starlumtot
!!     else
!!        do icell=1,nrcells
!!           emisquant_cum(icell) = 0.d0
!!        enddo
!!        emisquant_cum(nrcells+1) = 0.d0
!!     endif
!!  endif
  !
  ! Allocate the mc_cumulthermemis(:) array
  ! 
  if(allocated(mc_cumulthermemis)) deallocate(mc_cumulthermemis)
  allocate(mc_cumulthermemis(1:nrcells+1),STAT=ierr)
  if(ierr.gt.0) then
     write(stdo,*) 'ERROR: Could not allocate mc_cumulthermemis(:)'
     stop
  endif
  !
  ! Include the thermal emission as well
  !
  mc_thermemistot = 0.d0
  !
  ! First compute for each cell the total thermal emissivity in 
  ! erg/s/cm^3 (not per sterradian!)
  !
  mc_cumulthermemis(1) = 0.d0
  do icell=1,nrcells
     index = cellindex(icell)
     ener  = 0.d0
!----TO-ADD----
!    alp  = 0.d0
!----TO-ADD----
     do ispec=1,dust_nr_species
        !
        ! Compute the total emitted energy per second per cm^3 by this
        ! species at this frequency
        !
        ener = ener + dustdens(ispec,index) * kabs(ispec) *        &
                      bplanck(dusttemp(ispec,index),freq)
!----TO-ADD----
!       alp  = alp + dustdens(ispec,index) * kabs(ispec)
!----TO-ADD----
     enddo
     ener = ener * fourpi
!----TO-ADD----
!    b => amr_theleafs(icell)
!    call amr_min_cellsize(b,cellsize) 
!    tau = alp * cellsize
!    if(tau.gt.mc_scat_src_maxtau) then
!       mc_scat_src_skip(index) = .true.
!       ener = 0.d0
!    else
!       mc_scat_src_skip(index) = .false.
!    endif
!----TO-ADD----
     !
     ! Now compute from this the cumulative thermal emission from
     ! this cell compared to the previous cells.
     !
     mc_cumulthermemis(icell+1) = mc_cumulthermemis(icell) +    &
                                  ener * cellvolume(index)
  enddo
  !
  ! Store the total thermal emissivity
  !
  mc_thermemistot = mc_cumulthermemis(nrcells+1)
  !
  ! Now normalize to unity
  !
  mc_cumulthermemis(:) = mc_cumulthermemis(:) / ( mc_thermemistot + 1d-90 )
  !
  ! Fix a small bug, reported by Attila Juhasz on Jan 20, 2011: If there is
  ! no thermal emission, then the normalization must be by hand normalized
  ! to 1.d0
  ! 
  mc_cumulthermemis(nrcells+1) = 1.d0
  !
  ! Deallocate temporary array
  !
  if(allocated(kabs)) deallocate(kabs)
  !
end subroutine montecarlo_compute_freqdep_luminosities



!--------------------------------------------------------------------------
!                 MAIN ROUTINE FOR MONTE CARLO
!
! This is the main routine for the Monte Carlo code ala Bjorkman & Wood
! (2001) ApJ 554, 615.
!
! This routine follows the motion of nphot photon packages as they emerge
! from one or more stars and move through the medium. These photons scatter
! and absorb/reemit multiple times until they escape the system. Each
! absorbtion/reemission event also means updating the temperature of the
! cell, and a change in frequency of the photon package. The temperature of
! the cell is updated in such a way that all the photon packages
! before+including this one in total have a correct distribution over
! frequency. This is the trick first described by Bjorkman & Wood. This
! subroutine returns the average spectrum of the outcoming photons, averaged
! over all outgoing directions. 
!
! --------------------------------------------------------------------------
subroutine do_monte_carlo_bjorkmanwood(params,ierror,resetseed)
  implicit none
  type(mc_params) :: params
  doubleprecision :: tempav,dum
  doubleprecision :: aa,fact
  doubleprecision :: ener,lumtotinv
  logical :: ievenodd
  logical,optional :: resetseed
  integer :: ierror,countwrite,cntdump,nphot,index
  integer :: inu,iphot,count,ispec,istar,icell,nsrc,nstarsrc
  integer :: iseeddum,cnt,isd,ipstart,itemplate
  !
  ! Some consistency checks
  ! 
  if((nstars.eq.0).and.(incl_stellarsrc.eq.0).and. &
     (incl_extlum.eq.0).and.(incl_qvisc.eq.0)) then
     write(stdo,*) 'ERROR: There are no sources of photons in this setup.'
     stop 
  endif
  if(params%countwrite.lt.1) then
     write(stdo,*) 'ERROR: Countwrite must be > 0'
     stop 
  endif
  if(params%cntdump.lt.params%countwrite) then
     write(stdo,*) 'ERROR: Cntdump must be >= ',params%countwrite
     stop 
  endif
  if(params%incl_scatsrc_mctherm.eq.1) then
     if(scattering_mode.eq.0) then
        write(stdo,*) 'ERROR: If scattering is to be included in the'
        write(stdo,*) '       thermal Monte Carlo, then the scattering_mode'
        write(stdo,*) '       should of course not be 0...'
        stop
     endif
  endif
  !
  ! Initialize the Monte Carlo module by allocating the required arrays
  ! and doing a number of standard checks
  !
  call montecarlo_init(params,ierror,1,resetseed)
  !
  ! Set some local integers and logicals
  !
  countwrite = params%countwrite
  cntdump    = params%cntdump
  nphot      = params%nphot_therm
  intplt     = params%iranfreqmode
  ievenodd   = .true.
  ieventcounttot = 0
  !
  ! If write statistics, then open this file
  !
  if(params%debug_write_stats.ne.0) then
     open(unit=4,file='statistics.out',status='unknown')
     write(4,*) freq_nr
     write(4,*)
  endif
  !
  ! Put to zero various arrays 
  !
  if(incl_quantum.ne.0) then
     if((.not.(allocated(emisquant))).or.(.not.(allocated(miquant))).or. &
        (.not.(allocated(emisquant_cum)))) then
        write(stdo,*) 'INTERNAL ERROR: Including quantum, but quantum not initalized.'
        stop
     endif
     miquant(:,:)       = 0.d0
     emisquant(:,:)     = 0.d0
     emisquant_cum(:)   = 0.d0
  endif
  if(.not.(allocated(dusttemp))) then
     write(stdo,*) 'INTERNAL ERROR: dusttemp arrays not initalized.'
     stop
  endif
  dusttemp(:,:) = 0.d0
  !
  ! Now check if all input array data are OK
  !
  call mc_check_inputdata()
  !
  ! Prepare the database for the emissivity. This is the trick from
  ! Michiel Min that speeds up the code so much, because the CPU-intensive
  ! calculations with integrations over the frequency domain are now done
  ! beforehand and stored in a look-up table.
  !
  write(stdo,*) 'Computing emissivity database...'
  call flush(stdo)
  call make_emiss_dbase(params%ntemp,params%temp0,params%temp1)
  !
  ! Now compute all the luminosities and cumulative luminosity arrays
  !
  write(stdo,*) 'Computing total input luminosities...'
  call flush(stdo)
  call montecarlo_compute_total_luminosities(params,.false.)
  !
  ! Compute the energy per photon package
  ! 
  if(.not.mc_weighted_photons) then
     !
     ! Standard mode: a single photon package energy. Different
     ! luminosities of the various sources are represented by different
     ! numbers of emitted packages.
     !
     ! The energy per photon package is Ltot divided by nr of photons.
     !
     energy = ( qvisclumtot + stellarsrclumtot + emisquanttot +           &
                starlumtot + extlumtot ) / nphot
     !
     ! Check 
     !
     if(energy.le.0.d0) then
        write(stdo,*) 'ERROR in setup: There exists no source of photons.'
        write(stdo,*) '      Must have either stars, viscous heating, external'
        write(stdo,*) '      heating or any other netto energy source.'
        stop
     endif
     !
     ! If there is mirror symmetry (a mode only possible for spherical 
     ! coordinates) then divide this by 2 as we only follow the upper
     ! half
     !
     if(igrid_mirror.eq.1) then
        energy = energy * 0.5d0
     endif
     !
     ! Compute the cumulative luminosities of the various different kinds of
     ! sources of photons
     !
     lumtotinv = 1.d0 / ( qvisclumtot + emisquanttot + extlumtot +    &
                          stellarsrclumtot + starlumtot )
     mc_cumlum1  = ( qvisclumtot ) * lumtotinv
     mc_cumlum2  = ( qvisclumtot + emisquanttot ) * lumtotinv
     mc_cumlum3  = ( qvisclumtot + emisquanttot  + extlumtot ) * lumtotinv
     mc_cumlum4  = ( qvisclumtot + emisquanttot  + extlumtot + stellarsrclumtot ) * lumtotinv
  else
     !
     ! Weighted photon package mode. The different luminosities of the various
     ! sources are represented by different energies per package, but each
     ! source (if non-zero) will still have the same number of photon
     ! packages emitted. 
     ! 
     if(.not.allocated(mc_energy_stars).and.(nstars.gt.0)) stop 9210
     !
     ! Count the nr of sources
     !
     nsrc = 0
     if(qvisclumtot.gt.0.d0) nsrc = nsrc + 1
     if(emisquanttot.gt.0.d0) nsrc = nsrc + 1
     if(extlumtot.gt.0.d0) nsrc = nsrc + 1
     if(stellarsrclumtot.gt.0.d0) nsrc = nsrc + 1
     nstarsrc = 0
     do istar=1,nstars
        if(star_lum(istar).gt.0.d0) then
           nsrc = nsrc + 1
           nstarsrc = nstarsrc + 1
        endif
     enddo
     if(nsrc.eq.0) then
        write(stdo,*) 'ERROR in setup: There exists no source of photons.'
        write(stdo,*) '      Must have either stars, viscous heating, external'
        write(stdo,*) '      heating or any other netto energy source.'
        stop
     endif
     !
     ! Take into account the mirroring, if active
     !
     if(igrid_mirror.eq.1) then
        fact = 0.5d0
     else
        fact = 1.d0
     endif
     !
     ! Compute the energy per package for each source
     !
     mc_energy_qvisc      = fact * qvisclumtot * nsrc / nphot
     mc_energy_quant      = fact * emisquanttot * nsrc / nphot
     mc_energy_extlum     = fact * extlumtot * nsrc / nphot
     mc_energy_stellarsrc = fact * stellarsrclumtot * nsrc / nphot
     do istar=1,nstars
        mc_energy_stars(istar) = star_fraclum(istar) * fact *       &
                                 star_lum(istar) * nsrc / nphot
     enddo
     !
     ! Compute the thesholds for the random assignment of packages
     ! to the various sources
     !
     if(qvisclumtot.gt.0.d0) then
        mc_cumlum1  = 1.d0 / nsrc
     else
        mc_cumlum1  = 0.d0
     endif
     if(emisquanttot.gt.0.d0) then
        mc_cumlum2  = mc_cumlum1 + 1.d0 / nsrc
     else
        mc_cumlum2  = mc_cumlum1
     endif
     if(extlumtot.gt.0.d0) then
        mc_cumlum3  = mc_cumlum2 + 1.d0 / nsrc
     else
        mc_cumlum3  = mc_cumlum2
     endif
     if(stellarsrclumtot.gt.0.d0) then
        mc_cumlum4  = mc_cumlum3 + 1.d0 / nsrc
     else
        mc_cumlum4  = mc_cumlum3
     endif
     if(nstarsrc.gt.0) then
        star_lumcum(1) = 0.d0
        if(star_lum(1).gt.0.d0) then
           star_lumcum(2) = 1.d0 / nstarsrc
        else
           star_lumcum(2) = 0.d0
        endif
        do istar=2,nstars
           if(star_lum(istar).gt.0.d0) then
              star_lumcum(istar+1) = star_lumcum(istar) + 1.d0 / nstarsrc
           else
              write(stdo,*) 'ERROR: Cannot treat star with zero luminosity'
              write(stdo,*) '       Give tiny (but non-zero) luminosity instead.'
              stop
           endif
        enddo
        if(abs(star_lumcum(nstars+1)-1.d0).gt.1d-14) stop 7311
        star_lumcum(nstars+1) = 1.d0
     endif
  endif
  !   
  ! Open the path file
  ! For debugging only!
  !
  if(params%debug_write_path.eq.1) then
     open(unit=5,file='path.dat',status='unknown')
  endif
  !
  ! Count set to 1
  !
  count = 1
  cnt = 1
  write(stdo,*) ' '
  write(stdo,*) 'Starting the thermal Monte Carlo simulation....'
  call flush(stdo)
  !
  ! Default
  !
  ipstart = 1
  !
  ! Now, if the irestart.ge.1, then read the latest safety dump
  !
  if(params%irestart.ge.1) then
      !     
      ! Read the last safety backup
      !
      write(stdo,*) '****************************************'
      write(stdo,*) '    !   RESTARTING THE SIMULATION   !   '
      call flush(stdo)
!!!      call read_safety_backup_mctherm(ievenodd,params)
      write(stdo,*) 'ERRROR: Safety backup is not yet built in.'
      stop
      write(stdo,821) iphot
821   format('     !   AT PHOTON NR ',I9,'      !   ')
      !
      ! Now reinitialize the random number generator
      !
      if(params%irestart.eq.1) then 
         !
         ! Rerandomize using the iseed from the save file
         !
         write(stdo,*) '    ( ran2: randomizing using iseed )'
         write(stdo,*) '    (       from save file          )'
         if(iseeddum.gt.0) then
            isd=-iseeddum
         else
            isd=iseeddum
         endif
         aa=ran2(isd)
         iseed=isd
      elseif(params%irestart.eq.2) then
         !
         ! Rerandomize using iseed from montecarlo.inp
         !          
         write(stdo,*) '    ( ran2: randomizing using iseed )'
         write(stdo,*) '    (       from input file         )'
         if(iseed.gt.0) then
            isd=-iseed
         else
            isd=iseed
         endif
         aa=ran2(isd)
         iseed=isd
      elseif(params%irestart.eq.3) then
         !
         ! Use exactly the same random set as during last save
         ! (is already done, so do not need to do anything here)
         !
         write(stdo,*) '    ( ran2: restoring exact random  )'
         write(stdo,*) '    (       set as during save      )'
         iseed=iseeddum
      else
         write(stdo,*) 'ERROR: Restart option must be 1,2 or 3'
         stop 13
      endif
      write(stdo,*) '****************************************'
      ipstart=iphot
      cnt = 0
   endif
   !
   ! Launch all the photons
   !
   do iphot=ipstart,nphot
      !
      ! For the photon count routines:
      !
      iphotcurr = iphot
      !
      ! Message
      !
      if(count.ge.countwrite) then
         write(stdo,*) 'Photon nr ',iphot
         call flush(stdo)
         count=0
      endif
      !
      ! Safety dump?
      !
      if((cnt.ge.cntdump).and.(iphot.lt.nphot)) then
!!!         call make_safety_backup_mctherm(ievenodd,params)
         write(stdo,*) 'WARNING: Wanting to do safety backup, but this is'
         write(stdo,*) '         not yet implemented.'
         call flush(stdo)
         cnt=0
      endif
      !
      ! Call the walk routine
      !
      call walk_full_path_bjorkmanwood(params,ierror)
      !       
      ! If ierror.ne.0 then an error occurred, return with non-zero error code
      !
      if(ierror.ne.0) then
         return
      endif
      !
      ! Add photon to outgoing overall spectrum
      !
      if(.not.mc_photon_destroyed) then 
         mc_integerspec(ray_inu) = mc_integerspec(ray_inu) + 1
      endif
      !
      ! Update count
      !
      count = count + 1
      cnt   = cnt + 1
      !
      ! Write debugging stuff
      !
!      if(params%debug_write_eventcounts.ne.0) then
!         write(stdo,*) 'Nr of events = ',ieventcount
!      endif
   enddo
   !
   ! Close path file
   !
   if(params%debug_write_path.eq.1) then
      close(5)
   endif
   !
   ! If statistics writing active, close
   !
   if(params%debug_write_stats.ne.0) then
      close(4)
   endif
   !
   ! Write to the radmc_save.info that we're done
   !
   !open(unit=1,file='radmc_save.info')
   !write(1,*) '-'
   !close(1)
   !
   ! Compute the actual spectrum from the integer spectrum
   !
   do inu=1,freq_nr
      mc_average_spectrum(inu) = mc_integerspec(inu) * energy /       &
                              ( twopi * parsec**2 * freq_dnu(inu) )
   enddo
   !
   ! Compute the actual tdust
   !
   if(params%itempdecoup.eq.1) then
      !
      ! If temperatures are decoupled...
      !
      do icell=1,nrcells
         index = cellindex(icell)
         do ispec=1,dust_nr_species
            ener    = mc_cumulener(ispec,index) /                      &
                     ( dustdens(ispec,index) * cellvolume(index) )
            if(ener.gt.0.d0) then
               dusttemp(ispec,index) = compute_dusttemp_energy_bd(ener,ispec)
            else
               dusttemp(ispec,index) = 0.d0
            endif
         enddo
      enddo
   else
      !
      ! If temperatures are coupled...
      !
      do icell=1,nrcells
         index  = cellindex(icell)
         tempav = compute_dusttemp_coupled_bd(dust_nr_species,        &
                             mc_cumulener(:,index),                   &
                             dustdens(:,index),cellvolume(index))
         do ispec=1,dust_nr_species
            dusttemp(ispec,index) = tempav
         enddo
      enddo
   endif
   !
   ! Do a check
   !
   if(debug_check_all.eq.1) then
      do icell=1,nrcells
         index  = cellindex(icell)
         do ispec=1,dust_nr_species
            if(number_invalid(dusttemp(ispec,index)).ne.0) then
               if(number_invalid(dusttemp(ispec,index)).eq.1) then
                  write(stdo,*) 'ERROR: Discovered INF dust temperature at ', &
                       'ispec=',ispec,' index=',index
                  stop
               else
                  write(stdo,*) 'ERROR: Discovered NAN dust temperature at ', &
                       'ispec=',ispec,' index=',index
                  stop
               endif
            endif
         enddo
      enddo
   endif
   !
   ! Free the emission database
   !
   call free_emiss_dbase()
   !
   ! Done...
   !
end subroutine do_monte_carlo_bjorkmanwood


!--------------------------------------------------------------------------
!         MAIN ROUTINE FOR SINGLE-LAMBDA SCATTERING-ONLY MONTE CARLO
!
! This routine does a Monte Carlo simulation for scattering only. The
! absorption is indeed seen as extinction, i.e. photons are taken
! away. They are taken away in a continuous fashion, so that photon noise
! is limited as much as possible. Photon packages can be emitted from
! all fundamental photon sources (stars, external radiation etc), as well
! as from the dust (in contrast to the Bjorkman & Wood Monte Carlo above).
!
! --------------------------------------------------------------------------
subroutine do_monte_carlo_scattering(params,ierror,resetseed,scatsrc,meanint)
  implicit none
  type(mc_params) :: params
  integer :: ierror,inu
  doubleprecision :: ener,lumtotinv,temp,freq,fact
  logical :: ievenodd
  logical,optional :: resetseed
  logical,optional :: scatsrc,meanint
  logical :: compute_scatsrc,compute_meanint
  integer :: countwrite,cntdump,nphot,index
  integer :: iphot,count,ispec,istar,icell
  integer :: iseeddum,cnt,isd,itemplate,nsrc,nstarsrc
  !
  ! Some consistency checks
  ! 
  if(present(scatsrc)) then
     compute_scatsrc=scatsrc 
  else 
     compute_scatsrc=.false.
  endif
  if(present(meanint)) then
     compute_meanint=meanint
  else 
     compute_meanint=.false.
  endif
  if((.not.compute_scatsrc).and.(.not.compute_meanint)) then
     write(stdo,*) 'Monte Carlo Scattering: Must set scatsrc or meanint.'
     stop 3765
  endif
  if(compute_scatsrc.and.compute_meanint) then
     write(stdo,*) 'Monte Carlo Scattering: Cannot set BOTH scatsrc and meanint.'
     stop 3766
  endif
  if(nrcells.le.0) then
     write(stdo,*) 'ERROR: Spatial grid is not set.'
     stop
  endif
  if(params%countwrite.lt.1) then
     write(stdo,*) 'ERROR: Countwrite must be > 0'
     stop 
  endif
  if(params%cntdump.lt.params%countwrite) then
     write(stdo,*) 'ERROR: Cntdump must be >= ',params%countwrite
     stop 
  endif
  if(incl_quantum.ne.0) then
     write(stdo,*) 'ERROR: At the moment quantum heated grains are not included yet'
     stop
  endif
  if(.not.(allocated(dusttemp))) then
     write(stdo,*) 'INTERNAL ERROR: dusttemp arrays not initalized.'
     stop
  endif
  if((mc_nrfreq.le.0).or.(.not.allocated(mc_frequencies))) then
     write(stdo,*) 'ERROR: Cannot do scattering Monte Carlo without'
     write(stdo,*) '       MC frequency array set.'
     stop
  endif
  if(mc_scat_maxtauabs.le.0.d0) then
     write(stdo,*) 'ERROR: The mc_scat_maxtauabs is le 0, meaning that'
     write(stdo,*) '       the scattering Monte Carlo simulation (for images'
     write(stdo,*) '       and spectra) will go wrong... A reasonable value'
     write(stdo,*) '       would be 10.0 or 30.0 or so.'
     stop
  endif
  if(mc_scat_maxtauabs.lt.2.d0) then
     write(stdo,*) 'WARNING: The mc_scat_maxtauabs is lt 2. While this is'
     write(stdo,*) '       formally OK, we advise making this at least 10.'
     stop
  endif
  !
  ! Initialize the Monte Carlo module by allocating the required arrays
  ! and doing a number of standard checks
  !
  if(compute_scatsrc) then
     call montecarlo_init(params,ierror,101,resetseed)
  elseif(compute_meanint) then
     call montecarlo_init(params,ierror,102,resetseed)
  else
     stop 6658
  endif
  !
  ! Set some local integers and logicals
  !
  countwrite = params%countwrite
  cntdump    = params%cntdump
  if(compute_scatsrc) then
     nphot      = params%nphot_scat
  else
     nphot      = params%nphot_mono
  endif
  intplt     = params%iranfreqmode
  ieventcounttot = 0
  !
  ! If write statistics, then open this file
  !
  if(params%debug_write_stats.ne.0) then
     open(unit=4,file='statistics.out',status='unknown')
     write(4,*) freq_nr
     write(4,*)
  endif
  !
  ! Get the opacities (assume global opacities for the moment)
  !
  temp    = 100.d0     ! Arbitrary temperature...
  do inu=1,mc_nrfreq
     freq = mc_frequencies(inu)
     do ispec=1,dust_nr_species
        kappa_a(inu,ispec) = find_dust_kappa_interpol(freq,ispec,temp,1,0,0)
        kappa_s(inu,ispec) = find_dust_kappa_interpol(freq,ispec,temp,0,1,0)
        kappa_g(inu,ispec) = find_dust_kappa_interpol(freq,ispec,temp,0,0,1)
     enddo
  enddo
  !   
  ! Open the path file
  ! For debugging only!
  !
  if(params%debug_write_path.eq.1) then
     open(unit=5,file='path.dat',status='unknown')
  endif
  !
  ! Count set to 1
  !
  count = 1
  cnt = 1
  !
  ! Message
  !
  !write(stdo,*) ' '
  !write(stdo,*) 'Starting scattering Monte Carlo simulation....'
  !call flush(stdo)
  !
  ! Loop over all mc_frequencies
  !
  do inu=1,mc_nrfreq
     !
     ! Message
     !
     write(stdo,*) 'Wavelength nr ',inu,' corresponding to lambda=',  &
               1d4*cc/mc_frequencies(inu),' micron'
     call flush(stdo)
     !
     ! Determine the luminosity of the star(s) in all these wavelength bins
     !
     call montecarlo_compute_freqdep_luminosities(params,mc_frequencies(inu))
     !
     ! Compute the energy per photon package
     ! 
     if(.not.mc_weighted_photons) then
        !
        ! Standard mode: a single photon package energy. Different
        ! luminosities of the various sources are represented by different
        ! numbers of emitted packages.
        !
        ! The energy per photon package is Lnu divided by nr of photons.
        !
        energy = ( mc_thermemistot + stellarsrclumtot + emisquanttot +           &
                   starlumtot + extlumtot ) / nphot
        !
        ! Check 
        !
        if(energy.le.0.d0) then
           write(stdo,*) 'ERROR in setup: There exists no source of photons.'
           write(stdo,*) '      Must have either stars, viscous heating, external'
           write(stdo,*) '      heating or any other netto energy source.'
           write(stdo,*) '        E_stars      = ',starlumtot
           write(stdo,*) '        E_thermemis  = ',mc_thermemistot
           write(stdo,*) '        E_starsrc    = ',stellarsrclumtot
           write(stdo,*) '        E_quant      = ',emisquanttot
           write(stdo,*) '        E_extlum     = ',extlumtot
           stop
        endif
        !
        ! If there is mirror symmetry (a mode only possible for spherical 
        ! coordinates) then divide this by 2 as we only follow the upper
        ! half
        !
        if(igrid_mirror.eq.1) then
           energy = energy * 0.5d0
        endif
        !
        ! Compute the cumulative luminosities of the various different kinds of
        ! sources of photons
        !
        lumtotinv = 1.d0 / ( mc_thermemistot + emisquanttot + extlumtot +    &
                             stellarsrclumtot + starlumtot )
        mc_cumlum1  = ( mc_thermemistot ) * lumtotinv
        mc_cumlum2  = ( mc_thermemistot + emisquanttot ) * lumtotinv
        mc_cumlum3  = ( mc_thermemistot + emisquanttot  + extlumtot ) * lumtotinv
        mc_cumlum4  = ( mc_thermemistot + emisquanttot  + extlumtot + stellarsrclumtot ) * lumtotinv
     else
        !
        ! Weighted photon package mode. The different luminosities of the various
        ! sources are represented by different energies per package, but each
        ! source (if non-zero) will still have the same number of photon
        ! packages emitted. 
        ! 
        if(.not.allocated(mc_energy_stars).and.(nstars.gt.0)) stop 9211
        !
        ! Count the nr of sources
        !
        nsrc = 0
        if(mc_thermemistot.gt.0.d0) nsrc = nsrc + 1
        if(emisquanttot.gt.0.d0) nsrc = nsrc + 1
        if(extlumtot.gt.0.d0) nsrc = nsrc + 1
        if(stellarsrclumtot.gt.0.d0) nsrc = nsrc + 1
        nstarsrc = 0
        do istar=1,nstars
           if(star_lum(istar).gt.0.d0) then
              nsrc = nsrc + 1
              nstarsrc = nstarsrc + 1
           endif
        enddo
        if(nsrc.eq.0) then
           write(stdo,*) 'ERROR in setup: There exists no source of photons.'
           write(stdo,*) '      Must have either stars, thermal emission, external'
           write(stdo,*) '      heating or any other netto energy source.'
           stop
        endif
        !
        ! Take into account the mirroring, if active
        !
        if(igrid_mirror.eq.1) then
           fact = 0.5d0
        else
           fact = 1.d0
        endif
        !
        ! Compute the energy per package for each source
        !
        mc_energy_thermal    = fact * mc_thermemistot * nsrc / nphot
        mc_energy_quant      = fact * emisquanttot * nsrc / nphot
        mc_energy_extlum     = fact * extlumtot * nsrc / nphot
        mc_energy_stellarsrc = fact * stellarsrclumtot * nsrc / nphot
        do istar=1,nstars
           mc_energy_stars(istar) = star_fraclum(istar) * fact *       &
                                    star_lum(istar) * nsrc / nphot
        enddo
        !
        ! Compute the thesholds for the random assignment of packages
        ! to the various sources
        !
        if(mc_thermemistot.gt.0.d0) then
           mc_cumlum1  = 1.d0 / nsrc
        else
           mc_cumlum1  = 0.d0
        endif
        if(emisquanttot.gt.0.d0) then
           mc_cumlum2  = mc_cumlum1 + 1.d0 / nsrc
        else
           mc_cumlum2  = mc_cumlum1
        endif
        if(extlumtot.gt.0.d0) then
           mc_cumlum3  = mc_cumlum2 + 1.d0 / nsrc
        else
           mc_cumlum3  = mc_cumlum2
        endif
        if(stellarsrclumtot.gt.0.d0) then
           mc_cumlum4  = mc_cumlum3 + 1.d0 / nsrc
        else
           mc_cumlum4  = mc_cumlum3
        endif
        if(nstarsrc.gt.0.d0) then
           star_lumcum(1) = 0.d0
           if(star_lum(1).gt.0.d0) then
              star_lumcum(2) = 1.d0 / nstarsrc
           else
              star_lumcum(2) = 0.d0
           endif
           do istar=2,nstars
              if(star_lum(istar).gt.0.d0) then
                 star_lumcum(istar+1) = star_lumcum(istar) + 1.d0 / nstarsrc
              else
                 star_lumcum(istar+1) = star_lumcum(istar)
              endif
           enddo
           if(abs(star_lumcum(nstars+1)-1.d0).gt.1d-14) stop 7311
           star_lumcum(nstars+1) = 1.d0
        endif
     endif
     !
     ! Compute the energy limit for the Monte Carlo, i.e. when shall we
     ! throw away the photon and continue with the next one? 
     !
     mc_scat_energy_rellimit = exp(-mc_scat_maxtauabs)
     !
     ! Set ray_inu to inu
     !
     ray_inu = inu
     !
     ! Launch all the photons for this wavelength
     !
     do iphot=1,nphot
        !
        ! For the photon count routines:
        !
        iphotcurr = iphot
        !
        ! Message
        !
        if(count.ge.countwrite) then
           write(stdo,*) '   Photon nr ',iphot
           call flush(stdo)
           count=0
        endif
        !
        ! Safety dump?
        !
        if((cnt.ge.cntdump).and.(iphot.lt.nphot)) then
           write(stdo,*) 'WARNING: Wanting to do safety backup, but this is'
           write(stdo,*) '         not yet implemented.'
           call flush(stdo)
           cnt=0
        endif
        !
        ! Call the walk routine
        !
        call walk_full_path_scat(params,inu,ierror)
        !       
        ! If ierror.ne.0 then an error occurred, return with non-zero error code
        !
        if(ierror.ne.0) then
           return
        endif
        !
        ! Update count
        !
        count = count + 1
        cnt   = cnt + 1
        !
        ! Write debugging stuff
        !
        !      if(params%debug_write_eventcounts.ne.0) then
        !         write(stdo,*) 'Nr of events = ',ieventcount
        !      endif
     enddo
  enddo
  !
  ! If the optically very thick cells are skipped as sources of photons,
  ! then also make their scattering source functions equal to the planck
  ! function.
  !
!----TO-ADD----
! if(mc_scat_src_maxtau.lt.1d90) then
!    do icell=1,nrcells
!       index = cellindex(icell)
!       if(mc_scat_src_skip(index)) then
!          temp = dusttemp(1,index)  ! Take only first dust temp (== all others)
!          do inu=1,mc_nrfreq
!             mcscat_scatsrc(:,inu,index) = 0.d0
!             do ispec=1,dust_nr_species
!                 mcscat_scatsrc(:,inu,index) = mcscat_scatsrc(:,inu,index) + &
!                    dustdens(ispec,index) * kappa_s(inu,ispec)
!             enddo
!             mcscat_scatsrc(:,inu,index) = mcscat_scatsrc(:,inu,index) * &
!                         bplanck(temp,mc_frequencies(inu))
!          enddo
!       endif
!    enddo 
! endif
!----TO-ADD----
  !
  ! Close path file
  !
  if(params%debug_write_path.eq.1) then
     close(5)
  endif
  !
  ! If statistics writing active, close
  !
  if(params%debug_write_stats.ne.0) then
     close(4)
  endif
  !
  ! Write to the radmc_save.info that we're done
  !
  !open(unit=1,file='radmc_save.info')
  !write(1,*) '-'
  !close(1)
  !
  ! Done...
  !
end subroutine do_monte_carlo_scattering




!--------------------------------------------------------------------------
!         WALK THE ENTIRE RANDOM WALK UNTIL PHOTON LEAVES SYSTEM
!
!     This routine does the scatterings while keeping track of 
!     the energy lost by a package. It only stops after leaving 
!     the grid.
!--------------------------------------------------------------------------
subroutine walk_full_path_bjorkmanwood(params,ierror)
  implicit none
  type(mc_params) :: params
  integer :: ierror
  !
  double precision :: taupath,sint
  double precision :: dir(1:3),enold,albedo,dum,g,rn
  double precision :: pdirx,pdiry,pdirz,dirnewx,dirnewy,dirnewz
  integer :: ispec,iqactive,istar,icell,itemplate
  logical :: ok,arrived,therm,usesphere
!  type(amr_branch), pointer :: a
  !
  ! Do checks
  !
  if(debug_check_all.eq.1) then
     if((incl_quantum.eq.0).and.(emisquanttot.ne.0.d0)) then
        write(stdo,*) 'ERROR: Emisquant.ne.0 while quantum not switched on...'
        stop
     endif
     if((nstars.eq.0).and.(starlumtot.ne.0.d0)) then
        write(stdo,*) 'ERROR: Starlumtot.ne.0 while nstars==0...'
        stop
     endif
  endif
  !
  ! Reset
  !
  iqactive = 0
  mc_photon_destroyed = .false.
  !
  ! First choose whether to launch the photon from (one of) the star(s) or
  ! from the external radiation field or from inside the grid (due to the
  ! internally released energy) or from the locally emitted spectral
  ! source.. We make this choice on the basis of the total luminosity of all
  ! sources.
  !
  if(mc_cumlum4.gt.0.d0) then
     !
     ! There are more kinds of luminosity sources than just the star(s)
     !
     rn = ran2(iseed)
     !
  else
     !
     ! Only one source of luminosity: the star(s)
     ! By choosing rn=2 we avoid a costly random number generation and 
     ! we immediately go to the stellar emission
     !
     rn        = 2.d0
     !
  endif
  if(rn.ge.mc_cumlum4) then
     !
     ! ----- We have a photon emitted by a star -----
     !
     ! Switch on the quantum flag if quantum heating is on.
     !
     ! This is because in the quick quantum mode we treat stellar
     ! photons differently from dust emission photons. This quick 
     ! method does not include quantum heating by thermal emission.
     ! For PAHs, however, this is rarely the case, so this approximation
     ! is likely to be fine.
     !
     if(incl_quantum.ne.0) then
        if((incl_quantum.lt.0).and.(incl_quantum.gt.-10)) then
           iqactive = 1
        elseif(incl_quantum.eq.-10) then
           iqactive = -1
        elseif(incl_quantum.eq.-11) then
           iqactive = 2
        else
           iqactive = 0
        endif
     else
        iqactive = 0
     endif
     !
     ! Which star?
     !
     if(nstars.gt.1) then
        rn = ran2(iseed)
        call hunt(star_lumcum,nstars+1,rn,istar)
        if(istar.gt.nstars) stop
        if(istar.lt.1) stop
     else
        istar = 1
        if(nstars.eq.0) then
           write(stdo,*) 'INTERNAL ERROR with 0 stars in Monte Carlo module'
           stop
        endif
     endif
     !
     ! If we have the spherical star mode switched on, then 
     ! switch it on here as well
     !
     if(star_sphere) then
        usesphere = .true.
     else
        usesphere = .false.
     endif
     !
     ! If we have the weighted photons mode, we must do some
     ! extra stuff
     !
     if(mc_weighted_photons) then
        !
        ! Get the current energy of this photon package
        !
        energy = mc_energy_stars(istar)
        !
        ! If we have weighted photons mode, and a star is outside
        ! of the grid (more precise: outside of the containing sphere), 
        ! then we (at least for now) do not allow the treatment of the
        ! star as a sphere. NOTE: If the star lies inside the inner cavity
        ! of spherical coordinates, it is inside of the containing sphere,
        ! and it will be still treated as a sphere if that mode is on. This
        ! switching off of the sphere mode is only if the star is far away.
        !
        if(star_fraclum(istar).ne.1.d0) then
           usesphere = .false.
        endif
     endif
     !
     ! First choose a random theta at which the photon from the
     ! (spherically symmetric) star enters the inner edge of the
     ! coordinate system.
     !
     if(usesphere) then
        !
        ! Treat star as a sphere 
        !
        ! NOTE: With a finite stellar size, we also have to absorb photons
        !       that hit the star. 
        !
        ! NOTE: For now finite size stars are only allowed to be OUTSIDE
        !       the grid. Otherwise it becomes a nightmare.
        !
        ! Find a random point on the sphere
        !
        call montecarlo_randomdir(pdirx,pdiry,pdirz)
        ray_cart_x  = star_pos(1,istar) + pdirx*star_r(istar)*1.000000000001d0
        ray_cart_y  = star_pos(2,istar) + pdiry*star_r(istar)*1.000000000001d0
        ray_cart_z  = star_pos(3,istar) + pdirz*star_r(istar)*1.000000000001d0
        !
        ! If mirror symmetry, then enforce this here
        !
        if(igrid_mirror.eq.1) then
           if(ray_cart_z.lt.0.d0) then
              ray_cart_z = -ray_cart_z
           endif
        endif
        !
        ! NOTE: For now finite size stars are only allowed to be OUTSIDE
        !       the grid. Otherwise it becomes a nightmare.
        !
        ray_index = 0
        !
        ! Now determine a random direction for the photon. One must,
        ! however, be careful here because the direction is NOT simply a 3-D
        ! random direction with all inward directions rejected.  The thing
        ! is this: the intensity of a blackbody surface is the same at all
        ! inclinations. However, if one would simply use a normal random
        ! direction (using montecarlo_randomdir()) then one would see a
        ! brighter intensity as the surface is viewed under a more grazing
        ! inclination. This is incorrect.  So to compensate for this one
        ! must have the change of a photon being emitted in direction
        ! my=cos(theta) to be proportional to mu.
        !
        ! NOTE: I have been confused again, here, with this sqrt(rn). But it
        ! is indeed correct, as far as I know!!  The example that made me
        ! understand again why this is so is the example of a transport
        ! company with fast trucks, sending out per day 10 trucks, and a
        ! firm with slow trucks, also sending out per day 10 trucks. Both
        ! firms have the same flux of goods, even though firm2 has slow
        ! trucks. Firm2 just requires more trucks to keep the flux
        ! going. That is why photons with mu\simeq 0 (slow) would carry the
        ! same flux as photons with mu\simeq 1 (fast) if they would be
        ! emitted equally often. Since we know that mu\simeq 0 photons carry
        ! less flux, they must be emitted less often. Hence P(mu)=mu,
        ! i.e. Pcum(mu)=mu^2, or so to say: mu=sqrt(Pcum)=sqrt(rn).
        ! 
        rn            = ran2(iseed)
        ray_cart_dirx = sqrt(rn)
        dum           = twopi*ran2(iseed)
        rn            = sqrt(1.d0-rn)
        ray_cart_diry = rn*cos(dum)
        ray_cart_dirz = rn*sin(dum)
        call montecarlo_rotatevec(ray_cart_dirx,ray_cart_diry,ray_cart_dirz, &
                                  pdirx,pdiry,pdirz)
        !
     else
        !
        ! The star is assumed to be a point source
        !
        ray_cart_x  = star_pos(1,istar)
        ray_cart_y  = star_pos(2,istar)
        ray_cart_z  = star_pos(3,istar)
        !
        ! If mirror symmetry, then enforce this here
        ! NOTE: This should not be necessary: It should have already 
        !       been taken care of. But for now let's keep it.
        !
        if(igrid_mirror.eq.1) then
!           if(ray_cart_z.lt.0.d0) then
!              ray_cart_z = -ray_cart_z
!           endif
           if(ray_cart_z.ne.0.d0) then
              write(stdo,*) 'ERROR: With mirror symmetry, stars must lie on z=0 plane.'
              stop 732
           endif
        endif
        !
        ! Find random direction
        !
        if(star_fraclum(istar).eq.1.d0) then
           !
           ! Pure random direction in 4*pi
           !
           call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,  &
                                     ray_cart_dirz)
           !
           ! If we have mirror symmetry, then all stars MUST lie on the
           ! equatorial plane and their photons must point upward.
           !
           if(igrid_mirror.eq.1) then
              ray_cart_dirz = abs(ray_cart_dirz)
           endif
        else
           !
           ! Focused toward the model grid (the star lies outside the grid)
           !
           rn            = ran2(iseed)
           dum           = star_fraclum(istar)*rn
           ray_cart_dirx = 1.d0-2.d0*dum
           sint          = 2.d0*sqrt(dum-dum**2)
           dum           = twopi*ran2(iseed)
           ray_cart_diry = sint*cos(dum)
           ray_cart_dirz = sint*sin(dum)
           pdirx         = grid_contsph_x-star_pos(1,istar)
           pdiry         = grid_contsph_y-star_pos(2,istar)
           pdirz         = grid_contsph_z-star_pos(3,istar)
           dum           = 1.d0 / sqrt(pdirx**2+pdiry**2+pdirz**2)
           pdirx         = pdirx * dum
           pdiry         = pdiry * dum
           pdirz         = pdirz * dum
           call montecarlo_rotatevec(ray_cart_dirx,ray_cart_diry,ray_cart_dirz, &
                                     pdirx,pdiry,pdirz)
!#############################################################
!write(stdo,*) pdirx,pdiry,pdirz
!write(stdo,*) ray_cart_dirx,ray_cart_diry,ray_cart_dirz
!write(stdo,*) ray_cart_x,ray_cart_y,ray_cart_z
!stop
!#############################################################
        endif
        !
        ! Which cell is this star in, if it is in the grid?
        !
        ray_index = star_cellindex(istar)
        !
     endif
     !
     ! Find a random ray_inu for the photon package, based on the stellar
     ! flux
     !
     rn = ran2(iseed)
     call hunt(star_spec_cum(:,istar),freq_nr+1,rn,ray_inu)
     if(debug_check_all.eq.1) then
        if((ray_inu.lt.1).or.(ray_inu.gt.freq_nr)) then
           write(stdo,*) 'INTERNAL ERROR: ray_inu out of range for external radiation'
           stop
        endif
     endif
     !
  elseif(rn.ge.mc_cumlum3) then
     !
     ! ----- We have a photon emitted by the continuous stellar source -----
     !       (for simulations of galaxies and such)
     !
     ! First determine which cell to emit from
     !
     rn = ran2(iseed)
     call hunt(stellarsrc_lumcum,nrcells+1,rn,icell)
     ray_index = cellindex(icell)
     !
     ! Find the position within the cell from where to emit
     !
     call montecarlo_random_pos_in_cell(params,icell)
     !
     ! Find the direction in which to emit
     !
     call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
     !
     ! Now determine from which stellar population we will send the photon
     !
     if(stellarsrc_nrtemplates.eq.1) then
        itemplate = 1
     else
        dum = 0.d0
        do itemplate=1,stellarsrc_nrtemplates
           stellarsrc_cum(itemplate) = dum
           dum = dum + stellarsrc_dens(itemplate,ray_index) *           &
                       stellarsrc_templateslum(itemplate)
        enddo
        dum = 1.d0 / dum
        do itemplate=1,stellarsrc_nrtemplates
           stellarsrc_cum(itemplate) = stellarsrc_cum(itemplate) * dum
        enddo
        stellarsrc_cum(stellarsrc_nrtemplates+1) = 1.d0
        rn = ran2(iseed)
        call hunt(stellarsrc_cum,stellarsrc_nrtemplates+1,rn,itemplate)
        if(debug_check_all.eq.1) then
           if((itemplate.lt.1).or.(itemplate.gt.stellarsrc_nrtemplates)) then
              write(stdo,*) 'INTERNAL ERROR: Itemplate out of range'
              stop
           endif
        endif
     endif
     !
     ! Now determine which frequency we send the photon out
     !
     rn = ran2(iseed)
     call hunt(stellarsrc_speccum(:,itemplate),freq_nr+1,rn,ray_inu)
     if(debug_check_all.eq.1) then
        if((ray_inu.lt.1).or.(ray_inu.gt.freq_nr)) then
           write(stdo,*) 'INTERNAL ERROR: ray_inu out of range for continuous stellar source'
           stop
        endif
     endif
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_stellarsrc
     !
  elseif(rn.ge.mc_cumlum2) then
     !
     ! ----- We have a photon emitted by the external environment into -----
     !       our computational domain. This is done from a sphere 
     !       centered around the center of the coordinate system.
     !
     ray_index = 0
     !
     ! Find a random point on the sphere
     !
     call montecarlo_randomdir(pdirx,pdiry,pdirz)
     ray_cart_x  = grid_contsph_x + pdirx*grid_contsph_r
     ray_cart_y  = grid_contsph_y + pdiry*grid_contsph_r
     ray_cart_z  = grid_contsph_z + pdirz*grid_contsph_r
     !
     ! If mirror symmetry, then enforce this here
     !
     if(igrid_mirror.eq.1) then
        if(ray_cart_z.lt.0.d0) then
           ray_cart_z = -ray_cart_z
        endif
     endif
     !
     ! Now determine a random direction for the photon. Here we again
     ! have the angle difficulty we encountered for emission from a
     ! stellar surface. See above by sqrt(rn) is used.
     ! 
     rn            = ran2(iseed)
     ray_cart_dirx = sqrt(rn)
     dum           = twopi*ran2(iseed)
     rn            = sqrt(1.d0-rn)
     ray_cart_diry = rn*cos(dum)
     ray_cart_dirz = rn*sin(dum)
     call montecarlo_rotatevec(ray_cart_dirx,ray_cart_diry,ray_cart_dirz, &
                               -pdirx,-pdiry,-pdirz)
     !
     ! Now determine which frequency we send the photon out
     !
     rn = ran2(iseed)
     call hunt(extlum_intens_cum(:),freq_nr+1,rn,ray_inu)
     if(debug_check_all.eq.1) then
        if((ray_inu.lt.1).or.(ray_inu.gt.freq_nr)) then
           write(stdo,*) 'INTERNAL ERROR: ray_inu out of range for external radiation'
           stop
        endif
     endif
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_extlum
     !
  elseif(rn.ge.mc_cumlum1) then
     !
     ! ----- We have a photon emitted by quantum-heated grains -----
     !
     ! First determine which cell to emit from
     !
     rn = ran2(iseed)
     call hunt(emisquant_cum,nrcells+1,rn,icell)
     ray_index = cellindex(icell)
     !
     ! Find the position within the cell from where to emit
     !
     call montecarlo_random_pos_in_cell(params,icell)
     !
     ! Find the direction in which to emit
     !
     call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
     !
     ! Find the frequency at which to emit
     !
     rn = ran2(iseed)
     call hunt(emisquant_loccum(:,ray_index),freq_nr+1,rn,ray_inu)
     if(debug_check_all.eq.1) then
        if((ray_inu.lt.1).or.(ray_inu.gt.freq_nr)) then
           write(stdo,*) 'INTERNAL ERROR: ray_inu out of range for quantum source'
           stop
        endif
     endif
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_quant
     !
  else
     !
     ! ----- We have a photon emitted by internal heat production (e.g. -----
     !       viscous heating in an accretion disk)
     !
     ! First determine which cell to emit from
     !
     rn = ran2(iseed)
     call hunt(qvisc_cum,nrcells+1,rn,icell)
     ray_index = cellindex(icell)
     !
     ! Find the position within the cell from where to emit
     !
     call montecarlo_random_pos_in_cell(params,icell)
     !
     ! The photon will be emitted by thermal radiation, and will therefore
     ! also affect the local temperature accordingly.  This is
     ! mathematically identical to a discrete absorption and re-emission
     ! event. So therefore we call the discrete absorption-and-reemission
     ! event routine, even though this is an only-emission event. But that
     ! does not make a difference. Basically, the reason why the temperature
     ! is increased here is to ensure that the cell can indeed launch this 
     ! photon package.
     !
     ! NOTE: In the future it may be important to rethink the division of
     !       this primary energy over the dust species. I now put ray_inu=-1,
     !       which is a signal to do_asorption_event() to divide the energy
     !       by the density. This may not be correct since some species may
     !       be much more capable of reemitting energy than others. For now
     !       I simply divide it per unit dust mass because I have no better
     !       idea.
     !
     ray_inu  = -1    
     write(stdo,*) 'CHECK THIS. STOPPING FOR NOW.'
     stop
     iqactive = 0
     call do_absorption_event(params,iqactive,ierror)
     !
     ! Find the direction in which to emit
     !
     call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
     !
     ! The locally produced heat is usually radiated at 
     ! long wavelengths and thus will not excite PAHs.
     ! So switch off quantum (in old quantum mode).
     ! For new quantum mode, quantum is always active.
     !
     if((incl_quantum.lt.0).and.(incl_quantum.gt.-10)) then
        iqactive = 0
     elseif(incl_quantum.eq.-10) then
        iqactive = -1
     elseif(incl_quantum.eq.-11) then
        iqactive = 2
     else
        iqactive = 0
     endif
     mc_photon_destroyed = .false.
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_qvisc
     !
  endif
  !
  ! Now do some checks...
  !
  if(debug_check_all.eq.1) then
     if((ray_inu.lt.1).or.(ray_inu.gt.freq_nr)) then 
        write(stdo,*) 'INTERNAL ERROR: ray_inu.lt.1 or ray_inu.gt.mc_f: rayinu = ',ray_inu
        stop
     endif
     if((ray_index.lt.1).or.(ray_index.gt.nrcellsmax)) then
        write(stdo,*) 'INTERNAL ERROR: index out of range: ray_index = ',ray_index
        stop
     endif
     if(abs(ray_cart_dirx**2+ray_cart_diry**2+ray_cart_dirz**2-1.d0).gt.1d-14) then
        write(stdo,*) 'INTERNAL ERROR: dir vector not length unity: dir = ', &
               ray_cart_dirx,ray_cart_diry,ray_cart_dirz
        stop
     endif
  endif
  !
  ! Now associate pointer to cell.
  ! This is grid type dependent.
  !
  if(igrid_type.lt.100) then
     !
     ! AMR type grid
     !
     if(ray_index.gt.0) then
        !
        ! We are in a cell
        !
        if(amr_tree_present) then
           amrray_cell => amr_index_to_leaf(ray_index)%link
        else
           call amr_regular_get_ixyz(ray_index,amrray_ix_curr,amrray_iy_curr,amrray_iz_curr)
        endif
     else
        nullify(amrray_cell)
        amrray_ix_curr = -1
        amrray_iy_curr = -1
        amrray_iz_curr = -1
     endif
  else
     write(stdo,*) 'ERROR: This grid type not yet implemented'
     stop
  endif
  !
  ! Now decide: are we going to use the optimized motion method, which
  ! may be faster in high-optical depth regions? Or are we using the
  ! classical method? 
  !
  if(params%optimized_motion) then
     !
     ! .....................................................
     ! Use the optimized motion method. Faster, but riskier.
     ! .....................................................
     !
     ! Get the first random tau
     !
     taupath = -log(1.d0-ran2(iseed))
     !
     ! Now start the random walk
     !
     ok = .true.
     do while(ok)
        !
        ! Move photon to next scattering/absorption event
        ! 
        ! NOTE: If params%optimized_motion is set, then the walk_cells_routine
        !       may in fact also do sub-cell random walk, to speed things up.
        !
        ! NOTE: This routine will also automatically draw a new taupath
        !       using ran2, IFF not yet arrived.
        !
        call walk_cells_thermal_opt(params,taupath,iqactive,arrived,therm,ispec,ierror)
        !
        ! If arrived at end-point or escaped to infinity, then return
        !
        if(arrived) then
           return
        endif
        !
        ! If error, then also return
        !
        if(ierror.ne.0) then
           stop 1142
           return
        endif
        !
        ! If photon destroyed (by e.g. quantum heated grains which are
        ! dealt with elsewhere), then also return
        !
        if(mc_photon_destroyed) then
           return
        endif
        !
        ! Was it an absorption event or scattering event?
        !
        ! IMPORTANT NOTE for developers: If you want to add special
        ! thermal or scattering event types, please also add them
        ! in the walk_cells_thermal() in the place where the optimized
        ! random walk is done.
        !
        ! Note: If taupath is negative, then this is a signal that the
        !       optmized transfer of walk cells thermal already made
        !       a new taupath and already did the absorption or scattering
        !       in an attempt to do sub-cell propagation. Then of course
        !       don't redo this again here, otherwise the statistics is
        !       messed up.
        !
        if(taupath.gt.0.d0) then
           if(therm) then
              !
              ! Absorption event
              !
              call do_absorption_event(params,iqactive,ierror)
              call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
              !
              ! If photon destroyed (by e.g. quantum heated grains which are
              ! dealt with elsewhere), then also return
              !
              if(mc_photon_destroyed) then
                 return
              endif
              !
              ! Switch off quantum
              !
              ! ONLY for old quantum method
              !
              if(incl_quantum.gt.-10) then
                 iqactive = 0
              endif
              !
           else
              !
              ! Scattering event
              !
              if(scattering_mode.eq.1) then
                 !
                 ! Isotropic scattering
                 ! 
                 call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
                 !
              elseif(scattering_mode.eq.2) then
                 !
                 ! Anisotropic scattering off grain for dust species ispec
                 !
                 g = dust_gfactor(ray_inu,ispec)
                 call henyeygreenstein_randomdir(g,dirnewx,dirnewy,dirnewz)
                 call montecarlo_rotatevec(dirnewx,dirnewy,dirnewz,   &
                      ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
                 ray_cart_dirx = dirnewx
                 ray_cart_diry = dirnewy
                 ray_cart_dirz = dirnewz
                 !
              elseif(scattering_mode.eq.3) then
                 write(stdo,*) 'ERROR: Scattering with full phase function not yet implemented...'
                 stop
              else
                 if(scattering_mode.eq.0) then
                    write(stdo,*) 'ERROR in Monte Carlo: encountered a scattering event but '
                    write(stdo,*) '     the scattering mode is 0.'
                    stop
                 else
                    write(stdo,*) 'ERROR: scattering mode ',scattering_mode,' not known'
                    stop
                 endif
              endif
           endif
           !
           ! Draw a new taupath
           !
           taupath = -log(1.d0-ran2(iseed))
           !
        else
           !
           ! Apparently taupath was already done, so just make it positive
           !
           taupath = -taupath
           !
        endif
        !
        ! For debugging: Increase event counter
        !
        !!     ieventcount = ieventcount + 1
        ieventcounttot = ieventcounttot + 1
        !
        ! Next event
        !
     end do
     !
     ! ...end of optimized motion method
     !
  else
     !
     ! ...........................................
     ! Use classical (non-optimized motion) method
     ! ...........................................
     !
     ! Now start the random walk
     !
     ok = .true.
     do while(ok)
        !
        ! Get a random tau
        !
        taupath = -log(1.d0-ran2(iseed))
        !
        ! Move photon to next scattering/absorption event
        ! 
        ! ############ HERE YOU CAN IMPROVE IF X,Y,Z DEEP IN OPTICALLY THICK CELL
        ! The trick will be to get from walk_cells_thermal() a signal that you can 
        ! walk within a cell, giving the alpha values etc and the boundaries.
        ! Then this can be done without a special call and can be done 
        ! also in a special quick loop! This may speed up the code very much.
        ! For now, however, we will simply call the walk_cells_thermal all the time.
        !         
        call walk_cells_thermal(params,taupath,iqactive,arrived,therm,ispec,ierror)
        !
        ! If arrived at end-point or escaped to infinity, then return
        !
        if(arrived) then
           return
        endif
        !
        ! If error, then also return
        !
        if(ierror.ne.0) then
           stop 1142
           return
        endif
        !
        ! If photon destroyed (by e.g. quantum heated grains which are
        ! dealt with elsewhere), then also return
        !
        if(mc_photon_destroyed) then
           return
        endif
        !
        ! Was it an absorption event or scattering event?
        !
        if(therm) then
           !
           ! Absorption event
           !
           call do_absorption_event(params,iqactive,ierror)
           call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
           !
           ! If photon destroyed (by e.g. quantum heated grains which are
           ! dealt with elsewhere), then also return
           !
           if(mc_photon_destroyed) then
              return
           endif
           !
           ! Switch off quantum
           !
           ! ONLY for old quantum method
           !
           if(incl_quantum.gt.-10) then
              iqactive = 0
           endif
           !
        else
           !
           ! Scattering event
           !
           if(scattering_mode.eq.1) then
              !
              ! Isotropic scattering
              ! 
              call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
              !
           elseif(scattering_mode.eq.2) then
              !
              ! Anisotropic scattering off grain for dust species ispec
              !
              g = dust_gfactor(ray_inu,ispec)
              call henyeygreenstein_randomdir(g,dirnewx,dirnewy,dirnewz)
              call montecarlo_rotatevec(dirnewx,dirnewy,dirnewz,   &
                      ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
              ray_cart_dirx = dirnewx
              ray_cart_diry = dirnewy
              ray_cart_dirz = dirnewz
              !
           elseif(scattering_mode.eq.3) then
              write(stdo,*) 'ERROR: Scattering with full phase function not yet implemented...'
              stop
           else
              if(scattering_mode.eq.0) then
                 write(stdo,*) 'ERROR in Monte Carlo: encountered a scattering event but '
                 write(stdo,*) '     the scattering mode is 0.'
                 stop
              else
                 write(stdo,*) 'ERROR: scattering mode ',scattering_mode,' not known'
                 stop
              endif
           endif
        endif
        !
        ! For debugging: Increase event counter
        !
!!       ieventcount = ieventcount + 1
        ieventcounttot = ieventcounttot + 1
        !
        ! Next event
        !
     end do
     !
     ! ...end of classical (non-optimized motion) method
     !
  endif 
  !
end subroutine walk_full_path_bjorkmanwood


!--------------------------------------------------------------------------
!         WALK THE RANDOM WALK UNTIL PHOTON ENERGY IS LOW OR ESCAPED
!
!     This routine does the scatterings while keeping track of 
!     the energy lost by a package. It only stops after leaving 
!     the grid.
!--------------------------------------------------------------------------
subroutine walk_full_path_scat(params,inu,ierror)
  implicit none
  type(mc_params) :: params
  integer :: ierror,inu
  !
  double precision :: taupath,ener
  double precision :: dir(1:3),enold,albedo,dum,g,rn
  double precision :: pdirx,pdiry,pdirz,dirnewx,dirnewy,dirnewz
  integer :: ispec,iqactive,istar,icell,itemplate,sint
  logical :: ok,arrived,usesphere
!  type(amr_branch), pointer :: a
  !
  ! Do checks
  !
  if(debug_check_all.eq.1) then
     if((incl_quantum.eq.0).and.(emisquanttot.ne.0.d0)) then
        write(stdo,*) 'ERROR: Emisquant.ne.0 while quantum not switched on...'
        stop
     endif
     if((nstars.eq.0).and.(starlumtot.ne.0.d0)) then
        write(stdo,*) 'ERROR: Starlumtot.ne.0 while nstars==0...'
        stop
     endif
  endif
  !
  ! Reset
  !
  iqactive = 0
  mc_photon_destroyed = .false.
  ray_inu  = inu
  !
  ! First choose whether to launch the photon from (one of) the star(s) or
  ! from the external radiation field or from inside the grid (due to the
  ! internally released energy) or from the locally emitted spectral
  ! source.. We make this choice on the basis of the total luminosity of all
  ! sources.
  !
  if(mc_cumlum4.gt.0.d0) then
     !
     ! There are more kinds of luminosity sources than just the star(s)
     !
     rn = ran2(iseed)
     !
  else
     !
     ! Only one source of luminosity: the star(s)
     ! By choosing rn=2 we avoid a costly random number generation and 
     ! we immediately go to the stellar emission
     !
     rn        = 2.d0
     !
  endif
  if(rn.ge.mc_cumlum4) then
     !
     ! ----- We have a photon emitted by a star -----
     !
     ! Which star?
     !
     if(nstars.gt.1) then
        rn = ran2(iseed)
        call hunt(star_lumcum,nstars+1,rn,istar)
        if(istar.gt.nstars) stop
        if(istar.lt.1) stop
     else
        istar = 1
        if(nstars.eq.0) then
           write(stdo,*) 'INTERNAL ERROR with 0 stars in Monte Carlo module'
           stop
        endif
     endif
     !
     ! If we have the spherical star mode switched on, then 
     ! switch it on here as well
     !
     if(star_sphere) then
        usesphere = .true.
     else
        usesphere = .false.
     endif
     !
     ! If we have the weighted photons mode, we must do some
     ! extra stuff
     !
     if(mc_weighted_photons) then
        !
        ! Get the current energy of this photon package
        !
        energy = mc_energy_stars(istar)
        !
        ! If we have weighted photons mode, and a star is outside
        ! of the grid (more precise: outside of the containing sphere), 
        ! then we (at least for now) do not allow the treatment of the
        ! star as a sphere. NOTE: If the star lies inside the inner cavity
        ! of spherical coordinates, it is inside of the containing sphere,
        ! and it will be still treated as a sphere if that mode is on. This
        ! switching off of the sphere mode is only if the star is far away.
        !
        if(star_fraclum(istar).ne.1.d0) then
           usesphere = .false.
        endif
     endif
     !
     ! Choose a random theta at which the photon from the
     ! (spherically symmetric) star enters the inner edge of the
     ! coordinate system.
     !
     if(usesphere) then
        !
        ! Treat star as a sphere 
        !
        ! NOTE: With a finite stellar size, we also have to absorb photons
        !       that hit the star. 
        !
        ! NOTE: For now finite size stars are only allowed to be OUTSIDE
        !       the grid. Otherwise it becomes a nightmare.
        !
        ! Find a random point on the sphere
        !
        call montecarlo_randomdir(pdirx,pdiry,pdirz)
        ray_cart_x  = star_pos(1,istar) + pdirx*star_r(istar)*1.000000000001d0
        ray_cart_y  = star_pos(2,istar) + pdiry*star_r(istar)*1.000000000001d0
        ray_cart_z  = star_pos(3,istar) + pdirz*star_r(istar)*1.000000000001d0
        !
        ! If mirror symmetry, then enforce this here
        !
        if(igrid_mirror.eq.1) then
           if(ray_cart_z.lt.0.d0) then
              ray_cart_z = -ray_cart_z
           endif
        endif
        !
        ! NOTE: For now finite size stars are only allowed to be OUTSIDE
        !       the grid. Otherwise it becomes a nightmare.
        !
        ray_index = 0
        !
        ! Now determine a random direction for the photon. One must,
        ! however, be careful here because the direction is NOT simply a 3-D
        ! random direction with all inward directions rejected.  The thing
        ! is this: the intensity of a blackbody surface is the same at all
        ! inclinations. However, if one would simply use a normal random
        ! direction (using montecarlo_randomdir()) then one would see a
        ! brighter intensity as the surface is viewed under a more grazing
        ! inclination. This is incorrect.  So to compensate for this one
        ! must have the change of a photon being emitted in direction
        ! my=cos(theta) to be proportional to mu.
        !
        ! NOTE: I have been confused again, here, with this sqrt(rn). But it
        ! is indeed correct, as far as I know!!  The example that made me
        ! understand again why this is so is the example of a transport
        ! company with fast trucks, sending out per day 10 trucks, and a
        ! firm with slow trucks, also sending out per day 10 trucks. Both
        ! firms have the same flux of goods, even though firm2 has slow
        ! trucks. Firm2 just requires more trucks to keep the flux
        ! going. That is why photons with mu\simeq 0 (slow) would carry the
        ! same flux as photons with mu\simeq 1 (fast) if they would be
        ! emitted equally often. Since we know that mu\simeq 0 photons carry
        ! less flux, they must be emitted less often. Hence P(mu)=mu,
        ! i.e. Pcum(mu)=mu^2, or so to say: mu=sqrt(Pcum)=sqrt(rn).
        ! 
        rn            = ran2(iseed)
        ray_cart_dirx = sqrt(rn)
        dum           = twopi*ran2(iseed)
        rn            = sqrt(1.d0-rn)
        ray_cart_diry = rn*cos(dum)
        ray_cart_dirz = rn*sin(dum)
        call montecarlo_rotatevec(ray_cart_dirx,ray_cart_diry,ray_cart_dirz, &
                                  pdirx,pdiry,pdirz)
        !
     else
        !
        ! The star is assumed to be a point source
        !
        ray_cart_x  = star_pos(1,istar)
        ray_cart_y  = star_pos(2,istar)
        ray_cart_z  = star_pos(3,istar)
        !
        ! If mirror symmetry, then enforce this here
        ! NOTE: This should not be necessary: It should have already 
        !       been taken care of. But for now let's keep it.
        !
        if(igrid_mirror.eq.1) then
!           if(ray_cart_z.lt.0.d0) then
!              ray_cart_z = -ray_cart_z
!           endif
           if(ray_cart_z.ne.0.d0) then
              write(stdo,*) 'ERROR: With mirror symmetry, stars must lie on z=0 plane.'
              stop 732
           endif
        endif
        !
        ! Find random direction
        !
        if(star_fraclum(istar).eq.1.d0) then
           !
           ! Pure random direction in 4*pi
           !
           call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,  &
                                     ray_cart_dirz)
           !
           ! If we have mirror symmetry, then all stars MUST lie on the
           ! equatorial plane and their photons must point upward.
           !
           if(igrid_mirror.eq.1) then
              ray_cart_dirz = abs(ray_cart_dirz)
           endif
        else
           !
           ! Focused toward the model grid (the star lies outside the grid)
           !
           rn            = ran2(iseed)
           dum           = star_fraclum(istar)*rn
           ray_cart_dirx = 1.d0-2.d0*dum
           sint          = 2.d0*sqrt(dum-dum**2)
           dum           = twopi*ran2(iseed)
           ray_cart_diry = sint*cos(dum)
           ray_cart_dirz = sint*sin(dum)
           pdirx         = grid_contsph_x-star_pos(1,istar)
           pdiry         = grid_contsph_y-star_pos(2,istar)
           pdirz         = grid_contsph_z-star_pos(3,istar)
           dum           = 1.d0 / sqrt(pdirx**2+pdiry**2+pdirz**2)
           pdirx         = pdirx * dum
           pdiry         = pdiry * dum
           pdirz         = pdirz * dum
           call montecarlo_rotatevec(ray_cart_dirx,ray_cart_diry,ray_cart_dirz, &
                                     pdirx,pdiry,pdirz)
        endif
        !
        ! Which cell is this star in, if it is in the grid?
        !
        ray_index = star_cellindex(istar)
        !
     endif
     !
  elseif(rn.ge.mc_cumlum3) then
     !
     ! ----- We have a photon emitted by the continuous stellar source -----
     !       (for simulations of galaxies and such)
     !
     ! First determine which cell to emit from
     !
     rn = ran2(iseed)
     call hunt(stellarsrc_lumcum,nrcells+1,rn,icell)
     ray_index = cellindex(icell)
     !
     ! Find the position within the cell from where to emit
     !
     call montecarlo_random_pos_in_cell(params,icell)
     !
     ! Find the direction in which to emit
     !
     call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
     !
     ! Now determine from which stellar population we will send the photon
     !
     if(stellarsrc_nrtemplates.eq.1) then
        itemplate = 1
     else
        dum = 0.d0
        do itemplate=1,stellarsrc_nrtemplates
           stellarsrc_cum(itemplate) = dum
           dum = dum + stellarsrc_dens(itemplate,ray_index) *           &
                       mc_stellarsrc_templates(inu,itemplate)
        enddo
        dum = 1.d0 / dum
        do itemplate=1,stellarsrc_nrtemplates
           stellarsrc_cum(itemplate) = stellarsrc_cum(itemplate) * dum
        enddo
        stellarsrc_cum(stellarsrc_nrtemplates+1) = 1.d0
        rn = ran2(iseed)
        call hunt(stellarsrc_cum,stellarsrc_nrtemplates+1,rn,itemplate)
        if(debug_check_all.eq.1) then
           if((itemplate.lt.1).or.(itemplate.gt.stellarsrc_nrtemplates)) then
              write(stdo,*) 'INTERNAL ERROR: Itemplate out of range'
              stop
           endif
        endif
     endif
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_stellarsrc
     !
  elseif(rn.ge.mc_cumlum2) then
     !
     ! ----- We have a photon emitted by the external environment into -----
     !       our computational domain. This is done from a sphere 
     !       centered around the center of the coordinate system.
     !
     ray_index = 0
     !
     ! Find a random point on the sphere
     !
     call montecarlo_randomdir(pdirx,pdiry,pdirz)
     ray_cart_x  = grid_contsph_x + pdirx*grid_contsph_r
     ray_cart_y  = grid_contsph_y + pdiry*grid_contsph_r
     ray_cart_z  = grid_contsph_z + pdirz*grid_contsph_r
     !
     ! If mirror symmetry, then enforce this here
     !
     if(igrid_mirror.eq.1) then
        if(ray_cart_z.lt.0.d0) then
           ray_cart_z = -ray_cart_z
        endif
     endif
     !
     ! Now determine a random direction for the photon. Here we again
     ! have the angle difficulty we encountered for emission from a
     ! stellar surface. See above by sqrt(rn) is used.
     ! 
     rn            = ran2(iseed)
     ray_cart_dirx = sqrt(rn)
     dum           = twopi*ran2(iseed)
     rn            = sqrt(1.d0-rn)
     ray_cart_diry = rn*cos(dum)
     ray_cart_dirz = rn*sin(dum)
     call montecarlo_rotatevec(ray_cart_dirx,ray_cart_diry,ray_cart_dirz, &
                               -pdirx,-pdiry,-pdirz)
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_extlum
     !
  elseif(rn.ge.mc_cumlum1) then
     !
     ! ----- We have a photon emitted by quantum-heated grains -----
     !
     ! First determine which cell to emit from
     !
     rn = ran2(iseed)
     call hunt(emisquant_cum,nrcells+1,rn,icell)
     ray_index = cellindex(icell)
     !
     ! Find the position within the cell from where to emit
     !
     call montecarlo_random_pos_in_cell(params,icell)
     !
     ! Find the direction in which to emit
     !
     call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_quant
     !
  else
     !
     ! ----- We have a photon emitted by thermal emission -----
     !
     ! First determine which cell to emit from
     !
     rn = ran2(iseed)
     call hunt(mc_cumulthermemis,nrcells+1,rn,icell)
     ray_index = cellindex(icell)
     !
     ! Find the position within the cell from where to emit
     !
     call montecarlo_random_pos_in_cell(params,icell)
     !
     ! Find the direction in which to emit
     !
     call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
     !
     ! If weighted photon packages, then get the energy
     !
     if(mc_weighted_photons) energy = mc_energy_thermal
     !
  endif
  !
  ! Set the local energy to the starting value
  !
  ener = energy
  !
  ! Now do some checks...
  !
  if(debug_check_all.eq.1) then
     if((ray_index.lt.1).or.(ray_index.gt.nrcellsmax)) then
        write(stdo,*) 'INTERNAL ERROR: index out of range: ray_index = ',ray_index
        stop
     endif
     if(abs(ray_cart_dirx**2+ray_cart_diry**2+ray_cart_dirz**2-1.d0).gt.1d-14) then
        write(stdo,*) 'INTERNAL ERROR: dir vector not length unity: dir = ', &
               ray_cart_dirx,ray_cart_diry,ray_cart_dirz
        stop
     endif
  endif
  !
  ! Now associate pointer to cell.
  ! This is grid type dependent.
  !
  if(igrid_type.lt.100) then
     !
     ! AMR type grid
     !
     if(ray_index.gt.0) then
        !
        ! We are in a cell
        !
        if(amr_tree_present) then
           amrray_cell => amr_index_to_leaf(ray_index)%link
        else
           call amr_regular_get_ixyz(ray_index,amrray_ix_curr,amrray_iy_curr,amrray_iz_curr)
        endif
     else
        nullify(amrray_cell)
        amrray_ix_curr = -1
        amrray_iy_curr = -1
        amrray_iz_curr = -1
     endif
  else
     write(stdo,*) 'ERROR: This grid type not yet implemented'
     stop
  endif
  !
  ! Now start the random walk
  !
  ok = .true.
!!  ieventcount = 0         ! For debugging
  do while(ok)
     !
     ! Get a random tau, this time it is the scattering tau only
     !
     taupath = -log(1.d0-ran2(iseed))
     !
     ! Move photon to next scattering event
     ! 
     call walk_cells_scat(params,taupath,ener,inu,arrived,ispec,ierror)
     !
     ! If arrived at end-point or escaped to infinity, then return
     !
     if(arrived) then
        return
     endif
     !
     ! If error, then also return
     !
     if(ierror.ne.0) then
        stop 1142
        return
     endif
     !
     ! If photon destroyed (by e.g. quantum heated grains which are
     ! dealt with elsewhere), then also return. 
     !
     ! NOTE: In this MC mode a photon package is also destroyed if
     !       the package has extincted more than the threshold
     !       given by mc_scat_maxtauabs
     !
     if(mc_photon_destroyed) then
        return
     endif
     !
     ! Scattering event
     !
     if(scattering_mode.eq.1) then
        !
        ! Isotropic scattering
        ! 
        call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
        !
     elseif(scattering_mode.eq.2) then
        !
        ! Anisotropic scattering
        !
        ! The walk_cells_scat() routine already determined with which
        ! dust species the scattering took place: ispec.
        !
        g     = kappa_g(ray_inu,ispec)
        call henyeygreenstein_randomdir(g,dirnewx,dirnewy,dirnewz)
        call montecarlo_rotatevec(dirnewx,dirnewy,dirnewz,   &
             ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
        ray_cart_dirx = dirnewx
        ray_cart_diry = dirnewy
        ray_cart_dirz = dirnewz
        !
     elseif(scattering_mode.eq.3) then
        write(stdo,*) 'ERROR: Scattering with full phase function not yet implemented...'
        stop
     else
        write(stdo,*) 'ERROR: scattering mode ',scattering_mode,' not known'
        stop
     endif
     !
     ! For debugging: Increase event counter
     !
!!     ieventcount = ieventcount + 1
     ieventcounttot = ieventcounttot + 1
     !
     ! Next event
     !
  end do
  !
end subroutine walk_full_path_scat


!--------------------------------------------------------------------------
!       FOLLOW A PHOTON ALONG A RAY FOR THE THERMAL MONTE CARLO
!
!     This routine follows a photon until the next scattering
!     event. In the mean time it leaves its energy behind by
!     continuous absorption in the cells it crosses. 
!--------------------------------------------------------------------------
subroutine walk_cells_thermal(params,taupath,iqactive,arrived, & 
                              therm,ispecc,ierror)
  implicit none
  type(mc_params) :: params
  integer :: iqactive,ispec,ierror,ispecc,idir
  doubleprecision :: taupath,fr
  doubleprecision :: tau,dtau,albedo,absorb,dum,scatsrc0,costheta
  doubleprecision :: dexp,dener,ds,dummy,alpha_tot,g,rn
  logical :: ok,arrived,therm
  doubleprecision :: prev_x,prev_y,prev_z
  !
  ! Reset
  !
  tau      = 0.d0
  ierror   = 0
  !
  ! Default
  !
  mc_photon_destroyed = .false.
  arrived  = .false.
  !
  ! Pre-compute the phase functions for all dirs and species
  !
  ! *** NOTE: IF WE WISH TO INCLUDE LOCAL OBSERVERS, WE CANNOT PRE-COMPUTE THE SOURCE FUNCTION! ***
  !
  if((scattering_mode.gt.1).and.(params%incl_scatsrc_mctherm.eq.1)) then
     do idir=1,mcscat_nrdirs
        costheta = ray_cart_dirx*mcscat_dirs(1,idir) +              &
                   ray_cart_diry*mcscat_dirs(2,idir) +              &
                   ray_cart_dirz*mcscat_dirs(3,idir)
        do ispec=1,dust_nr_species
           g                            = kappa_g(ray_inu,ispec)
           mcscat_phasefunc(idir,ispec) = henyeygreenstein_phasefunc(g,costheta)
        enddo
     enddo
  endif
  !
  ! In this subroutine we will not make use of the amrray option to
  ! advance only partly within a cell. We will check here if we have
  ! an event within this cell and compute here the location of this
  ! event. Therefore the ray_dsend is set to 1d99.
  !
  ray_dsend = 1d99
  !
  ! Now do the loop
  !
  ok = .true.
  do while(ok)
     !
     ! For cell photon statistics, increase the counter
     !
     if(params%debug_write_stats.ne.0) then
        if(ray_index.ge.1) then
           if(mc_iphotcurr.gt.mc_ilastphot(ray_index)) then
              mc_iphotcount(ray_index) = mc_iphotcount(ray_index) + 1
              mc_ilastphot(ray_index)  = mc_iphotcurr
           endif
        endif
     endif
     !
     ! Back up the current position
     !
     prev_x     = ray_cart_x
     prev_y     = ray_cart_y
     prev_z     = ray_cart_z
     !
     ! Find new position
     !
     if(igrid_type.lt.100) then 
        !
        ! We use a regular or AMR grid
        !
        if(igrid_coord.lt.100) then
           !
           ! We use cartesian coordinates
           !
           call amrray_find_next_location_cart(ray_dsend,            &
                ray_cart_x,ray_cart_y,ray_cart_z,                    &
                ray_cart_dirx,ray_cart_diry,ray_cart_dirz,           &
                ray_index,ray_indexnext,ray_ds,arrived)
        elseif(igrid_coord.lt.200) then
           !
           ! We use spherical coordinates
           !
           call amrray_find_next_location_spher(ray_dsend,           &
                ray_cart_x,ray_cart_y,ray_cart_z,                    &
                ray_cart_dirx,ray_cart_diry,ray_cart_dirz,           &
                ray_index,ray_indexnext,ray_ds,arrived)
        else
           write(stdo,*) 'ERROR: Cylindrical coordinates not yet implemented'
           stop
        endif
        !
        ! In case stars are treated as spheres, copy this information
        ! Note that later we will check if the photon may have reached
        ! its end point (an absorption/scattering event) before reaching
        ! the sphere or a grid wall. In that case, mc_photon_destroyed
        ! will be reset to .false., see below.
        !
        if(amrray_ispherehit.lt.0) then
           mc_photon_destroyed = .true.
           arrived             = .true.    ! To end this segment and return
        else
           mc_photon_destroyed = .false.
        endif
        !
     else
        write(stdo,*) 'SORRY: Delaunay or Voronoi grids not yet implemented'
        stop
     endif
     !
     ! Path length
     ! 
     ! ######### CHECK: WHY NOT USE ray_ds ??? ########
     !
     ds   = sqrt( (ray_cart_x-prev_x)**2 + (ray_cart_y-prev_y)**2 + (ray_cart_z-prev_z)**2 )
     !
     ! Compute the alpha's, dtau, albedo and absorb
     !
     ! NOTE: For non-Cartesian coordinates it can happen that we have not
     !       yet arrived, but are currently not in a cell either. Hence 
     !       we need to check if ray_index.ge.1
     !
     if(ray_index.ge.1) then
        alpha_a_tot = 0.d0
        alpha_s_tot = 0.d0
        do ispec=1,dust_nr_species
           alpha_a(ispec) = dustdens(ispec,ray_index) * kappa_a(ray_inu,ispec)
           alpha_s(ispec) = dustdens(ispec,ray_index) * kappa_s(ray_inu,ispec)
           alpha_a_tot    = alpha_a_tot + alpha_a(ispec)
           alpha_s_tot    = alpha_s_tot + alpha_s(ispec)
        enddo
        alpha_tot = alpha_a_tot + alpha_s_tot
        albedo    = alpha_s_tot / alpha_tot
        dtau      = alpha_tot * ds
     else
        do ispec=1,dust_nr_species
           alpha_a(ispec) = 0.d0
           alpha_s(ispec) = 0.d0
        enddo
        alpha_a_tot = 0.d0
        alpha_s_tot = 0.d0
        alpha_tot   = 0.d0
        albedo      = 0.d0
        dtau        = 0.d0
     endif
     absorb = 1.d0 - albedo
     !
     ! Check if we already reached the end point
     !
     if(tau+dtau.ge.taupath) then
        !
        ! Yes: reached the end point, i.e. reached new scattering or
        ! absorption event
        !
        ! Check...
        !
        if(debug_check_all.eq.1) then
           if(ray_index.lt.1) stop 94001
           if(dtau.eq.0.d0) stop 94002
           if(alpha_a_tot.eq.0.d0) stop 94003
        endif
        !
        ! Compute the fraction of the segment we have advanced
        !
        fr     = (taupath-tau)/dtau
        !
        ! Add energy to cell
        !
        dum = absorb * (taupath-tau) * energy / alpha_a_tot
        if(iqactive.le.0) then
           do ispec=1,dust_nr_species
              mc_cumulener(ispec,ray_index) = mc_cumulener(ispec,ray_index) +  &
                   dum * alpha_a(ispec) 
           enddo
        else
           do ispec=1,dust_nr_species
              if(dust_quantum(ispec).eq.0) then 
                 mc_cumulener(ispec,ray_index) = mc_cumulener(ispec,ray_index) +  &
                      dum * alpha_a(ispec) 
              endif
           enddo
        endif
        !
        ! Add photons to scattering source
        !
        ! NOTE: In the original RADMC I did not divide by 4*pi yet; only
        !       in the writing to scatsource.dat I did that. Here I do 
        !       it immediately.
        !
        if(params%incl_scatsrc_mctherm.eq.1) then
           if(alpha_s_tot.gt.0.d0) then
              scatsrc0 = albedo * (taupath-tau) * energy /                        &
                     ( cellvolume(ray_index) * freq_dnu(ray_inu) * 12.566371d0 )
              if(scattering_mode.eq.1) then
                 !
                 ! Isotropic scattering: simply add the source term
                 !
                 mcscat_scatsrc(1,ray_inu,ray_index) =                            &
                      mcscat_scatsrc(1,ray_inu,ray_index) + scatsrc0
              elseif(scattering_mode.gt.1) then
                 !
                 ! Anisotropic scattering: add only for the given directions
                 !
                 do idir=1,mcscat_nrdirs
                    dum = 0.d0
                    do ispec=1,dust_nr_species
                       dum       = dum + mcscat_phasefunc(idir,ispec) *           &
                                         alpha_s(ispec) / alpha_s_tot
                    enddo
                    mcscat_scatsrc(idir,ray_inu,ray_index) =                      &
                         mcscat_scatsrc(idir,ray_inu,ray_index) + scatsrc0 * dum
                 enddo
              endif
           endif
        endif
        !
        ! Add photons to mean intensity of 
        ! primary (quantum-heating) photons
        !
        if(iqactive.ne.0) then
           miquant(ray_inu,ray_index) = miquant(ray_inu,ray_index) +               &
                   (taupath-tau) * energy /                                        &
                   ( cellvolume(ray_index) * freq_dnu(ray_inu) *                   &
                     alpha_tot * 12.566371d0 )
        endif
        !
        ! Compute the location of this point
        !
        ray_cart_x = prev_x + fr * ( ray_cart_x - prev_x )
        ray_cart_y = prev_y + fr * ( ray_cart_y - prev_y )
        ray_cart_z = prev_z + fr * ( ray_cart_z - prev_z )
        !
        ! Here we determine if it was an absorption or a scattering
        ! event. 
        !
        rn    = ran2(iseed)
        therm = (rn.ge.albedo)
        if(therm) then
           !
           ! It was absorption. We do not need to know ispecc. Put it to zero, for safety.
           !
           ispecc = 0
        else
           !
           ! It was a scattering event.
           !
           if(scattering_mode.gt.1) then
              !
              ! For anisotropic scattering we need to know off which dust species
              ! the photon has scattered. Determine this here, and call it ispecc.
              !
              alphacum(1) = 0.d0
              do ispec=1,dust_nr_species
                 alphacum(ispec+1) = alphacum(ispec) + alpha_s(ispec)
              enddo
              if(alphacum(dust_nr_species+1).le.0.d0) stop 2244
              do ispec=1,dust_nr_species
                 alphacum(ispec) = alphacum(ispec) / alphacum(dust_nr_species+1)
              enddo
              alphacum(dust_nr_species+1) = 1.d0
              rn = ran2(iseed)
              call hunt(alphacum,dust_nr_species+1,rn,ispecc)
              if((ispecc.lt.1).or.(ispecc.gt.dust_nr_species)) stop 7266
           else
              rn = ran2(iseed)   ! NOTE: This is not necessary, but is just
                                 !       there to pass the selftest. 
                                 !       See notes 15.11.09 in Radmc_3D_LOG.txt
              ispecc=0    ! Force error (with array bound check) if accidently used
           endif
        endif
        !
        ! Done with this segment of the random walk, so return and 
        ! handle the scattering or absorption event
        !
        arrived = .false.
        mc_photon_destroyed = .false.
        return
        !
     endif
     !
     ! Endpoint of this segment.
     !
     if(ray_index.ge.1) then
        !
        ! We are in a cell. So add energy to cell
        !
        dum = absorb * dtau * energy / alpha_a_tot
        do ispec=1,dust_nr_species
           if((dust_quantum(ispec).eq.0).or.(iqactive.le.0)) then 
              mc_cumulener(ispec,ray_index) =                          &
                   mc_cumulener(ispec,ray_index) + dum * alpha_a(ispec) 
           endif
        enddo
        !
        ! Add photons to scattering source
        !
        ! NOTE: In the original RADMC I did not divide by 4*pi yet; only
        !       in the writing to scatsource.dat I did that. Here I do 
        !       it immediately.
        !
        if((params%incl_scatsrc_mctherm.eq.1).and.(alpha_s_tot.gt.0.d0)) then
           scatsrc0 = albedo * dtau * energy /                                 &
                ( cellvolume(ray_index) * freq_dnu(ray_inu) * 12.566371d0 )
           if(scattering_mode.eq.1) then
              !
              ! Isotropic scattering: simply add the source term
              !
              mcscat_scatsrc(1,ray_inu,ray_index) =                            &
                   mcscat_scatsrc(1,ray_inu,ray_index) + scatsrc0
           elseif(scattering_mode.gt.1) then
              !
              ! Anisotropic scattering: add only for the given directions
              !
              do idir=1,mcscat_nrdirs
                 dum = 0.d0
                 do ispec=1,dust_nr_species
                    dum       = dum + mcscat_phasefunc(idir,ispec) *           &
                                      alpha_s(ispec) / alpha_s_tot
                 enddo
                 mcscat_scatsrc(idir,ray_inu,ray_index) =                      &
                      mcscat_scatsrc(idir,ray_inu,ray_index) + scatsrc0 * dum
              enddo
           endif
        endif
        !
        ! Add photons to mean intensity of 
        ! primary (quantum-heating) photons
        !             
        if(iqactive.ne.0) then
           miquant(ray_inu,ray_index) = miquant(ray_inu,ray_index) +   &
                   dtau * energy /                                     &
                   ( cellvolume(ray_index) * freq_dnu(ray_inu) *       &
                     alpha_tot * 12.566371d0 )
        endif
     endif
     !
     ! Increase the tau
     !
     tau = tau + dtau
     !
     ! Now make next cell the current cell
     !
     ray_index = ray_indexnext
     !
     ! Do the same for the pointers.
     ! This is grid type dependent.
     !
     if(igrid_type.lt.100) then
        !
        ! AMR type grid
        !
        if(ray_index.gt.0) then
           !
           ! We are in a cell
           !
           if(amr_tree_present) then
              amrray_cell => amr_index_to_leaf(ray_index)%link
           else
              call amr_regular_get_ixyz(ray_index,amrray_ix_curr,amrray_iy_curr,amrray_iz_curr)
           endif
        else
           nullify(amrray_cell)
           amrray_ix_curr = -1
           amrray_iy_curr = -1
           amrray_iz_curr = -1
        endif
     else
        write(stdo,*) 'ERROR: This grid type not yet implemented'
        stop
     endif
     !
     ! Now, if this new/next segment goes off to infinity, then
     ! we are done.
     !
     if(arrived) then
        !
        ! We escaped to infinity. 
        !
        return
     endif
     !
  enddo
  !
end subroutine walk_cells_thermal


!--------------------------------------------------------------------------
!       FOLLOW A PHOTON ALONG A RAY FOR THE THERMAL MONTE CARLO
!                 *** OPTIMIZED MOTION VERSION ***
!     This routine follows a photon until the next scattering
!     event. In the mean time it leaves its energy behind by
!     continuous absorption in the cells it crosses. 
!
!     NOTE: A subcell efficient MC is now built in, in case of small
!           steps. This may speed things up. If you dont want to use
!           it, then set params%optimized_motion=.false.
!--------------------------------------------------------------------------
subroutine walk_cells_thermal_opt(params,taupath,iqactive,arrived, & 
                              therm,ispecc,ierror)
  implicit none
  type(mc_params) :: params
  integer :: iqactive,ispec,ierror,ispecc,idir,iddr
  doubleprecision :: taupath,fr
  doubleprecision :: tau,dtau,albedo,absorb,dum,scatsrc0,costheta
  doubleprecision :: dexp,dener,ds,alpha_tot,g,rn,dsmin,sdone
  logical :: ok,arrived,therm
  doubleprecision :: prev_x,prev_y,prev_z,dirnewx,dirnewy,dirnewz
  doubleprecision :: axi(1:2,1:3)
  !
  ! Reset
  !
  tau      = 0.d0
  ierror   = 0
  !
  ! Default
  !
  mc_photon_destroyed = .false.
  arrived  = .false.
  !
  ! Pre-compute the phase functions for all dirs and species
  !
  ! *** NOTE: IF WE WISH TO INCLUDE LOCAL OBSERVERS, WE CANNOT PRE-COMPUTE THE SOURCE FUNCTION! ***
  !
  ! *** For now we do not allow the fast mode when incl_scatsrc_mctherm ***
  ! *** No particular reason for this, just that I am unsure if it works ***
  !
  if((scattering_mode.gt.1).and.(params%incl_scatsrc_mctherm.eq.1)) then
     do idir=1,mcscat_nrdirs
        costheta = ray_cart_dirx*mcscat_dirs(1,idir) +              &
                   ray_cart_diry*mcscat_dirs(2,idir) +              &
                   ray_cart_dirz*mcscat_dirs(3,idir)
        do ispec=1,dust_nr_species
           g                            = kappa_g(ray_inu,ispec)
           mcscat_phasefunc(idir,ispec) = henyeygreenstein_phasefunc(g,costheta)
        enddo
     enddo
  endif
  !
  ! In this subroutine we will not make use of the amrray option to
  ! advance only partly within a cell. We will check here if we have
  ! an event within this cell and compute here the location of this
  ! event. Therefore the ray_dsend is set to 1d99.
  !
  ray_dsend = 1d99
  !
  ! If we use optimized motion lateron, we need to make sure to reset
  ! the sdone variable, which counts the total amount of distance we
  ! travelled in the last cell of this path.
  !
  sdone = -1.d0
  !
  ! Now do the loop
  !
  ok = .true.
  do while(ok)
     !
     ! Do some safety checks
     !
     if(debug_check_all.eq.1) then
        if(ray_index.ge.1) then
           if(igrid_type.lt.100) then
              !
              ! AMR grid
              !
              ! Get the cell walls
              !
              if(amr_tree_present) then
                 if(.not.associated(amrray_cell)) stop 7270
                 do iddr=1,3
                    axi(1,iddr) = amr_finegrid_xi(amrray_cell%ixyzf(iddr),iddr,amrray_cell%level)
                    axi(2,iddr) = amr_finegrid_xi(amrray_cell%ixyzf(iddr)+1,iddr,amrray_cell%level)
                 enddo
              else
                 if(amrray_ix_curr.le.0) stop 7270
                 axi(1,1) = amr_finegrid_xi(amrray_ix_curr,1,0)
                 axi(2,1) = amr_finegrid_xi(amrray_ix_curr+1,1,0)
                 axi(1,2) = amr_finegrid_xi(amrray_iy_curr,2,0)
                 axi(2,2) = amr_finegrid_xi(amrray_iy_curr+1,2,0)
                 axi(1,3) = amr_finegrid_xi(amrray_iz_curr,3,0)
                 axi(2,3) = amr_finegrid_xi(amrray_iz_curr+1,3,0)
              endif
              !
              if(igrid_coord.lt.100) then
                 !
                 ! Cartesian coordinates
                 !
                 if((ray_cart_x.lt.axi(1,1)).or.(ray_cart_x.gt.axi(2,1)).or. &
                    (ray_cart_y.lt.axi(1,2)).or.(ray_cart_y.gt.axi(2,2)).or. &
                    (ray_cart_z.lt.axi(1,3)).or.(ray_cart_z.gt.axi(2,3))) then
                    write(stdo,*) 'INTERNAL ERROR: Point outside of cell in montecarlo module'
                    write(stdo,*) ray_cart_x,axi(1,1),axi(2,1)
                    write(stdo,*) ray_cart_y,axi(1,2),axi(2,2)
                    write(stdo,*) ray_cart_z,axi(1,3),axi(2,3)
                    stop
                 endif
              else
                 !
                 ! These coordiantes not yet active
                 !
                 write(stdo,*) 'INTERNAL ERROR: Non-Cart coordinates not yet implemented'
                 stop 
              endif
           else
              !
              ! Unstructured grids not yet active
              !
              write(stdo,*) 'INTERNAL ERROR: Unstructured grids not yet implemented'
              stop            
           endif
        endif
     endif
     !
     ! For cell photon statistics, increase the counter
     !
     if(params%debug_write_stats.ne.0) then
        if(ray_index.ge.1) then
           if(mc_iphotcurr.gt.mc_ilastphot(ray_index)) then
              mc_iphotcount(ray_index) = mc_iphotcount(ray_index) + 1
              mc_ilastphot(ray_index)  = mc_iphotcurr
           endif
        endif
     endif
     !
     ! Back up the current position
     !
     prev_x     = ray_cart_x
     prev_y     = ray_cart_y
     prev_z     = ray_cart_z
     !
     ! Find new position
     !
     if(igrid_type.lt.100) then 
        !
        ! We use a regular or AMR grid
        !
        if(igrid_coord.lt.100) then
           !
           ! We use cartesian coordinates
           !
           call amrray_find_next_location_cart(ray_dsend,            &
                ray_cart_x,ray_cart_y,ray_cart_z,                    &
                ray_cart_dirx,ray_cart_diry,ray_cart_dirz,           &
                ray_index,ray_indexnext,ray_ds,arrived,dsmin)
        elseif(igrid_coord.lt.200) then
           !
           ! We use spherical coordinates
           !
           call amrray_find_next_location_spher(ray_dsend,           &
                ray_cart_x,ray_cart_y,ray_cart_z,                    &
                ray_cart_dirx,ray_cart_diry,ray_cart_dirz,           &
                ray_index,ray_indexnext,ray_ds,arrived,dsmin)
        else
           write(stdo,*) 'ERROR: Cylindrical coordinates not yet implemented'
           stop
        endif
        !
        ! In case stars are treated as spheres, copy this information
        ! Note that later we will check if the photon may have reached
        ! its end point (an absorption/scattering event) before reaching
        ! the sphere or a grid wall. In that case, mc_photon_destroyed
        ! will be reset to .false., see below.
        !
        if(amrray_ispherehit.lt.0) then
           mc_photon_destroyed = .true.
           arrived             = .true.    ! To end this segment and return
        else
           mc_photon_destroyed = .false.
        endif
     else
        write(stdo,*) 'SORRY: Delaunay or Voronoi grids not yet implemented'
        stop
     endif
     !
     ! Path length
     ! 
     ! ######### CHECK: WHY NOT USE ray_ds ??? ########
     !
     ds   = sqrt( (ray_cart_x-prev_x)**2 + (ray_cart_y-prev_y)**2 + (ray_cart_z-prev_z)**2 )
     !
     ! Compute the alpha's, dtau, albedo and absorb
     !
     ! NOTE: For non-Cartesian coordinates it can happen that we have not
     !       yet arrived, but are currently not in a cell either. Hence 
     !       we need to check if ray_index.ge.1
     !
     if(ray_index.ge.1) then
        alpha_a_tot = 0.d0
        alpha_s_tot = 0.d0
        do ispec=1,dust_nr_species
           alpha_a(ispec) = dustdens(ispec,ray_index) * kappa_a(ray_inu,ispec)
           alpha_s(ispec) = dustdens(ispec,ray_index) * kappa_s(ray_inu,ispec)
           alpha_a_tot    = alpha_a_tot + alpha_a(ispec)
           alpha_s_tot    = alpha_s_tot + alpha_s(ispec)
        enddo
        alpha_tot = alpha_a_tot + alpha_s_tot
        albedo    = alpha_s_tot / alpha_tot
        dtau      = alpha_tot * ds
     else
        do ispec=1,dust_nr_species
           alpha_a(ispec) = 0.d0
           alpha_s(ispec) = 0.d0
        enddo
        alpha_a_tot = 0.d0
        alpha_s_tot = 0.d0
        alpha_tot   = 0.d0
        albedo      = 0.d0
        dtau        = 0.d0
     endif
     absorb = 1.d0 - albedo
     !
     ! Check if we already reached the end point
     !
     if(tau+dtau.ge.taupath) then
        !
        ! Yes: reached the end point, i.e. reached new scattering or
        ! absorption event
        !
        ! Here we can already determine in advance what kind of event it
        ! will be
        !
        rn    = ran2(iseed)
        therm = (rn.ge.albedo)
        !
        ! Check...
        !
        if(debug_check_all.eq.1) then
           if(ray_index.lt.1) stop 94001
           if(dtau.eq.0.d0) stop 94002
           if(alpha_a_tot.eq.0.d0) stop 94003
        endif
        !
        ! Compute the fraction of the segment we have advanced
        !
        fr     = (taupath-tau)/dtau
        !
        ! Compute the location of this point
        !
        ray_cart_x = prev_x + fr * ( ray_cart_x - prev_x )
        ray_cart_y = prev_y + fr * ( ray_cart_y - prev_y )
        ray_cart_z = prev_z + fr * ( ray_cart_z - prev_z )
        !
        ! Adjust ds
        !
        ds    = fr * ds
        !
        ! Entry point in case of optimized transfer
        !
200     continue
        !
        ! Add energy to cell
        !
        dum = absorb * (taupath-tau) * energy / alpha_a_tot
        if(iqactive.le.0) then
           do ispec=1,dust_nr_species
              mc_cumulener(ispec,ray_index) = mc_cumulener(ispec,ray_index) +  &
                   dum * alpha_a(ispec) 
           enddo
        else
           do ispec=1,dust_nr_species
              if(dust_quantum(ispec).eq.0) then 
                 mc_cumulener(ispec,ray_index) = mc_cumulener(ispec,ray_index) +  &
                      dum * alpha_a(ispec) 
              endif
           enddo
        endif
        !
        ! Add photons to scattering source
        !
        ! NOTE: In the original RADMC I did not divide by 4*pi yet; only
        !       in the writing to scatsource.dat I did that. Here I do 
        !       it immediately.
        !
        if(params%incl_scatsrc_mctherm.eq.1) then
           if(alpha_s_tot.gt.0.d0) then
              scatsrc0 = albedo * (taupath-tau) * energy /                        &
                     ( cellvolume(ray_index) * freq_dnu(ray_inu) * 12.566371d0 )
              if(scattering_mode.eq.1) then
                 !
                 ! Isotropic scattering: simply add the source term
                 !
                 mcscat_scatsrc(1,ray_inu,ray_index) =                            &
                      mcscat_scatsrc(1,ray_inu,ray_index) + scatsrc0
              elseif(scattering_mode.gt.1) then
                 !
                 ! Anisotropic scattering: add only for the given directions
                 !
                 do idir=1,mcscat_nrdirs
                    dum = 0.d0
                    do ispec=1,dust_nr_species
                       dum       = dum + mcscat_phasefunc(idir,ispec) *           &
                                         alpha_s(ispec) / alpha_s_tot
                    enddo
                    mcscat_scatsrc(idir,ray_inu,ray_index) =                      &
                         mcscat_scatsrc(idir,ray_inu,ray_index) + scatsrc0 * dum
                 enddo
              endif
           endif
        endif
        !
        ! Add photons to mean intensity of 
        ! primary (quantum-heating) photons
        !
        if(iqactive.ne.0) then
           miquant(ray_inu,ray_index) = miquant(ray_inu,ray_index) +               &
                   (taupath-tau) * energy /                                        &
                   ( cellvolume(ray_index) * freq_dnu(ray_inu) *                   &
                     alpha_tot * 12.566371d0 )
        endif
        !
        ! Determine off which grain we scatter, if it was a scattering
        ! event
        !
        if(therm) then
           !
           ! It was absorption. We do not need to know ispecc. Put it to zero, for safety.
           !
           ispecc = 0
        else
           !
           ! It was a scattering event.
           !
           if(scattering_mode.gt.1) then
              !
              ! For anisotropic scattering we need to know off which dust species
              ! the photon has scattered. Determine this here, and call it ispecc.
              !
              alphacum(1) = 0.d0
              do ispec=1,dust_nr_species
                 alphacum(ispec+1) = alphacum(ispec) + alpha_s(ispec)
              enddo
              if(alphacum(dust_nr_species+1).le.0.d0) stop 2244
              do ispec=1,dust_nr_species
                 alphacum(ispec) = alphacum(ispec) / alphacum(dust_nr_species+1)
              enddo
              alphacum(dust_nr_species+1) = 1.d0
              rn = ran2(iseed)
              call hunt(alphacum,dust_nr_species+1,rn,ispecc)
              if((ispecc.lt.1).or.(ispecc.gt.dust_nr_species)) stop 7266
           else
              ispecc=0    ! Force error (with array bound check) if accidently used
           endif
        endif
        !
        ! Now we have to decide whether we now proceed with optimized random
        ! walking within the cell, thus doing many scattering/absorption
        ! events at lower cost
        !
        if(params%optimized_motion) then
           !
           ! We check whether it makes sense to even try. This is to
           ! prevent loss of time through useless procedures if it is
           ! clear from the outset that it will not pay back.
           !
           if(ds*params%optim_dtau.lt.dsmin) then
!!!!           if(ds*params%optim_dtau.lt.dsmin+1d99) then   ! Always try...
              !
              ! Try optimized motion, that is: as long as we don't 
              ! move out of the cell, we don't have to do any costly
              ! calculations.
              !
              ! Note: we will not change dtau anymore, so the above
              ! condition will remain to be met until the end of the
              ! subroutine.
              !
              !
              ! Do the handling of the current discrete event, so that
              ! we know the new frequency of the photon, and we can,
              ! with a new taupath, calculate how far the next stretch
              ! will go.
              !
              ! NOTE: This is the same as in the caller-routine 
              !
              if(therm) then
                 !
                 ! Absorption event
                 !
                 call do_absorption_event(params,iqactive,ierror)
                 call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
                 !
                 ! If photon destroyed (by e.g. quantum heated grains which are
                 ! dealt with elsewhere), then also return
                 !
                 ! CHECK: Will this go right in this case of optmized motion?
                 !
                 if(mc_photon_destroyed) then
                    return
                 endif
                 !
                 ! Switch off quantum
                 !
                 ! ONLY for old quantum method
                 !
                 if(incl_quantum.gt.-10) then
                    iqactive = 0
                 endif
                 !
                 ! Recompute the alpha's, dtau, albedo and absorb
                 !
                 alpha_a_tot = 0.d0
                 alpha_s_tot = 0.d0
                 do ispec=1,dust_nr_species
                    alpha_a(ispec) = dustdens(ispec,ray_index) * kappa_a(ray_inu,ispec)
                    alpha_s(ispec) = dustdens(ispec,ray_index) * kappa_s(ray_inu,ispec)
                    alpha_a_tot    = alpha_a_tot + alpha_a(ispec)
                    alpha_s_tot    = alpha_s_tot + alpha_s(ispec)
                 enddo
                 alpha_tot = alpha_a_tot + alpha_s_tot
                 albedo    = alpha_s_tot / alpha_tot
                 absorb = 1.d0 - albedo
                 !
              else
                 !
                 ! Scattering event
                 !
                 if(scattering_mode.eq.1) then
                    !
                    ! Isotropic scattering
                    ! 
                    call montecarlo_randomdir(ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
                    !
                 elseif(scattering_mode.eq.2) then
                    !
                    ! Anisotropic scattering off grain for dust species ispecc
                    !
                    g = dust_gfactor(ray_inu,ispecc)
                    call henyeygreenstein_randomdir(g,dirnewx,dirnewy,dirnewz)
                    call montecarlo_rotatevec(dirnewx,dirnewy,dirnewz,   &
                         ray_cart_dirx,ray_cart_diry,ray_cart_dirz)
                    ray_cart_dirx = dirnewx
                    ray_cart_diry = dirnewy
                    ray_cart_dirz = dirnewz
                    !
                 elseif(scattering_mode.eq.3) then
                    write(stdo,*) 'ERROR: Scattering with full phase function not yet implemented...'
                    stop
                 else
                    if(scattering_mode.eq.0) then
                       write(stdo,*) 'ERROR in Monte Carlo: encountered a scattering event but '
                       write(stdo,*) '     the scattering mode is 0.'
                       stop
                    else
                       write(stdo,*) 'ERROR: scattering mode ',scattering_mode,' not known'
                       stop
                    endif
                 endif
              endif
              !
              ! Now reset or update the total path we move within this
              ! cell in the optimized transfer method.
              !
              if(sdone.lt.-0.5d0) then
                 !
                 ! This is the start of the optimized motion thing
                 !
                 sdone     = ds               ! First stretch already done
                 tau       = 0.d0
                 alpha_tot = alpha_tot+1d-99  ! For safety
                 dsmin     = dsmin*0.99d0     ! For safety
              else
                 !
                 ! Increase the sdone variable due to the previous
                 ! little stretch
                 !
                 sdone = sdone + ds 
              endif
              !
              ! Note: sdone is now the position along the random sub-cell
              !       walk we are doing. 
              !
              ! Now find out where we would end up next
              !
              taupath = -log(1.d0-ran2(iseed))
              ds      = taupath/alpha_tot
              !
              ! Check if this exceeds dsmin
              !
              if(sdone+ds.ge.dsmin) then
                 !
                 ! This exceeds dsmin, so don't do another step
                 !
                 taupath = -taupath      ! Just a signal to the calling routine
                 arrived = .false.
                 return
              endif
              !
              ! For debugging: Increase event counter
              !
              ieventcounttot = ieventcounttot + 1
              !
              !
              ! Compute the location of the next point
              !
              ray_cart_x = ray_cart_x + ds * ray_cart_dirx
              ray_cart_y = ray_cart_y + ds * ray_cart_diry
              ray_cart_z = ray_cart_z + ds * ray_cart_dirz
              !
              ! Determine whether the *next* event is an absorption 
              ! or scattering event
              !
              rn    = ran2(iseed)
              therm = (rn.ge.albedo)
              !
              ! With the new direction (and possibly also new freq)
              ! go back to start of the sub-cell random walk
              !
              goto 200
              !
           endif
        endif
        !
        ! Done with this segment of the random walk.
!        !
!        ! Draw a random taupath for the next stretch
!        !
!        taupath = -log(1.d0-ran2(iseed))
        !
        ! Now return and handle the scattering or absorption event
        !
        arrived = .false.
        return
        !
     endif
     !
     ! We reached the endpoint of this segment, i.e. we are now exactly 
     ! at the cell wall. This means we must enter the next cell and 
     ! continue our journey.
     !
     if(ray_index.ge.1) then
        !
        ! We are in a cell. So add energy to cell
        !
        dum = absorb * dtau * energy / alpha_a_tot
        do ispec=1,dust_nr_species
           if((dust_quantum(ispec).eq.0).or.(iqactive.le.0)) then 
              mc_cumulener(ispec,ray_index) =                          &
                   mc_cumulener(ispec,ray_index) + dum * alpha_a(ispec) 
           endif
        enddo
        !
        ! Add photons to scattering source
        !
        ! NOTE: In the original RADMC I did not divide by 4*pi yet; only
        !       in the writing to scatsource.dat I did that. Here I do 
        !       it immediately.
        !
        if((params%incl_scatsrc_mctherm.eq.1).and.(alpha_s_tot.gt.0.d0)) then
           scatsrc0 = albedo * dtau * energy /                                 &
                ( cellvolume(ray_index) * freq_dnu(ray_inu) * 12.566371d0 )
           if(scattering_mode.eq.1) then
              !
              ! Isotropic scattering: simply add the source term
              !
              mcscat_scatsrc(1,ray_inu,ray_index) =                            &
                   mcscat_scatsrc(1,ray_inu,ray_index) + scatsrc0
           elseif(scattering_mode.gt.1) then
              !
              ! Anisotropic scattering: add only for the given directions
              !
              do idir=1,mcscat_nrdirs
                 dum = 0.d0
                 do ispec=1,dust_nr_species
                    dum       = dum + mcscat_phasefunc(idir,ispec) *           &
                                      alpha_s(ispec) / alpha_s_tot
                 enddo
                 mcscat_scatsrc(idir,ray_inu,ray_index) =                      &
                      mcscat_scatsrc(idir,ray_inu,ray_index) + scatsrc0 * dum
              enddo
           endif
        endif
        !
        ! Add photons to mean intensity of 
        ! primary (quantum-heating) photons
        !             
        if(iqactive.ne.0) then
           miquant(ray_inu,ray_index) = miquant(ray_inu,ray_index) +   &
                   dtau * energy /                                     &
                   ( cellvolume(ray_index) * freq_dnu(ray_inu) *       &
                     alpha_tot * 12.566371d0 )
        endif
     endif
     !
     ! Increase the tau
     !
     tau = tau + dtau
     !
     ! Now make next cell the current cell
     !
     ray_index = ray_indexnext
     !
     ! Do the same for the pointers.
     ! This is grid type dependent.
     !
     if(igrid_type.lt.100) then
        !
        ! AMR type grid
        !
        if(ray_index.gt.0) then
           !
           ! We are in a cell
           !
           if(amr_tree_present) then
              amrray_cell => amr_index_to_leaf(ray_index)%link
           else
              call amr_regular_get_ixyz(ray_index,amrray_ix_curr,amrray_iy_curr,amrray_iz_curr)
           endif
        else
           nullify(amrray_cell)
           amrray_ix_curr = -1
           amrray_iy_curr = -1
           amrray_iz_curr = -1
        endif
     else
        write(stdo,*) 'ERROR: This grid type not yet implemented'
        stop
     endif
     !
     ! Now, if this new/next segment goes off to infinity, then
     ! we are done.
     !
     if(arrived) then
        !
        ! We escaped to infinity. 
        !
        return
     endif
     !
  enddo
  !
end subroutine walk_cells_thermal_opt


!--------------------------------------------------------------------------
!       FOLLOW A PHOTON ALONG A RAY FOR THE SCATTERING MONTE CARLO
!
!     This routine follows a photon until the next scattering
!     event. In the mean time it leaves its energy behind by
!     continuous absorption in the cells it crosses. 
!
! NOTE: inu in the argument list is the position in the mcscat_inus()
!       array. The 'real' inu is, of course, mcscat_inus(inu) or 
!       equivalently ray_inu.
!--------------------------------------------------------------------------
subroutine walk_cells_scat(params,taupath,ener,inu,arrived,ispecc,ierror)
  implicit none
  type(mc_params) :: params
  integer :: ispec,ierror,inu,ispecc,idir,iddr
  doubleprecision :: taupath,fr,ener,enerav
  doubleprecision :: tau,dtau,dum,dtauabs,dtauscat,xptauabs,xxtauabs
  doubleprecision :: ds,rn,scatsrc0,mnint
  logical :: ok,arrived
  doubleprecision :: prev_x,prev_y,prev_z
  doubleprecision :: costheta,g,phasefunc,dummy
  doubleprecision :: axi(1:2,1:3)
  !
  ! Reset
  !
  tau      = 0.d0
  ierror   = 0
  !
  ! Default
  !
  mc_photon_destroyed = .false.
  arrived  = .false.
  ray_inu  = inu
  !
  ! In this subroutine we will not make use of the amrray option to
  ! advance only partly within a cell. We will check here if we have
  ! an event within this cell and compute here the location of this
  ! event. Therefore the ray_dsend is set to 1d99.
  !
  ray_dsend = 1d99
  !
  ! If we have scattering included (sort of logical we have it included),
  ! then we can pre-compute the phase functions for all dust species.
  !
  ! Note: if we use local observer, then we cannot pre-compute the 
  !       phase function for non-isotropic scattering.
  !
  if(scattering_mode.gt.1) then
     if(.not.mcscat_localobserver) then
        do idir=1,mcscat_nrdirs
           costheta = ray_cart_dirx*mcscat_dirs(1,idir) +              &
                      ray_cart_diry*mcscat_dirs(2,idir) +              &
                      ray_cart_dirz*mcscat_dirs(3,idir)
           do ispec=1,dust_nr_species
              g                            = kappa_g(ray_inu,ispec)
              mcscat_phasefunc(idir,ispec) = henyeygreenstein_phasefunc(g,costheta)
           enddo
        enddo
     endif
  endif
  !
  ! Now do the loop
  !
  ok = .true.
  do while(ok)
     !
     ! Do some safety checks
     !
     if(debug_check_all.eq.1) then
        if(ray_index.ge.1) then
           if(igrid_type.lt.100) then
              !
              ! AMR grid
              !
              ! Get the cell walls
              !
              if(amr_tree_present) then
                 if(.not.associated(amrray_cell)) stop 7270
                 do iddr=1,3
                    axi(1,iddr) = amr_finegrid_xi(amrray_cell%ixyzf(iddr),iddr,amrray_cell%level)
                    axi(2,iddr) = amr_finegrid_xi(amrray_cell%ixyzf(iddr)+1,iddr,amrray_cell%level)
                 enddo
              else
                 if(amrray_ix_curr.le.0) stop 7270
                 axi(1,1) = amr_finegrid_xi(amrray_ix_curr,1,0)
                 axi(2,1) = amr_finegrid_xi(amrray_ix_curr+1,1,0)
                 axi(1,2) = amr_finegrid_xi(amrray_iy_curr,2,0)
                 axi(2,2) = amr_finegrid_xi(amrray_iy_curr+1,2,0)
                 axi(1,3) = amr_finegrid_xi(amrray_iz_curr,3,0)
                 axi(2,3) = amr_finegrid_xi(amrray_iz_curr+1,3,0)
              endif
              !
              if(igrid_coord.lt.100) then
                 !
                 ! Cartesian coordinates
                 !
                 if((ray_cart_x.lt.axi(1,1)).or.(ray_cart_x.gt.axi(2,1)).or. &
                    (ray_cart_y.lt.axi(1,2)).or.(ray_cart_y.gt.axi(2,2)).or. &
                    (ray_cart_z.lt.axi(1,3)).or.(ray_cart_z.gt.axi(2,3))) then
                    write(stdo,*) 'INTERNAL ERROR: Point outside of cell in montecarlo module'
                    write(stdo,*) ray_cart_x,axi(1,1),axi(2,1)
                    write(stdo,*) ray_cart_y,axi(1,2),axi(2,2)
                    write(stdo,*) ray_cart_z,axi(1,3),axi(2,3)
                    stop
                 endif
              else
                 !
                 ! These coordiantes not yet active
                 !
                 write(stdo,*) 'INTERNAL ERROR: Non-Cart coordinates not yet implemented'
                 stop 
              endif
           else
              !
              ! Unstructured grids not yet active
              !
              write(stdo,*) 'INTERNAL ERROR: Unstructured grids not yet implemented'
              stop            
           endif
        endif
     endif
     !
     ! For cell photon statistics, increase the counter
     !
     if(params%debug_write_stats.ne.0) then
        if(ray_index.ge.1) then
           if(mc_iphotcurr.gt.mc_ilastphot(ray_index)) then
              mc_iphotcount(ray_index) = mc_iphotcount(ray_index) + 1
              mc_ilastphot(ray_index)  = mc_iphotcurr
           endif
        endif
     endif
     !
     ! Back up the current position
     !
     prev_x     = ray_cart_x
     prev_y     = ray_cart_y
     prev_z     = ray_cart_z
     !
     ! Find new position
     !
     if(igrid_type.lt.100) then 
        !
        ! We use a regular or AMR grid
        !
        if(igrid_coord.lt.100) then
           !
           ! We use cartesian coordinates
           !
           call amrray_find_next_location_cart(ray_dsend,            &
                ray_cart_x,ray_cart_y,ray_cart_z,                    &
                ray_cart_dirx,ray_cart_diry,ray_cart_dirz,           &
                ray_index,ray_indexnext,ray_ds,arrived)
        elseif(igrid_coord.lt.200) then
           !
           ! We use spherical coordinates
           !
           call amrray_find_next_location_spher(ray_dsend,           &
                ray_cart_x,ray_cart_y,ray_cart_z,                    &
                ray_cart_dirx,ray_cart_diry,ray_cart_dirz,           &
                ray_index,ray_indexnext,ray_ds,arrived)
        else
           write(stdo,*) 'ERROR: Cylindrical coordinates not yet implemented'
           stop
        endif
        !
        ! In case stars are treated as spheres, copy this information
        ! Note that later we will check if the photon may have reached
        ! its end point (an absorption/scattering event) before reaching
        ! the sphere or a grid wall. In that case, mc_photon_destroyed
        ! will be reset to .false., see below.
        !
        if(amrray_ispherehit.lt.0) then
           mc_photon_destroyed = .true.
           arrived             = .true.    ! To end this segment and return
        else
           mc_photon_destroyed = .false.
        endif
     else
        write(stdo,*) 'SORRY: Delaunay or Voronoi grids not yet implemented'
        stop
     endif
     !
     ! Path length
     ! 
     ! ######### CHECK: WHY NOT USE ray_ds ??? ########
     !
     ds   = sqrt( (ray_cart_x-prev_x)**2 + (ray_cart_y-prev_y)**2 + (ray_cart_z-prev_z)**2 )
     !
     ! Compute the alpha's, dtau
     !
     ! NOTE: For non-Cartesian coordinates it can happen that we have not
     !       yet arrived, but are currently not in a cell either. Hence 
     !       we need to check if ray_index.ge.1
     !
     ! NOTE: Since we are here interested in Monte Carlo in terms of 
     !       scattering only (treating absorption simply as a loss term),
     !       we define dtau as only the scattering dtau.
     !
     ! NOTE: Since we must be able to use this subroutine also without 
     !       scattering, e.g. if we use radmc3d mcmono, we add 1d-99 to
     !       all opacities to make sure that they are non-zero.
     !
     if(ray_index.ge.1) then
        alpha_a_tot = 0.d0
        alpha_s_tot = 0.d0
        do ispec=1,dust_nr_species
           alpha_a(ispec) = dustdens(ispec,ray_index) * &
                            kappa_a(ray_inu,ispec) + 1d-99
           alpha_s(ispec) = dustdens(ispec,ray_index) * &
                            kappa_s(ray_inu,ispec) + 1d-99
           alpha_a_tot    = alpha_a_tot + alpha_a(ispec)
           alpha_s_tot    = alpha_s_tot + alpha_s(ispec)
        enddo
        dtau      = alpha_s_tot * ds
     else
        do ispec=1,dust_nr_species
           alpha_a(ispec) = 0.d0
           alpha_s(ispec) = 0.d0
        enddo
        alpha_a_tot = 0.d0
        alpha_s_tot = 0.d0
        dtau        = 0.d0
     endif
     !
     ! Check if we already reached the end point
     !
     if(tau+dtau.ge.taupath) then
        !
        ! Yes: reached the end point, i.e. reached new scattering event
        !
        ! Check...
        !
        if(debug_check_all.eq.1) then
           if(ray_index.lt.1) stop 94001
        endif
        !
        ! Compute dtauabs and dtauscat as well as exp(-dtauabs) and
        ! (1-exp(-dtauabs))/dtauabs.
        !
        dtauscat = taupath-tau
        dtauabs  = dtauscat * alpha_a_tot / alpha_s_tot
        xptauabs = exp(-dtauabs)
        if(dtauabs.gt.1d-6) then
           xxtauabs = (1.d0-xptauabs)/dtauabs
        else
           xxtauabs = 1.d0-0.5d0*dtauabs
        endif
        !
        ! Compute enerav
        !
        enerav  = ener * xxtauabs
        !
        ! Add photons to scattering source
        !
        ! NOTE: In the original RADMC I did not divide by 4*pi yet; only
        !       in the writing to scatsource.dat I did that. Here I do 
        !       it immediately.
        !
        ! Note: Since we follow only the scattering events, the tau
        !       and taupath are scattering tau only. So no `albedo'
        !       is necessary here.
        !
        ! Note: Since the energy of the photon package is peeled off 
        !       continuously by the absorption, we must use the average
        !       energy over this ray element, enerav.
        !
        if((alpha_s_tot.gt.0.d0).and.(allocated(mcscat_scatsrc))) then
           scatsrc0 = dtauscat * enerav / ( cellvolume(ray_index) * 12.566371d0 )
           if(scattering_mode.eq.1) then
              !
              ! Isotropic scattering: simply add the source term
              !
              mcscat_scatsrc(1,ray_inu,ray_index) =                            &
                      mcscat_scatsrc(1,ray_inu,ray_index) + scatsrc0
           elseif(scattering_mode.gt.1) then
              !
              ! Anisotropic scattering: add only for the given directions
              !
              if(.not.mcscat_localobserver) then
                 !
                 ! For the observer at infinity this is easy. We already
                 ! have the pre-calculated phase functions.
                 !
                 do idir=1,mcscat_nrdirs
                    dum = 0.d0
                    do ispec=1,dust_nr_species
                       dum       = dum + mcscat_phasefunc(idir,ispec) *           &
                                         alpha_s(ispec) / alpha_s_tot
                    enddo
                    mcscat_scatsrc(idir,ray_inu,ray_index) =                      &
                         mcscat_scatsrc(idir,ray_inu,ray_index) + scatsrc0 * dum
                 enddo
              else
                 !
                 ! For now we do not include local observer + anisotropic
                 ! scattering yet. What needs to be done here is to calculate
                 ! on-the-fly the direction toward the observer, and then
                 ! to calculate the phase function from this. 
                 !
                 write(stdo,*) 'ABORTING: For now, non-isotropic scattering and '
                 write(stdo,*) '          local observer are not yet compatible.'
                 stop
                 !
                 ! Here a preliminary attempt at this. But this is not
                 ! at all yet tested!! 
                 !
                 mcscat_dirs(1,1) = mcscat_localobs_pos(1) - ray_cart_x
                 mcscat_dirs(2,1) = mcscat_localobs_pos(2) - ray_cart_y
                 mcscat_dirs(3,1) = mcscat_localobs_pos(3) - ray_cart_z
                 dummy = 1.d0 / sqrt( mcscat_dirs(1,1)**2 + &
                         mcscat_dirs(2,1)**2 + mcscat_dirs(3,1)**2 )
                 mcscat_dirs(1,1) = mcscat_dirs(1,1) * dummy
                 mcscat_dirs(2,1) = mcscat_dirs(2,1) * dummy
                 mcscat_dirs(3,1) = mcscat_dirs(3,1) * dummy
                 costheta = ray_cart_dirx*mcscat_dirs(1,1) +          &
                            ray_cart_diry*mcscat_dirs(2,1) +          &
                            ray_cart_dirz*mcscat_dirs(3,1)
                 do ispec=1,dust_nr_species
                    g                         = kappa_g(ray_inu,ispec)
                    mcscat_phasefunc(1,ispec) = henyeygreenstein_phasefunc(g,costheta)
                 enddo
                 dum = 0.d0
                 do ispec=1,dust_nr_species
                    dum       = dum + mcscat_phasefunc(1,ispec) *          &
                                      alpha_s(ispec) / alpha_s_tot
                 enddo
                 mcscat_scatsrc(1,ray_inu,ray_index) =                     &
                      mcscat_scatsrc(1,ray_inu,ray_index) + scatsrc0 * dum
              endif
           endif
        endif
        !
        ! Add photons to mean intensity
        !
        ! Note: Since the energy of the photon package is peeled off 
        !       continuously by the absorption, we must use the average
        !       energy over this ray element, enerav.
        !
        ! Note: Since we only stop in the middle of a cell if we have
        !       non-zero scattering opacity, we can compute the mean
        !       intensity via the scattering. 
        !
        if(allocated(mcscat_meanint)) then
           scatsrc0 = dtauscat * enerav / ( cellvolume(ray_index) * 12.566371d0 )
           mcscat_meanint(ray_inu,ray_index) =                            &
                mcscat_meanint(ray_inu,ray_index) + scatsrc0 / alpha_s_tot
        endif
        !
        ! Compute the location of this point
        !
        fr     = (taupath-tau)/dtau
        ray_cart_x = prev_x + fr * ( ray_cart_x - prev_x )
        ray_cart_y = prev_y + fr * ( ray_cart_y - prev_y )
        ray_cart_z = prev_z + fr * ( ray_cart_z - prev_z )
        !
        ! For anisotropic scattering we need to know off which dust species
        ! the photon has scattered. Determine this here, and call it ispecc.
        !
        if(scattering_mode.gt.1) then
           alphacum(1) = 0.d0
           do ispec=1,dust_nr_species
              alphacum(ispec+1) = alphacum(ispec) + alpha_s(ispec)
           enddo
           if(alphacum(dust_nr_species+1).le.0.d0) stop 2244
           do ispec=1,dust_nr_species
              alphacum(ispec) = alphacum(ispec) / alphacum(dust_nr_species+1)
           enddo
           alphacum(dust_nr_species+1) = 1.d0
           rn = ran2(iseed)
           call hunt(alphacum,dust_nr_species+1,rn,ispecc)
           if((ispecc.lt.1).or.(ispecc.gt.dust_nr_species)) stop 7266
        else
           rn = ran2(iseed)   ! NOTE: This is not necessary, but is just
                              !       there to pass the selftest. 
                              !       See notes 15.11.09 in Radmc_3D_LOG.txt
           ispecc=0    ! Force error (with array bound check) if accidently used
        endif
        !
        ! Compute the new ener
        !  
        ener    = ener * xptauabs
        !
        ! Check if we drop below the "throw photon away" limit
        !
        ! **** NOTE: I should also set mc_photon_destroyed = .true. ****
        ! ****       but it does not matter right now.              ****
        !
        if(ener.le.mc_scat_energy_rellimit*energy) then
           arrived = .true.
        else
           arrived = .false.
        endif
        !
        ! Done with this segment of the random walk, so return and 
        ! handle the scattering or absorption event
        !
        return
        !
     endif
     !
     ! Endpoint of this segment.
     !
     if(ray_index.ge.1) then
        !
        ! We are in a cell. 
        !
        ! Compute dtauabs and dtauscat as well as exp(-dtauabs) and
        ! (1-exp(-dtauabs))/dtauabs.
        !
        ! MAJOR BUGFIX 18.01.10:
        !
        dtauscat = dtau
        dtauabs  = dtauscat * alpha_a_tot / alpha_s_tot
        xptauabs = exp(-dtauabs)
        if(dtauabs.gt.1d-6) then
           xxtauabs = (1.d0-xptauabs)/dtauabs
        else
           xxtauabs = 1.d0-0.5d0*dtauabs
        endif
        !
        ! Compute enerav
        !
        enerav  = ener * xxtauabs
        !
        ! Add photons to scattering source
        !
        ! NOTE: In the original RADMC I did not divide by 4*pi yet; only
        !       in the writing to scatsource.dat I did that. Here I do 
        !       it immediately.
        !
        ! Note: Since we follow only the scattering events, the tau
        !       and taupath are scattering tau only. So no `albedo'
        !       is necessary here.
        !
        ! Note: Since the energy of the photon package is peeled off 
        !       continuously by the absorption, we must use the average
        !       energy over this ray element, enerav.
        !
        if(alpha_s_tot.gt.0.d0) then
           scatsrc0 = dtau * enerav / ( cellvolume(ray_index) * 12.566371d0 )
           if(scattering_mode.eq.1) then
              !
              ! Isotropic scattering: simply add the source term
              !
              mcscat_scatsrc(1,ray_inu,ray_index) =                            &
                      mcscat_scatsrc(1,ray_inu,ray_index) + scatsrc0
           elseif(scattering_mode.gt.1) then
              !
              ! Anisotropic scattering: add only for the given directions
              !
              do idir=1,mcscat_nrdirs
                 dum = 0.d0
                 do ispec=1,dust_nr_species
                    dum       = dum + mcscat_phasefunc(idir,ispec) *           &
                                      alpha_s(ispec) / alpha_s_tot
                 enddo
                 mcscat_scatsrc(idir,ray_inu,ray_index) =                      &
                      mcscat_scatsrc(idir,ray_inu,ray_index) + scatsrc0 *dum
              enddo
           endif
        endif
        !
        ! Add photons to mean intensity
        !
        ! Note: Since the energy of the photon package is peeled off 
        !       continuously by the absorption, we must use the average
        !       energy over this ray element, enerav.
        !
        if(allocated(mcscat_meanint)) then
           mnint = ds * enerav / ( cellvolume(ray_index) * 12.566371d0 )
           mcscat_meanint(ray_inu,ray_index) =                            &
                mcscat_meanint(ray_inu,ray_index) + mnint
        endif
        !
        ! Compute the new ener and the new tauabs
        !  
        ener = ener * xptauabs
        !
        ! Check if we drop below the "throw photon away" limit
        !
        ! **** NOTE: I should also set mc_photon_destroyed = .true. ****
        ! ****       but it does not matter right now.              ****
        !
        if(ener.le.mc_scat_energy_rellimit*energy) then
           arrived = .true.
        endif
        !
     endif
     !
     ! Increase the tau
     !
     tau = tau + dtau
     !
     ! Now make next cell the current cell
     !
     ray_index = ray_indexnext
     !
     ! Do the same for the pointers.
     ! This is grid type dependent.
     !
     if(igrid_type.lt.100) then
        !
        ! AMR type grid
        !
        if(ray_index.gt.0) then
           !
           ! We are in a cell
           !
           if(amr_tree_present) then
              amrray_cell => amr_index_to_leaf(ray_index)%link
           else
              call amr_regular_get_ixyz(ray_index,amrray_ix_curr,amrray_iy_curr,amrray_iz_curr)
           endif
        else
           nullify(amrray_cell)
           amrray_ix_curr = -1
           amrray_iy_curr = -1
           amrray_iz_curr = -1
        endif
     else
        write(stdo,*) 'ERROR: This grid type not yet implemented'
        stop
     endif
     !
     ! Now, if this new/next segment goes off to infinity, then
     ! we are done.
     !
     if(arrived) then
        !
        ! We escaped to infinity. 
        !
        return
     endif
     !
  enddo
  !
end subroutine walk_cells_scat



!-------------------------------------------------------------------
!                DO OVERALL CHECK ON INPUT DATA
!-------------------------------------------------------------------
subroutine mc_check_inputdata
  
  implicit none
  integer :: index,icell,ispec
  !
  ! Check the dust density
  !
  if(allocated(dustdens)) then
     do ispec=1,dust_nr_species
        do icell=1,nrcells
           index = cellindex(icell)
           if((index.le.0).or.(index.gt.nrcellsmax)) then
              write(stdo,*) 'ERROR: Internal error while checking dust density'
              stop
           endif
           if(number_invalid(dustdens(ispec,index)).ne.0) then
              write(stdo,*) 'ERROR: Dust density is NAN or INF at some point!'
              stop
           endif
           if(dustdens(ispec,index).lt.0.d0) then
              write(stdo,*) 'ERROR: Dust density is negative at some point!'
              stop
           endif
           if(dustdens(ispec,index).lt.1d-99) then
              dustdens(ispec,index) = 1d-99
           endif
        enddo
     enddo
  endif
  !
  ! Check the dust temperature
  !
  if(allocated(dusttemp)) then
     do ispec=1,dust_nr_species
        do icell=1,nrcells
           index = cellindex(icell)
           if((index.le.0).or.(index.gt.nrcellsmax)) then
              write(stdo,*) 'ERROR: Internal error while checking dust temperature'
              stop
           endif
           if(number_invalid(dusttemp(ispec,index)).ne.0) then
              write(stdo,*) 'ERROR: Dust temperature is NAN or INF at some point!'
              stop
           endif
           if(dusttemp(ispec,index).lt.0.d0) then
              write(stdo,*) 'ERROR: Dust temperature is negative at some point!'
              stop
           endif
        enddo
     enddo
  endif
end subroutine mc_check_inputdata



!-------------------------------------------------------------------
!                         DO ABSORPTION EVENT
!
!     This subroutine increases the temperature of a cell according
!     to the mass it contains. And it randomly creates a new inucur.
!-------------------------------------------------------------------
subroutine do_absorption_event(params,iqactive,ierror)
  implicit none
  type(mc_params) :: params
  integer :: iqactive,ierror
  !
  integer :: ispec
  doubleprecision :: cumen,tempav,rn,fracquant,alpha_a_nu
  doubleprecision :: dum,dum1,dum2,rhodusttot
  logical :: calctemp
  !
  if(freq_dnu(1).eq.0.d0) stop 32222
  !
  ! Default
  !
  mc_photon_destroyed = .false.
  !
  ! First determine how the energy is going to be divided over
  ! the dust species and sizes
  !
  if(ray_inu.ge.1) then 
     !
     ! The normal way: dependent on frequency of incoming photon
     !
     if(dust_nr_species.eq.1) then
        !
        ! If there is only a single dust species: then we do not need to
        ! do any effort
        !
        mc_enerpart(1) = energy
        !
     else
        !
        ! If there are multiple dust species, then we need to figure out
        ! how to divide the absorbed energy over the various dust species.
        ! This depends on the frequency of the photon.
        !
        alpha_a_nu = 0.d0
        do ispec=1,dust_nr_species
           mc_enerpart(ispec) = dustdens(ispec,ray_index) *                &
                                kappa_a(ray_inu,ispec) 
           alpha_a_nu         = alpha_a_nu + mc_enerpart(ispec)
        enddo
        do ispec=1,dust_nr_species
           mc_enerpart(ispec) = energy * mc_enerpart(ispec) / alpha_a_nu
        enddo
        !
     endif
     !
     ! If quantum heating active:
     ! Original method:
     !    If the photon is absorbed by a quantum heated grain, then 
     !    destroy photon (because the quantum heating is done elsewhere)
     ! New method (NEW QUANTUM: 28.04.06):
     !    The photon is re-emitted according to the quantum source
     !
     if(iqactive.eq.1) then
        !
        ! ORIGINAL METHOD
        !
        ! Determine what is the chance that the photon is absorbed
        ! by a quantum-heated grain
        !
        fracquant = 0.d0
        do ispec=1,dust_nr_species
           if(dust_quantum(ispec).ne.0) then
              fracquant = fracquant + mc_enerpart(ispec)
           endif
        enddo
        fracquant = fracquant / energy
        !
        ! SMALL BUGFIX (06.07.05): 1.d0 --> 1.d0+1d-10
        !
        if(fracquant.gt.1.d0+MINIEPS) stop 92091
        !
        ! Now pick a random number and determine what happens.
        ! If absorbed by a quantum heated grain, then destroy the
        ! photon.
        !     
        rn = ran2(iseed)
        if(rn.le.fracquant) then
           mc_photon_destroyed = .true.
           return
        endif
     endif
  elseif(ray_inu.eq.-1) then
     !
     ! This is primary energy (e.g. viscous heating), and ray_inu.eq.-1 means
     ! that this energy is divided over the species by mass... This only
     ! works if dust_nr_size is always 1
     !
     ! NEWSTUFF 06.06.04
     !
     rhodusttot=0.d0
     do ispec=1,dust_nr_species
        rhodusttot = rhodusttot + dustdens(ispec,ray_index)
     enddo
     do ispec=1,dust_nr_species
        mc_enerpart(ispec) = energy * dustdens(ispec,ray_index) / rhodusttot
     enddo
  else
     write(stdo,*) 'INTERNAL ERROR: Dont know what to do with ray_inu = ',ray_inu
     stop
  endif
  !
  ! Stupidity check
  !
  if(debug_check_all.eq.1) then
     dum = 0.d0
     do ispec=1,dust_nr_species
        dum = dum + mc_enerpart(ispec)
     enddo
     if(abs(dum/energy-1.d0).gt.1.d-6) stop 34123
  endif
  !
  ! Determine if the dust temperature(s) and the frequency distribution
  ! for this cell must be recomputed
  !
  if(params%ifast.eq.0) then
     !
     ! For the original method, always recompute temperature and
     ! freq dist
     !
     calctemp = .true.
  else
     !
     ! For the new, fast method, check if the energy step is too high.
     !
     calctemp = .false.
     if(params%itempdecoup.eq.1) then
        !
        ! If temperatures are decoupled...
        !
        do ispec=1,dust_nr_species
           if((dust_quantum(ispec).eq.0).or.(iqactive.le.0)) then 
              if(abs((mc_cumulener(ispec,ray_index)+mc_enerpart(ispec))/      &
                      mc_cumulener_bk(ispec,ray_index)-1.d0)                  &
                      .gt.params%enthres) then
                 calctemp = .true.
              endif
           endif
        enddo
     else
        !
        ! If temperatures are coupled...
        !     
        dum1 = 0.d0
        dum2 = 0.d0
        dum  = 0.d0
        do ispec=1,dust_nr_species
           if((dust_quantum(ispec).eq.0).or.(iqactive.le.0)) then 
              dum1 = dum1 + mc_cumulener(ispec,ray_index)
              dum2 = dum2 + mc_enerpart(ispec)
              dum  = dum  + mc_cumulener_bk(ispec,ray_index)
           endif
        enddo
        if(abs((dum1+dum2)/dum-1.d0).gt.params%enthres) then
           calctemp = .true.
        endif
     endif
  endif
  !
  ! Determine the temperature from the local energy
  ! (this need not be hugely accurate)
  !
  if(calctemp) then
     if(params%itempdecoup.eq.1) then
        !
        ! If temperatures are decoupled...
        !
        do ispec=1,dust_nr_species
           if((dust_quantum(ispec).eq.0).or.(iqactive.le.0)) then 
              cumen = mc_cumulener(ispec,ray_index) /                      &
                      ( dustdens(ispec,ray_index) * cellvolume(ray_index) )
              dusttemp(ispec,ray_index) = compute_dusttemp_energy_bd(cumen,ispec)
           endif
        enddo
     else
        !
        ! If temperatures are coupled...
        !
        ! NOTE: Quantum-heated grains are included in the
        !       temperature determination below, even though
        !       the present photon has not been absorbed by one
        !       of them. This is because all grain temperatures
        !       must remain coupled.
        !
        do ispec=1,dust_nr_species
           tempav = compute_dusttemp_coupled_bd(dust_nr_species,       &
                           mc_cumulener(1,ray_index),                  &
                           dustdens(1,ray_index),cellvolume(ray_index))
           dusttemp(ispec,ray_index) = tempav
        enddo
     endif
     !
     ! Do some testing, if required
     !
!     if(debug_check_all.eq.1) then
!        do ispec=1,dust_nr_species
!           if(number_invalid(dusttemp(ispec,ray_index)).ne.0) then
!              if(number_invalid(dusttemp(ispec,ray_index)).eq.1) then
!                 write(stdo,*) 'ERROR: Discovered INF dust temperature at ', &
!                      'ispec=',ispec,' index=',ray_index
!                 stop
!              else
!                 write(stdo,*) 'ERROR: Discovered NAN dust temperature at ', &
!                      'ispec=',ispec,' index=',ray_index
!                 stop
!              endif
!           endif
!        enddo
!     endif
  endif
  !
  ! Now pick a new frequency, according to dB_nu(T)/dT
  !
  ! NOTE: This must be mc_enerpart and not mc_cumulener, because
  !       suppose we have two dust species, with spec 2 having 0 opacity
  !       in the optical. If an optical photon is absorbed, it must
  !       have been by spec 1, and thus also spec 1 must re-emit!
  !
  call pick_randomfreq_db(dust_nr_species,dusttemp(1,ray_index), &
                          mc_enerpart,ray_inu)
  !
  ! If the temperature(s) and the freqdist have been recomputed,
  ! then we must reset the backup cumulative energy to the present
  ! one
  !
  if(calctemp) then
     do ispec=1,dust_nr_species
        mc_cumulener_bk(ispec,ray_index) = mc_cumulener(ispec,ray_index)
     enddo
  endif
  !
  ! Now reset the iqactive, since from now on all the events for
  ! this photon package are thermal (i.e. no quantum-heating
  ! events anymore). ONLY IN ORIGINAL PAH METHOD. NEW QUANTUM: 28.04.06
  !
  if(iqactive.eq.1) then 
     iqactive = 0
  endif
  !
  ! Done...
  !
end subroutine do_absorption_event



!--------------------------------------------------------------------------
!                          Random direction
!--------------------------------------------------------------------------
subroutine montecarlo_randomdir(dirx,diry,dirz)
  implicit none
  doubleprecision :: dirx,diry,dirz,linv,l2
  !
  ! Do a loop: only accept directions that are within unit circle
  !
  l2 = 2.d0
  do while(l2.gt.1.d0)
     dirx = 2.d0*ran2(iseed)-1.d0
     diry = 2.d0*ran2(iseed)-1.d0
     dirz = 2.d0*ran2(iseed)-1.d0
     l2   = dirx*dirx + diry*diry + dirz*dirz
     if(l2.lt.1.d-4) l2=2.0
  enddo
  linv = 1.d0 / sqrt(l2)
  !
  ! Now normalize to unity
  !
  dirx = dirx * linv
  diry = diry * linv
  dirz = dirz * linv
  !
end subroutine montecarlo_randomdir



!--------------------------------------------------------------------------
!                           Rotate vector 
!
! Rotate a vector such that the unit vector (1,0,0) is rotated in the
! direction of (dirx,diry,dirz). Note that this does not uniquely specify
! the rotation of the vector, as it is still possible to start with a
! rotation around the x-axis. But as long as that rotation is anyway
! randomized, this is not a problem.
! **** I checked this on 26.04.09, but perhaps check it again some time ***
! --------------------------------------------------------------------------
subroutine montecarlo_rotatevec(vecx,vecy,vecz,dirx,diry,dirz)
  implicit none
  doubleprecision :: vecx,vecy,vecz,dirx,diry,dirz,l
  doubleprecision :: vx,vy,vz,dx,dy
  !
  ! First a rotation around the y-axis
  !
  l    = sqrt(dirx**2+diry**2)
  vx   = l    * vecx - dirz * vecz
  vy   = vecy
  vz   = dirz * vecx + l   * vecz
  !
  ! Then a rotation around the z-axis
  !
  if(l.gt.1d-10) then
     dx   = dirx / l
     dy   = diry / l
     vecx = dx * vx  - dy * vy
     vecy = dy * vx  + dx * vy
     vecz = vz
  else
     vecx = vx
     vecy = vy
     vecz = vz
  endif
  !
end subroutine montecarlo_rotatevec



!--------------------------------------------------------------------------
!                    Find random position in a cell
!--------------------------------------------------------------------------
subroutine montecarlo_random_pos_in_cell(params,icell)
  implicit none
  type(mc_params) :: params
  integer :: icell,iddr
  double precision :: rn, dum,r,theta,phi,cost,sint,cosp,sinp
  doubleprecision :: axi(1:2,1:3)
  integer :: ix,iy,iz
  !
  if(igrid_type.lt.100) then
     !
     ! The rectangular AMR grid
     !
     ! Get the cell walls
     !
     if(amr_tree_present) then
        do iddr=1,3
           axi(1,iddr) = amr_finegrid_xi(amr_theleafs(icell)%link%ixyzf(iddr),iddr,amr_theleafs(icell)%link%level)
           axi(2,iddr) = amr_finegrid_xi(amr_theleafs(icell)%link%ixyzf(iddr)+1,iddr,amr_theleafs(icell)%link%level)
        enddo
     else
        call amr_regular_get_ixyz(icell,ix,iy,iz)
        axi(1,1) = amr_finegrid_xi(ix,1,0)
        axi(2,1) = amr_finegrid_xi(ix+1,1,0)
        axi(1,2) = amr_finegrid_xi(iy,2,0)
        axi(2,2) = amr_finegrid_xi(iy+1,2,0)
        axi(1,3) = amr_finegrid_xi(iz,3,0)
        axi(2,3) = amr_finegrid_xi(iz+1,3,0)
     endif
     !
     ! Now do thing different for different coordinate systems
     !
     if(igrid_coord.lt.100) then
        !
        ! Cartesian coordinates
        !
        rn         = ran2(iseed)
        dum        = axi(2,1) - axi(1,1)
        ray_cart_x = axi(1,1) + dum * rn
        rn         = ran2(iseed)
        dum        = axi(2,2) - axi(1,2)
        ray_cart_y = axi(1,2) + dum * rn
        rn         = ran2(iseed)
        dum        = axi(2,3) - axi(1,3)
        ray_cart_z = axi(1,3) + dum * rn
     elseif(igrid_coord.lt.200) then
        !
        ! Spherical coordinates
        ! 
        ! NOTE: We are going to simplify this here now. In reality one
        !       should account for the curved geometry of the cell. But
        !       that would be costly to compute and for small enough
        !       cells this is not too important.
        !
        rn         = ran2(iseed)
        dum        = axi(2,1) - axi(1,1)
        r          = axi(1,1) + dum * rn
        rn         = ran2(iseed)
        dum        = axi(2,2) - axi(1,2)
        theta      = axi(1,2) + dum * rn
        rn         = ran2(iseed)
        dum        = axi(2,3) - axi(1,3)
        phi        = axi(1,3) + dum * rn
        cost       = cos(theta)
        sint       = sin(theta)
        cosp       = cos(phi)
        sinp       = sin(phi)
        ray_cart_x = r*sint*cosp
        ray_cart_y = r*sint*sinp
        ray_cart_z = r*cost
     else
        stop 4955
     endif
     !
  elseif(igrid_type.eq.101) then
     !
     ! The Delaunay triangulated grid
     !
     write(stdo,*) 'NOT YET READY: Delaunay unstructured grid'
     stop
  elseif(igrid_type.eq.201) then
     !
     ! The Voronoi grid
     !
     ! Here we assume that the photon is emitted from the center of the
     ! cell (**** APPROXIMATION ****), because for this gridding mode the
     ! cell shape is too complex to easily find a correct random position.
     ! 
     ! **** TO BE IMPROVED ****
     !
     write(stdo,*) 'NOT YET READY: Voronoi unstructured grid'
     stop
  else
     write(stdo,*) 'ERROR: Grid type ',igrid_type,' not known.'
     stop
  endif
end subroutine montecarlo_random_pos_in_cell


!--------------------------------------------------------------------------
!              HENYEY-GREENSTEIN RANDOM SCATTERING
!
!     This routine gives the random mu_scat and phi_scat according
!     to the small-angle-scattering formula of Henyey-Greenstein.
!
!                1         1 - g^2
!        P(mu) = - -------------------------
!                2 (1 + g^2 - 2 g mu )^(3/2)
!
!     We do this by introducing a variable xi
!
!                1 1 - g  /        1 + g                  \
!        xi   := - -----  | ------------------------- - 1 |
!                2   g    \ (1 + g^2 - 2 g mu )^(1/2)     /
!
!--------------------------------------------------------------------------
subroutine henyeygreenstein_randomdir(g,dirx,diry,dirz)
  implicit none
  doubleprecision g,dirx,diry,dirz,xi,g2,l2,linv,phi
  !
  if(g.ne.0.d0) then
     g2   = g*g
     xi   = ran2(iseed)
     dirx = (0.5d0/g)*(1.+g2-((1.-g2)/(1.-g+2*g*xi))**2)
  else
     dirx = 2.d0*ran2(iseed)-1.d0
  endif
  phi  = 2*pi*ran2(iseed)-0.5*pi
  l2   = 2.d0
  do while(l2.gt.1.d0)
     diry = 2.d0*ran2(iseed)-1.d0
     dirz = 2.d0*ran2(iseed)-1.d0
     l2   = diry*diry + dirz*dirz
     if(l2.lt.1.d-4) l2=2.0
  enddo
  linv = sqrt((1.d0-dirx**2)/l2)
  !
  ! Now normalize to sqrt(1-mu^2)  where mu = dirx
  !
  diry = diry * linv
  dirz = dirz * linv
  !
  ! Safety check
  !
  if(debug_check_all.eq.1) then
     if(abs(dirx**2+diry**2+dirz**2-1.d0).gt.1d-14) then
        write(stdo,*) 'INTERNAL ERROR: Henyey-Greenstein direction vector not OK.'
        stop
     endif
  endif
  !
end subroutine henyeygreenstein_randomdir


!--------------------------------------------------------------------------
!                    HENYEY GREENSTEIN PHASE FUNCTION
!--------------------------------------------------------------------------
function henyeygreenstein_phasefunc(g,costheta)
  implicit none
  double precision :: g,g2,costheta,henyeygreenstein_phasefunc
  g2 = g*g
  henyeygreenstein_phasefunc = 0.5d0 * (1.d0-g2)/(1.d0-2*g*costheta+g2)**1.5
  return
end function henyeygreenstein_phasefunc



!--------------------------------------------------------------------------
!                  WRITE MEAN INTENSITY TO FILE
!--------------------------------------------------------------------------
subroutine write_meanint_to_file()
  implicit none
  integer :: icell,index,inu,reclend,irec,i,ierr
  integer(kind=8) :: nn,kk
  logical :: fex
  double precision, allocatable :: data(:)
  !
  ! Now write the dust temperature
  !
  if(igrid_type.lt.100) then
     !
     ! Regular (AMR) grid
     ! 
     ! Just make sure that the cell list is complete
     !
     if(amr_tree_present) then
        call amr_compute_list_all()
     endif
     !
     ! Do a stupidity check
     !
     if(nrcells.ne.amr_nrleafs) stop 3209
     !
     ! Open file and write the mean intensity to it
     !
     if(.not.rto_unformatted) then 
        !
        ! Write the mean intensity in ascii form
        !
        open(unit=1,file='mean_intensity.out')
        write(1,*) 1                                   ! Format number
        write(1,*) nrcellsinp
        write(1,*) mc_nrfreq
        do inu=1,mc_nrfreq
           call amr_resetcount()
           call amr_nextcell(index)
           do while(index.ge.0)
              if(index.gt.0) then
                 write(1,*) mcscat_meanint(inu,index)
              else
                 write(1,*) '0'
              endif
              call amr_nextcell(index)
           enddo
        enddo
        close(1)
     else
        !
        ! Write the mean intensity in f77-style unformatted form,
        ! using a record length given by rto_reclen
        !
        reclend = rto_reclen/8
        if(allocated(data)) deallocate(data)
        allocate(data(1:reclend),STAT=ierr)
        if(ierr.ne.0) then
           write(stdo,*) 'ERROR: Could not allocate data array'
           stop
        endif
        open(unit=1,file='mean_intensity.uout',form='unformatted')
        nn = 1
        kk = rto_reclen
        write(1) nn,kk               ! Format number and record length
        nn = nrcellsinp
        kk = mc_nrfreq
        write(1) nn,kk
        do inu=1,mc_nrfreq
           call amr_resetcount()
           call amr_nextcell(index)
           do irec=1,(nrcellsinp-1)/reclend+1
              i = 1
              data(:) = 0.d0
              do while((index.ge.0).and.(i.le.reclend))
                 if(index.gt.0) then
                    data(i) = mcscat_meanint(inu,index)
                 else
                    data(i) = 0.d0
                 endif
                 i       = i + 1
                 call amr_nextcell(index)
              enddo
              write(1) data
           enddo
        enddo
        deallocate(data)
     endif
  else
     !
     ! Other grids not yet implemented
     !
     write(stdo,*) 'ERROR: Only regular and AMR grids implemented'
     stop
  endif
  !
  ! If the grid is internally made, then we must make sure that
  ! the grid has been written to file, otherwise the output file
  ! created here makes no sense.
  !
  if((.not.grid_was_read_from_file).and.(.not.grid_was_written_to_file)) then
     call write_grid_file()
     grid_was_written_to_file = .true.     ! Avoid multiple writings
  endif
end subroutine write_meanint_to_file



!===========================================================================
!              ROUTINES FOR STORING TEMPERATURE ARRAYS
!===========================================================================


!-------------------------------------------------------------------
!               HELPER FUNCTION FOR THE ABSORPTION EVENT
!
!     This function returns 0 for that temperature where the new
!     spectrum of the cell equals the old one plus the energy of
!     the photonj package.
!-------------------------------------------------------------------
function absevfunc(temp)
  implicit none
  doubleprecision absevfunc,temp
  !
  integer inu
  doubleprecision demis
  !
  demis = 0.d0
  do inu=1,freq_nr
     fnu_diff(inu) = cellalpha(inu) * bplanck(temp,freq_nu(inu))
     demis         = demis + fnu_diff(inu) * freq_dnu(inu)
  enddo
  demis = demis * fourpi 
  absevfunc = demis 
  return
end function absevfunc


!----------------------------------------------------------------------
!             COMPUTE THE ENERGY BELONGING TO A TEMPERATURE
!
!     This routine computes the energy output per gram of dust
!     of a given species and size for an array of temperatures.
!----------------------------------------------------------------------
subroutine make_emiss_dbase(ntemp,temp0,temp1)
  implicit none
  integer :: ntemp
  double precision :: temp0,temp1
  !
  integer :: itemp,ispec,ierr,inu
  !
  ! Check
  !
  if((freq_nr.lt.1).or.(dust_nr_species.lt.1)) then
     write(stdo,*) 'INTERNAL ERROR IN MONTECARLO: Could not make temp grid'
     stop
  endif
  !
  ! Allocate arrays
  !
  allocate(db_temp(ntemp),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate db_temp'
     stop 
  endif
  allocate(db_enertemp(ntemp,dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate db_temp'
     stop 
  endif
  allocate(db_logenertemp(ntemp,dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate db_temp'
     stop 
  endif
  allocate(db_emiss(freq_nr,ntemp,dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate db_emiss'
     stop 
  endif
  allocate(db_cumulnorm(freq_nr+1,ntemp,dust_nr_species),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate db_cumulnorm'
     stop 
  endif
  allocate(db_cumul(freq_nr+1),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate db_cumul'
     stop 
  endif
  !
  ! Allocate array used internally for picking random frequency
  !
  allocate(enercum(dust_nr_species+1),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate enercum'
     stop 
  endif
  !
  ! Allocate temporary arrays
  !
  allocate(cellalpha(freq_nr),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate cellalpha'
     stop 
  endif
  allocate(fnu_diff(freq_nr),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate fnu_diff'
     stop 
  endif
  allocate(diffemis(freq_nr),STAT=ierr)
  if(ierr.ne.0) then
     write(stdo,*) 'ERROR in Montecarlo Module: Could not allocate diffemis'
     stop 
  endif
  !
  ! Make temperature grid
  !
  db_ntemp = ntemp
  do itemp=1,ntemp
     db_temp(itemp) = temp0 * (temp1/temp0)**((itemp-1.d0)/(ntemp-1.d0))
  enddo
  !     
  ! A loop over all dust species
  !
  do ispec=1,dust_nr_species
     !
     ! Compute the emissivity and the total emitted energy
     ! per second per gram dust of this species/size.
     !
     do inu=1,freq_nr
        cellalpha(inu) = kappa_a(inu,ispec)
     enddo
     do itemp=1,db_ntemp
        db_enertemp(itemp,ispec) = absevfunc(db_temp(itemp))
        db_logenertemp(itemp,ispec) = log(db_enertemp(itemp,ispec))
        do inu=1,freq_nr
           db_emiss(inu,itemp,ispec) = fnu_diff(inu)
        enddo
     enddo
     !
     ! Now compute the derivative of the emissivity
     ! with respect to energy, so that we can obtain
     ! a right distribution function for picking a
     ! new photon frequency. We immediately make
     ! a cumulative version of this for each
     ! species, so that the picking of a new frequency
     ! for a photon goes in two steps:
     !  1) Pick a species/size from the energies
     !  2) Pick, within this selected species, a freq
     !
     ! NOTE!!!! VERY IMPORTANT!!!: 
     !   This is done slightly differently from the
     !   original method in which the exact difference
     !   of fnu and fnu_old was taken. But this is not
     !   too impportant, because 1) the normalization
     !   if irrelevant and 2) for sufficient photon
     !   statistics the derivative is close enough to
     !   the difference that it is OK.
     !
     ! For the first temperature we simply take the
     ! emissivity function itself
     !
     itemp = 1
     do inu=1,freq_nr
        diffemis(inu) = db_emiss(inu,itemp,ispec)
     enddo
     db_cumul(1) = 0.d0
     do inu=1,freq_nr
        db_cumul(inu+1) = db_cumul(inu) + diffemis(inu) * freq_dnu(inu)
     enddo
     do inu=1,freq_nr+1
        db_cumulnorm(inu,itemp,ispec) = db_cumul(inu) / db_cumul(freq_nr+1)
     enddo
     !
     ! For the rest take indeed the difference
     !              
     do itemp=2,db_ntemp
        do inu=1,freq_nr
           diffemis(inu) = db_emiss(inu,itemp,ispec) -                      &
                           db_emiss(inu,itemp-1,ispec) 
        enddo
        db_cumul(1) = 0.d0
        do inu=1,freq_nr
           db_cumul(inu+1) = db_cumul(inu) + diffemis(inu) * freq_dnu(inu)
        enddo
        do inu=1,freq_nr+1
           db_cumulnorm(inu,itemp,ispec) = db_cumul(inu) / db_cumul(freq_nr+1)
        enddo
     enddo
  enddo
  !
  ! Free temporary arrays
  !
  deallocate(cellalpha,fnu_diff,diffemis)
  !
end subroutine make_emiss_dbase


!----------------------------------------------------------------------
!                  FREE TEMPERATURE DATABASE
!----------------------------------------------------------------------
subroutine free_emiss_dbase()
  implicit none
  db_ntemp=0
  if(allocated(db_temp)) deallocate(db_temp)
  if(allocated(db_cumulnorm)) deallocate(db_cumulnorm)
  if(allocated(db_cumul)) deallocate(db_cumul)
  if(allocated(db_enertemp)) deallocate(db_enertemp)
  if(allocated(db_logenertemp)) deallocate(db_logenertemp)
  if(allocated(db_emiss)) deallocate(db_emiss)
  if(allocated(enercum)) deallocate(enercum)
end subroutine free_emiss_dbase



!----------------------------------------------------------------------
!               COMPUTE THE TEMPERATURE FOR A GIVEN ENERGY
!
!     This routine computes the dust temperature for a given 
!     energy-per-gram-of-dust for this species.
!----------------------------------------------------------------------
function compute_dusttemp_energy_bd(ener,ispec)
  implicit none
  !
  integer :: ispec
  doubleprecision :: ener,compute_dusttemp_energy_bd,logener
  !
  integer :: itemp
  doubleprecision :: eps,temp
  !
  ! First find the energy in the grid
  !
  ! Linear method:
  !
  !!!!!! call hunt(db_enertemp(1,ispec),db_ntemp,ener,itemp)
  !
  ! Logarithmic method:
  !
  logener = log(ener)
  call hunt(db_logenertemp(1,ispec),db_ntemp,logener,itemp)
  !
  ! Check if we are in range
  !
  if(itemp.ge.db_ntemp) then
     write(stdo,*) 'ERROR: Too high temperature discovered' 
     stop 76823
  endif
  if(itemp.le.0) then
     !
     ! Temperature presumably below lowest temp in dbase
     !
     eps = ener/db_enertemp(1,ispec)
     if(eps.gt.1.d0) stop 9911
     temp = eps*db_temp(1)
  else
     !
     ! Temperature in the right range: default action
     !
     eps = (ener-db_enertemp(itemp,ispec)) /                            &
           (db_enertemp(itemp+1,ispec) - db_enertemp(itemp,ispec))
     if((eps.gt.1.d0).or.(eps.lt.0.d0)) stop 9912
     temp = (1.d0-eps)*db_temp(itemp) + eps*db_temp(itemp+1)
  endif
  !
  ! Return
  !
  compute_dusttemp_energy_bd = temp
  return
  !
end function compute_dusttemp_energy_bd


!----------------------------------------------------------------------
!               COMPUTE THE TEMPERATURE FOR A GIVEN ENERGY
!
!     This routine computes the dust temperature for a given 
!     energy-per-gram-of-dust for this species.
!----------------------------------------------------------------------
function compute_dusttemp_coupled_bd(nspec,energy,rho,vol)
  implicit none
  !
  integer :: nspec
  doubleprecision :: vol
  doubleprecision :: energy(nspec)
  doubleprecision :: rho(nspec)
  doubleprecision :: compute_dusttemp_coupled_bd
  !
  integer :: itemp,ispec
  doubleprecision :: eps,temp,entot,dummy1,dummy2
  doubleprecision :: en1,en2
  !
  ! Compute the total energy per gram dust
  !
  entot = 0.d0
  do ispec=1,dust_nr_species
     entot  = entot  + energy(ispec)
  enddo
  entot = entot / vol
  !
  ! First find the energy in the grid
  !
  ! NOTE: It is difficult to use the logarithmic energy search as we
  !       can easily do for the decoupled temperatures, because we do
  !       not yet know the composition of the dust globally. One could
  !       adapt hunt_temp but that would require many evaluations of
  !       a log, and that might kill all the advantage of the log
  !       search. So we will keep this linear.
  !
  call hunt_temp(rho,entot,itemp)
  !
  ! Check if we are in range
  !
  if(itemp.ge.db_ntemp) then
     write(stdo,*) 'ERROR: Too high temperature discovered' 
     stop 76823
  endif
  if(itemp.le.0) then
     !
     ! Temperature presumably below lowest temp in dbase
     !
     en2 = 0.d0
     do ispec=1,dust_nr_species
        en2 = en2 + rho(ispec) * db_enertemp(1,ispec)
     enddo
     eps = entot/en2
     if(eps.gt.1.d0) stop 59911
     temp = eps*db_temp(1)
  else
     !
     ! Temperature in the right range: default action
     !
     en1 = 0.d0
     do ispec=1,dust_nr_species
        en1 = en1 + rho(ispec)*db_enertemp(itemp,ispec)
     enddo
     en2 = 0.d0
     do ispec=1,dust_nr_species
        en2 = en2 + rho(ispec)*db_enertemp(itemp+1,ispec)
     enddo
     eps = (entot-en1) / (en2-en1)
     if((eps.gt.1.d0).or.(eps.lt.0.d0)) stop 99124
     temp = (1.d0-eps)*db_temp(itemp) + eps*db_temp(itemp+1)
  endif
  !
  ! Return
  !
  compute_dusttemp_coupled_bd = temp
  return
  !
end function compute_dusttemp_coupled_bd


!----------------------------------------------------------------------
!               COMPUTE NEW IFREQ FROM LOCAL SITUATION
!----------------------------------------------------------------------
subroutine pick_randomfreq_db(nspec,temp,mc_enerpart,inupick)
  implicit none
  !
  integer :: nspec,inupick
  doubleprecision :: temp(nspec)
  doubleprecision :: mc_enerpart(nspec)
  !
  doubleprecision rn,eps
  integer inu,ispec,itemp
  !
  ! Figure out which species to take. This can 
  ! be determined from the energies
  !
  enercum(1) = 0.d0
  do ispec=1,dust_nr_species
     enercum(ispec+1) = enercum(ispec) + mc_enerpart(ispec)
  enddo
  if(dust_nr_species.gt.1) then 
     rn = ran2(iseed)*enercum(dust_nr_species+1)
     call hunt(enercum,dust_nr_species,rn,ispec)
     if((ispec.lt.1).or.(ispec.gt.dust_nr_species)) stop 50209
  else
     ispec = 1
  endif
  !
  ! Find the temperature in the database
  !
  call hunt(db_temp,db_ntemp,temp(ispec),itemp)
  !
  ! Check if we are in range
  !
  if(itemp.ge.db_ntemp) then
     write(stdo,*) 'ERROR: Too high temperature discovered' 
     stop 77823
  endif
  if(itemp.le.0) then
     !
     ! Temperature presumably below lowest temp in dbase
     !
     itemp = 1
     eps   = 0.d0
  else
     !
     ! Temperature in the right range: default action
     !
     eps = (temp(ispec)-db_temp(itemp)) /                             &
           (db_temp(itemp+1)-db_temp(itemp))
     if((eps.gt.1.d0).or.(eps.lt.0.d0)) stop 94941
  endif
  !
  ! Now, within this species/size, find the frequency
  !
  if(intplt.eq.1) then
     !
     ! Do this properly
     !
     do inu=1,freq_nr+1
        db_cumul(inu) = (1.d0-eps)*db_cumulnorm(inu,itemp,ispec) +   &
                        eps*db_cumulnorm(inu,itemp+1,ispec)
     enddo
     if(abs(db_cumul(freq_nr+1)-1.d0).gt.1d-11) stop 43465
     rn = ran2(iseed)
     call hunt(db_cumul,freq_nr,rn,inupick)
     if((inupick.lt.1).or.(inupick.gt.freq_nr)) stop 8189
  else
     !
     ! Do this approximatively
     !
     if(eps.gt.0.5d0) then
        if(itemp.lt.db_ntemp-1) itemp=itemp+1
     endif
     rn = ran2(iseed)
     call hunt(db_cumulnorm(1,itemp,ispec),freq_nr,rn,inupick)
  endif
  !
end subroutine pick_randomfreq_db




!------------------------------------------------------------------
!              HUNT IN TOTAL ENERGY FOR TEMPERATURE
!     (MODIFIED NRECIP ROUTINE)
!        energy = total energy divided by volume
!------------------------------------------------------------------
subroutine hunt_temp(rho,energy,jlo)
  integer :: jlo,n
  doubleprecision :: energy,xx,x
  doubleprecision :: rho(dust_nr_species)
  integer :: inc,jhi,jm
  logical :: ascnd
  !
  n = db_ntemp
  x = energy
  ascnd=.true.
  if(jlo.le.0.or.jlo.gt.n)then
     jlo=0
     jhi=n+1
     goto 3
  endif
  inc=1
  xx=0.d0
  do ispec=1,dust_nr_species
     xx = xx + rho(ispec) * db_enertemp(jlo,ispec)
  enddo
  if(x.ge.xx.eqv.ascnd)then
1    jhi=jlo+inc
     xx=0.d0
     do ispec=1,dust_nr_species
        xx = xx + rho(ispec) * db_enertemp(jhi,ispec)
     enddo
     if(jhi.gt.n)then
        jhi=n+1
     else if(x.ge.xx.eqv.ascnd)then
        jlo=jhi
        inc=inc+inc
        goto 1
     endif
  else
     jhi=jlo
2    jlo=jhi-inc
     xx=0.d0
     do ispec=1,dust_nr_species
        xx = xx + rho(ispec) * db_enertemp(jlo,ispec)
     enddo
     if(jlo.lt.1)then
        jlo=0
     else if(x.lt.xx.eqv.ascnd)then
        jhi=jlo
        inc=inc+inc
        goto 2
     endif
  endif
3 if(jhi-jlo.eq.1)return
  jm=(jhi+jlo)/2
  xx=0.d0
  do ispec=1,dust_nr_species
     xx = xx + rho(ispec) * db_enertemp(jm,ispec)
  enddo
  if(x.gt.xx.eqv.ascnd)then
     jlo=jm
  else
     jhi=jm
  endif
  goto 3
end subroutine hunt_temp
!  (C) Copr. 1986-92 Numerical Recipes Software =v1.9"217..


end module montecarlo_module
